@@ -1,4 +1,4 @@
-/* $Id: about.cpp 11647 2006-05-26 01:03:08Z rusty $ */
+/* $Id: about.cpp 13056 2006-08-09 15:57:51Z zookeeper $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -29,14 +29,9 @@ namespace about
 
 std::vector<std::string> get_text(std::string campaign) {
 	static const char *credits[] = {
-		"_" N_("+Core Developers"),
-		"_" N_("+  Main Developer"),
-		"-  David White (Sirp)",
-		"- ",
-		"_" N_("+  Artwork and graphics designer"),
-		"-   Francisco Muñoz (fmunoz)",
-		"- ",
 		"_" N_("+Developers"),
+		"-   David White (Sirp)",
+		"-   *",
 		"-   Alfredo Beaumont (ziberpunk)",
 		"-   András Salamon (ott)",
 		"-   Benoît Timbert (Noyga)",
