@@ -1,6 +1,6 @@
-/* $Id: about.hpp,v 1.16 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: about.hpp 10087 2006-02-13 04:14:27Z darthfool $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,6 +14,8 @@
 #ifndef ABOUT_H_INCLUDED
 #define ABOUT_H_INCLUDED
 
+#include "global.hpp"
+#include "config.hpp"
 class display;
 
 #include <vector>
@@ -22,8 +24,11 @@ class display;
 namespace about
 {
 
-void show_about(display &disp);
-std::vector<std::string> get_text();
+extern config about_list;
+
+void show_about(display &disp, std::string campaign="");
+void set_about(const config& cfg);
+std::vector<std::string> get_text(std::string campaign="");
 
 }
 
