@@ -1,4 +1,4 @@
-/* $Id: actions.cpp 12598 2006-07-17 06:58:57Z isaac $ */
+/* $Id: actions.cpp 14748 2006-12-19 18:45:01Z noyga $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -164,7 +164,7 @@ std::string recruit_unit(const gamemap& map, int side,
 				disp->invalidate(recruit_location);
 				disp->draw();
 				events::pump();
-				disp->non_turbo_delay();
+				disp->delay(10);
 			}
 			un->second.set_standing(*disp,recruit_location);
 		}
@@ -226,13 +226,20 @@ gamemap::location under_leadership(const unit_map& units,
 battle_context::battle_context(const gamemap& map, const std::vector<team>& teams, const unit_map& units,
 							   const gamestatus& status, const game_data& gamedata,
 							   const gamemap::location& attacker_loc, const gamemap::location& defender_loc,
-							   int attacker_weapon, int defender_weapon, double harm_weight, const combatant *prev_def)
+							   int attacker_weapon, int defender_weapon, double aggression, const combatant *prev_def)
 	: attacker_stats_(NULL), defender_stats_(NULL), attacker_combatant_(NULL), defender_combatant_(NULL)
 {
 	const unit& attacker = units.find(attacker_loc)->second;
 	const unit& defender = units.find(defender_loc)->second;
+	const double harm_weight = 1.0 - aggression;
 
-	if (attacker_weapon == -1 && attacker.attacks().size() == 1)
+	// A Python AI can send an invalid weapon and crash Wesnoth
+	// Haven't found a way for the Python API to prevent this 
+	// problem. So instead of segfaulting it sends an assertion
+	// failure.
+	wassert(attacker_weapon< (int)attacker.attacks().size());
+		
+	if (attacker_weapon == -1 && attacker.attacks().size() == 1 && attacker.attacks()[0].attack_weight() > 0 )
 		attacker_weapon = 0;
 
 	if (attacker_weapon == -1) {
@@ -247,17 +254,22 @@ battle_context::battle_context(const gamemap& map, const std::vector<team>& team
 
 	// If those didn't have to generate statistics, do so now.
 	if (!attacker_stats_) {
-		const attack_type *def = NULL;
+		const attack_type *adef = NULL;
+		const attack_type *ddef = NULL;
+		if (attacker_weapon >= 0) {
+			wassert(attacker_weapon < (int)attacker.attacks().size());
+			adef = &attacker.attacks()[attacker_weapon];
+		}
 		if (defender_weapon >= 0) {
 			wassert(defender_weapon < (int)defender.attacks().size());
-			def = &defender.attacks()[defender_weapon];
+			ddef = &defender.attacks()[defender_weapon];
 		}
 		wassert(!defender_stats_ && !attacker_combatant_ && !defender_combatant_);
 		attacker_stats_ = new unit_stats(attacker, attacker_loc, attacker_weapon,
-										 true, defender, defender_loc, def,
+										 true, defender, defender_loc, ddef,
 										units, teams, status, map, gamedata);
 		defender_stats_ = new unit_stats(defender, defender_loc, defender_weapon, false,
-										 attacker, attacker_loc, &attacker.attacks()[attacker_weapon],
+										 attacker, attacker_loc, adef,
 										 units, teams, status, map, gamedata);
 	}
 }
@@ -276,6 +288,14 @@ battle_context::battle_context(const unit_stats &att, const unit_stats &def)
 	defender_combatant_ = NULL;
 }
 
+battle_context::~battle_context()
+{
+	delete attacker_stats_;
+	delete defender_stats_; 
+	delete attacker_combatant_;
+	delete defender_combatant_; 
+}
+
 battle_context& battle_context::operator=(const battle_context &other)
 {
 	if (&other != this) {
@@ -360,11 +380,13 @@ unsigned battle_context::choose_attacker_weapon(const unit &attacker, const unit
 	unsigned int i;
 	for (i = 0; i < attacker.attacks().size(); i++) {
 		const attack_type &att = attacker.attacks()[i];
-		if (att.defense_weight() > 0) {
+		if (att.attack_weight() > 0) {
 			choices.push_back(i);
 		}
 	}
-	wassert(choices.size() > 0);
+	// no weapon !
+	if (choices.size() == 0)
+		return -1;
 	if (choices.size() == 1) {
 		*defender_weapon = choose_defender_weapon(attacker, defender, choices[0], map, teams, units,
 												  status, gamedata, attacker_loc, defender_loc, prev_def);
@@ -442,8 +464,8 @@ bool battle_context::better_combat(const combatant &us_a, const combatant &them_
 		return true;
 
 	// Compare: damage to them - damage to us (average_hp replaces -damage)
-	a = us_a.average_hp() - them_a.average_hp();
-	b = us_b.average_hp() - them_b.average_hp();
+	a = us_a.average_hp()*harm_weight - them_a.average_hp();
+	b = us_b.average_hp()*harm_weight - them_b.average_hp();
 	if (a - b < -0.01)
 		return false;
 	if (a - b > 0.01)
@@ -648,7 +670,7 @@ void attack(display& gui, const gamemap& map,
             const gamestatus& state,
             const game_data& info,
 			bool update_display)
-{
+{	
 	//stop the user from issuing any commands while the units are fighting
 	const events::command_disabler disable_commands;
 
@@ -658,6 +680,14 @@ void attack(display& gui, const gamemap& map,
 	if(a == units.end() || d == units.end()) {
 		return;
 	}
+
+	// no attack weapon => stop here and don't attack
+	if (attack_with < 0) {
+		a->second.set_attacks(a->second.attacks_left()-1);
+		a->second.set_movement(-1);
+		return;
+	}
+
 	bool OOS_error = false;
 	std::stringstream errbuf;
 
@@ -885,7 +915,7 @@ void attack(display& gui, const gamemap& map,
 				LOG_NG << "firing attack_end event\n";
 				game_events::fire("attack_end",attacker,defender,dat);
 				LOG_NG << "firing die event\n";
-				game_events::fire("die",loc,a->first);
+				game_events::fire("die",loc,attacker_loc);
 				d = units.find(loc);
 				a = units.end();
 
@@ -1128,7 +1158,7 @@ void attack(display& gui, const gamemap& map,
 				LOG_NG << "firing attack_end event\n";
 				game_events::fire("attack_end",attacker,defender,dat);
 				LOG_NG << "firing die event\n";
-				game_events::fire("die",loc,d->first);
+				game_events::fire("die",loc,defender_loc);
 				a = units.find(loc);
 				d = units.end();
 
@@ -1728,6 +1758,9 @@ bool clear_shroud_unit(const gamemap& map,
 	//clear the location the unit is at
 	clear_shroud_loc(map,teams[team],loc,&cleared_locations);
 
+	 //remove all redundant location, if on this location is unit, sighed event is called twice
+	std::unique(cleared_locations.begin(),cleared_locations.end());
+	 
 	for(std::vector<gamemap::location>::const_iterator it =
 	    cleared_locations.begin(); it != cleared_locations.end(); ++it) {
 
@@ -1735,11 +1768,16 @@ bool clear_shroud_unit(const gamemap& map,
 		if(sighted != units.end() &&
 		  (sighted->second.invisible(*it,units,teams) == false
 		  || teams[team].is_enemy(sighted->second.side()) == false)) {
-			if(seen_units == NULL || known_units == NULL) {
-				static const std::string sighted("sighted");
-				game_events::raise(sighted,*it,loc);
-			} else if(known_units->count(*it) == 0 && !utils::string_bool(sighted->second.get_state("stoned"))) {
-				seen_units->insert(*it);
+			if(!(seen_units == NULL || known_units == NULL) && known_units->count(*it) == 0) {
+				if (!utils::string_bool(sighted->second.get_state("stoned")))
+				{
+					seen_units->insert(*it);
+				}
+				if ( teams[team].uses_shroud() || teams[team].uses_fog())
+				{
+					static const std::string sighted_str("sighted");
+					game_events::raise(sighted_str,*it,loc);
+				}
 			}
 		}
 	}
@@ -1817,7 +1855,6 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	const bool check_shroud = should_clear_shroud && team.auto_shroud_updates() &&
 		(team.uses_shroud() || team.uses_fog());
 
-	//if we use shroud/fog of war, count out the units we can currently see
 	std::set<gamemap::location> known_units;
 	if(check_shroud) {
 		for(unit_map::const_iterator u = units.begin(); u != units.end(); ++u) {
@@ -1853,8 +1890,9 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		}
 
 		//if we use fog or shroud, see if we have sighted an enemy unit, in
-		//which case we should stop immediately.
-		if(check_shroud) {
+		//which case we should stop immediately. Cannot use check shroud,
+		//because also need check if delay shroud is on.
+		if(should_clear_shroud && (team.uses_shroud() || team.uses_fog())) {
 			if(units.count(*step) == 0 && !map.is_village(*step)) {
 				LOG_NG << "checking for units from " << (step->x+1) << "," << (step->y+1) << "\n";
 
@@ -1864,10 +1902,16 @@ size_t move_unit(display* disp, const game_data& gamedata,
 				//we have to swap out any unit that is already in the hex, so we can put our
 				//unit there, then we'll swap back at the end.
 				const temporary_unit_placer unit_placer(units,*step,ui->second);
-
-				should_clear_stack |= clear_shroud_unit(map,status,gamedata,units,*step,teams,
-				                                        ui->second.side()-1,&known_units,&seen_units);
-
+				if( team.auto_shroud_updates())
+        {
+					should_clear_stack |= clear_shroud_unit(map,status,gamedata,units,*step,teams,
+					    ui->second.side()-1,&known_units,&seen_units);
+				}
+				else
+				{
+					clear_shroud_unit(map,status,gamedata,units,*step,teams,
+							ui->second.side()-1,&known_units,&seen_units);
+				}
 				if(should_clear_stack) {
 					disp->invalidate_all();
 				}
@@ -1930,10 +1974,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	//remove it until the move is done, so that while the unit is moving status etc will
 	//still display the correct number of units.
 	if(disp != NULL) {
-		ui->second.set_hidden(true);
-		disp->invalidate(ui->first);
 		unit_display::move_unit(*disp,map,steps,ui->second,units,teams);
-		ui->second.set_hidden(false);
 	}
 
 	ui->second.set_movement(moves_left);
