@@ -1,4 +1,4 @@
-/* $Id: ai.cpp 12626 2006-07-17 22:06:29Z xan $ */
+/* $Id: ai.cpp 14607 2006-12-11 05:33:08Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -35,7 +35,9 @@
 class idle_ai : public ai_interface {
 public:
 	idle_ai(info& i) : ai_interface(i) {}
-	void play_turn() {}
+	void play_turn() {
+		game_events::fire("ai turn");
+	}
 };
 
 class sample_ai : public ai_interface {
@@ -43,6 +45,7 @@ public:
 	sample_ai(info& i) : ai_interface(i) {}
 
 	void play_turn() {
+		game_events::fire("ai turn");
 		do_attacks();
 		get_villages();
 		do_moves();
@@ -148,13 +151,15 @@ protected:
 
 	void do_recruitment() {
 		const std::set<std::string>& options = current_team().recruits();
-        const int choice = (rand()%options.size());
-        std::set<std::string>::const_iterator i = options.begin();
-        std::advance(i,choice);
+        	if (!options.empty()) {
+			const int choice = (rand()%options.size());
+		        std::set<std::string>::const_iterator i = options.begin();
+		        std::advance(i,choice);
 
-		const bool res = recruit(*i);
-		if(res) {
-			do_recruitment();
+			const bool res = recruit(*i);
+			if(res) {
+				do_recruitment();
+			}
 		}
 	}
 };
@@ -352,7 +357,7 @@ void ai_interface::diagnostic(const std::string& msg)
 void ai_interface::log_message(const std::string& msg)
 {
 	if(game_config::debug) {
-		info_.disp.add_chat_message("ai",info_.team_num,msg,display::MESSAGE_PUBLIC);
+		info_.disp.add_chat_message("ai",info_.team_num,msg,display::MESSAGE_PUBLIC,false);
 	}
 }
 
@@ -467,8 +472,21 @@ gamemap::location ai_interface::move_unit_partial(location from, location to, st
 
 				info_.disp.scroll_to_tiles(from.x,from.y,to.x,to.y);
 
+				up = info_.units.extract(u_it->first);
+				bool hidden = up->second.get_hidden();
+				up->second.set_hidden(true);
+				info_.units.add(up);
+				info_.disp.invalidate(up->first);
+				info_.disp.draw();
+				u_it = info_.units.find(up->first); //get a new iterator, since it was invalidated
+				if(u_it == info_.units.end()) {
+					LOG_STREAM(err, ai) << "Could not find unit at " << up->first << '\n';
+					wassert(false);
+					return location();
+				}
 				up = info_.units.extract(u_it->first);
 				unit_display::move_unit(info_.disp,info_.map,steps,up->second,info_.units,info_.teams);
+				up->second.set_hidden(hidden);
 			}
 		}
 	}
@@ -968,7 +986,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 		//		const unit_map::const_iterator tgt = units_.find(target_loc);
 
 		const location arrived_at = move_unit(from,to,possible_moves);
-		if(arrived_at != to) {
+		if(arrived_at != to || units_.find(to) == units_.end()) {
 			LOG_STREAM(warn, ai) << "unit moving to attack has ended up unexpectedly at "
 			                     << arrived_at << " when moving to " << to << " moved from "
 			                     << from << '\n';
@@ -1334,7 +1352,10 @@ bool ai::retreat_units(std::map<gamemap::location,paths>& possible_moves, const
 
 				// If we can't move, we should be more aggressive in lashing out.
 				if (best_pos == i->first) {
-					return desperate_attack(i->first);
+					if (i->second.attacks_left()) {
+						return desperate_attack(i->first);
+					}
+					return false;
 				}
 
 				if(best_pos.valid()) {
@@ -1498,7 +1519,7 @@ void ai::analyze_potential_recruit_combat()
 			}
 
 			weighting += j->second.cost();
-			score += compare_unit_types(info->second,gameinfo_.unit_types.find(j->second.id())->second)*j->second.cost();
+			score += compare_unit_types(info->second,j->second.type())*j->second.cost();
 		}
 
 		if(weighting != 0) {
