@@ -1,4 +1,4 @@
-/* $Id: ai_interface.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: ai_interface.hpp 11936 2006-06-08 07:16:37Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -15,8 +15,9 @@
 
 class display;
 class gamemap;
+
+#include "generic_event.hpp"
 #include "pathfind.hpp"
-#include "unit.hpp"
 
 class ai_interface {
 public:
@@ -31,7 +32,7 @@ public:
 	///that an AI might need access to in order to make and implement its decisions
 	struct info {
 		info(display& disp, const gamemap& map, const game_data& gameinfo, unit_map& units,
-			std::vector<team>& teams, int team_num, const gamestatus& state, class turn_info& turn_data)
+			std::vector<team>& teams, unsigned int team_num, const gamestatus& state, class turn_info& turn_data)
 			: disp(disp), map(map), gameinfo(gameinfo), units(units), teams(teams),
 			  team_num(team_num), state(state), turn_data_(turn_data)
 		{}
@@ -53,7 +54,7 @@ public:
 
 		///the number of the team the AI is. Note that this number is 1-based, so it
 		///has to have 1 subtracted from it for it to be used as an index of 'teams'
-		int team_num;
+		unsigned int team_num;
 
 		///information about what turn it is, and what time of day
 		const gamestatus& state;
@@ -65,7 +66,9 @@ public:
 
 	///the constructor. All derived classes should take an argument of type info& which
 	///they should pass to this constructor
-	ai_interface(info& arg) : info_(arg), last_interact_(0) {}
+	ai_interface(info& arg) : info_(arg), last_interact_(0), user_interact_("ai_user_interact"),
+		unit_recruited_("ai_unit_recruited"), unit_moved_("ai_unit_moved"),
+		enemy_attacked_("ai_enemy_attacked") {}
 	virtual ~ai_interface() {}
 
 	///the function that is called when the AI must play its turn. Derived classes should
@@ -76,23 +79,26 @@ public:
 	team& current_team();
 	const team& current_team() const;
 
-	///function to update network players as to what the AI has done so far this turn
-	void sync_network();
-
 	///function to show a diagnostic message on the screen
 	void diagnostic(const std::string& msg);
 
 	///function to display a debug message as a chat message is displayed
 	void log_message(const std::string& msg);
 
+	//exposes events to allow for attaching/detaching handlers
+	events::generic_event& user_interact() { return user_interact_; }
+	events::generic_event& unit_recruited() { return unit_recruited_; }
+	events::generic_event& unit_moved() { return unit_moved_; }
+	events::generic_event& enemy_attacked() { return enemy_attacked_; }
+
 protected:
 	///this function should be called to attack an enemy.
 	///'attacking_unit': the location of the attacking unit
 	///'target': the location of the target unit. This unit must be in range of the
 	///attacking unit's weapon
-	///'weapon': the number of the weapon (0-based) which should be used in the attack.
+	///'att_weapon': the number of the weapon (0-based) which should be used in the attack.
 	///must be a valid weapon of the attacking unit
-	void attack_enemy(const location& attacking_unit, const location& target, int weapon);
+	void attack_enemy(const location& attacking_unit, const location& target, int att_weapon, int def_weapon);
 
 	///this function should be called to move a unit. Once the unit has been moved, its
 	///movement allowance is set to 0.
@@ -136,13 +142,23 @@ protected:
 	///function which should be called frequently to allow the user to interact with
 	///the interface. This function will make sure that interaction doesn't occur
 	///too often, so there is no problem with calling it very regularly
-	void user_interact();
+	void raise_user_interact();
 
+	///notifies all interested observers of the event respectively
+	void raise_unit_recruited();
+	void raise_unit_moved();
+	void raise_enemy_attacked();
 private:
 	info info_;
 	int last_interact_;
+	events::generic_event user_interact_;
+	events::generic_event unit_recruited_;
+	events::generic_event unit_moved_;
+	events::generic_event enemy_attacked_;
 };
 
+///this function returns all currently available AIs
+std::vector<std::string> get_available_ais();
 ///this function is used to create a new AI object with the specified algorithm name
 ai_interface* create_ai(const std::string& algorithm_name, ai_interface::info& info);
 
