@@ -1,4 +1,4 @@
-/* $Id: ai_python.cpp 12772 2006-07-25 19:08:19Z elias $ */
+/* $Id: ai_python.cpp 14167 2006-11-05 21:36:13Z elias $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -22,13 +22,18 @@
 #include "gamestatus.hpp"
 #include "filesystem.hpp"
 #include "menu_events.hpp"
+#include "game_events.hpp"
+#include "game_config.hpp"
 
 #include <fstream>
 
+#define LOG_AI LOG_STREAM(info, ai)
+
 static python_ai* running_instance;
 bool python_ai::init_ = false;
 PyObject* python_ai::python_error_ = NULL;
 
+#define MSG_UNIT "Invalid unit"
 void python_ai::set_error(const char *fmt, ...)
 {
 	char buf[1024];
@@ -47,7 +52,7 @@ static PyObject* wrapper_unittype_resistance_against(wesnoth_unittype* type, PyO
 
 static PyObject* wrapper_unittype_get_name(wesnoth_unittype* unit, void* /*closure*/)
 {
-	return Py_BuildValue("s",( const char* )unit->unit_type_->language_name().c_str());
+	return Py_BuildValue("s",( const char* )unit->unit_type_->id().c_str());
 }
 
 #define ut_get( x ) \
@@ -431,81 +436,112 @@ bool python_ai::is_unit_valid(const unit* unit, bool do_set_error)
 
 static PyObject* unit_get_name(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("s",( const char* )unit->unit_->name().c_str());
 }
 
 static PyObject* unit_is_enemy(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",running_instance->current_team().is_enemy(unit->unit_->side()) == true ? 1 : 0);
 }
 
 static PyObject* unit_can_recruit(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",unit->unit_->can_recruit() == true ? 1 : 0);
 }
 
 static PyObject* unit_side(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",unit->unit_->side());
 }
 
 static PyObject* unit_movement_left(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",unit->unit_->movement_left());
 }
 
 static PyObject* unit_can_attack(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",unit->unit_->attacks_left());
 }
 
 static PyObject* unit_hitpoints(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",(int)unit->unit_->hitpoints());
 }
 
 static PyObject* unit_max_hitpoints(wesnoth_unit* unit, void* /*closure*/)
 {
-       if (!running_instance->is_unit_valid(unit->unit_))
-               return NULL;
-       return Py_BuildValue("i",(int)unit->unit_->max_hitpoints());
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
+		return NULL;
+	}
+        return Py_BuildValue("i",(int)unit->unit_->max_hitpoints());
 }
 
 static PyObject* unit_experience(wesnoth_unit* unit, void* /*closure*/)
 {
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return Py_BuildValue("i",(int)unit->unit_->experience());
 }
 
 static PyObject* unit_max_experience(wesnoth_unit* unit, void* /*closure*/)
 {
-       if (!running_instance->is_unit_valid(unit->unit_))
-               return NULL;
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
+		return NULL;
+	}
        return Py_BuildValue("i",(int)unit->unit_->max_experience());
 }
 
 static PyObject* unit_poisoned(wesnoth_unit* unit, void* /*closure*/)
 {
-    if (!running_instance->is_unit_valid(unit->unit_))
-        return NULL;
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
+		return NULL;
+	}
     return Py_BuildValue("i",utils::string_bool(unit->unit_->get_state("poisoned")));
 }
 
+static PyObject* unit_stoned(wesnoth_unit* unit, void* /*closure*/)
+{
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
+		return NULL;
+	}
+    return Py_BuildValue("i",utils::string_bool(unit->unit_->get_state("stoned")));
+}
+
 static PyObject* unit_query_valid(wesnoth_unit* unit, void* /*closure*/)
 {
 	return Py_BuildValue("i",running_instance->is_unit_valid(unit->unit_, false) == true ? 1 : 0);
@@ -513,7 +549,7 @@ static PyObject* unit_query_valid(wesnoth_unit* unit, void* /*closure*/)
 
 static PyGetSetDef unit_getseters[] = {
 	{ "name",			(getter)unit_get_name,		NULL,
-        "Name of the unit.",	NULL },
+        "Name of the unit (''description'' from WML).",	NULL },
 	{ "is_enemy",		(getter)unit_is_enemy,		NULL,
         "True if this is an enemy unit, False if it is allied.",	NULL },
 	{ "can_recruit",	(getter)unit_can_recruit,	NULL,
@@ -537,6 +573,8 @@ static PyGetSetDef unit_getseters[] = {
         "If the unit can still attack.",	NULL},
 	{ "poisoned",		(getter)unit_poisoned,	NULL,
         "If the unit is poisoned.",	NULL },
+	{ "stoned",		(getter)unit_stoned,	NULL,
+        "If the unit is stoned.",	NULL },
 	{ NULL,				NULL,						NULL,	NULL,	NULL }
 };
 
@@ -544,8 +582,10 @@ static PyObject* wrapper_unit_type( wesnoth_unit* unit, PyObject* args )
 {
 	if ( !PyArg_ParseTuple( args, "" ) )
 		return NULL;
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	return wrap_unittype(unit->unit_->type());
 }
 
@@ -553,8 +593,10 @@ static PyObject* wrapper_unit_attacks( wesnoth_unit* unit, PyObject* args )
 {
 	if ( !PyArg_ParseTuple( args, "" ) )
 		return NULL;
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	PyObject* list = PyList_New(unit->unit_->attacks().size());
 	for ( size_t attack = 0; attack < unit->unit_->attacks().size(); attack++)
 		PyList_SetItem(list,attack,wrap_attacktype(unit->unit_->attacks()[attack]));
@@ -566,8 +608,10 @@ static PyObject* wrapper_unit_damage_from( wesnoth_unit* unit, PyObject* args )
 	wesnoth_attacktype* attack;
 	if ( !PyArg_ParseTuple( args, "O!", &wesnoth_attacktype_type, &attack ) )
 		return NULL;
-	if (!running_instance->is_unit_valid(unit->unit_))
+	if (!running_instance->is_unit_valid(unit->unit_)){
+		running_instance->set_error(MSG_UNIT);
 		return NULL;
+	}
 	static gamemap::location no_loc;
 	return Py_BuildValue("i",unit->unit_->damage_from(*attack->attack_type_,true,no_loc));
 }
@@ -607,15 +651,15 @@ static PyMethodDef unit_methods[] = {
         "Finds a path from 'from' to 'to' costing less than 'max_cost' "
         "movement points to reach and returns it as a list of locations. "
         "path[0] will be 'from', path[-1] will be 'to'. "
-        "If no path can be found (for example, if the target is occupied by "
-        "another unit, or it would cost more than max_cost), an empty list is returned."},
+        "If no path can be found (for example, if the target is not reachable, "
+        "or it would cost more than max_cost), an empty list is returned."},
     { "attack_statistics", (PyCFunction)python_ai::wrapper_unit_attack_statistics, METH_VARARGS,
         "Parameters: location from, location to, int attack = -1\n"
         "Returns: own_hp, enemy_hp\n"
         "Returns two dictionaries with the expected battle results when the "
         "unit attacks from 'from' to the unit at 'to', optionally using the "
         "attack with index 'attack', or if no attack is given the attack which "
-        "would be presented to the playe in the attack dialog. The "
+        "would be presented to the player in the attack dialog. The "
         "dictionaries contain the expected hitpoints after "
         "the fight, as a mapping from hitpoints to percent, where percent are "
         "specified as floating point value from 0 to 1. For example, a return of: "
@@ -1269,7 +1313,15 @@ PyObject* python_ai::wrapper_get_location(PyObject* /*self*/, PyObject* args)
 	int x, y;
     if ( !PyArg_ParseTuple( args, "ii", &x, &y ) )
         return NULL;
-
+    if(x < 0 || x >= running_instance->get_info().map.x()){
+	running_instance->set_error("Invalid x value %d", x);
+	return NULL;
+    }
+    if(y < 0 || y >= running_instance->get_info().map.y()){
+	running_instance->set_error("Invalid y value %d", y);
+	return NULL;
+    }
+    
     gamemap::location loc(x,y);
 	return wrap_location(loc);
 }
@@ -1505,6 +1557,10 @@ PyObject* python_ai::wrapper_unit_attack_statistics(wesnoth_unit* self, PyObject
 		return NULL;
 	if (!running_instance->is_unit_valid(self->unit_))
 		return NULL;
+	if (weapon < -1 || weapon >= (int) self->unit_->attacks().size()){
+		set_error("Invalid weapon %d", weapon);
+		return NULL;
+	}
 
     info& inf = running_instance->get_info();
 
@@ -1573,6 +1629,13 @@ PyObject* python_ai::wrapper_get_variable(PyObject* /*self*/, PyObject* args)
     return Py_BuildValue("s", memory[variable].c_str());
 }
 
+PyObject* python_ai::wrapper_get_version(PyObject* /*self*/, PyObject* args)
+{
+    if (!PyArg_ParseTuple(args, "" ))
+        return NULL;
+    return Py_BuildValue("s", game_config::version.c_str());
+}
+
 static PyMethodDef wesnoth_python_methods[] = {
     { "log_message",				python_ai::wrapper_log_message,			METH_VARARGS,
         "Parameters: string\n"
@@ -1641,7 +1704,9 @@ static PyMethodDef wesnoth_python_methods[] = {
         "Returns: value\n"
         "Retrieves a persistent variable 'variable' from the AI, which has "
         "previously been set with set_variable."},
-	{ NULL,						NULL,						0, NULL}
+    { "get_version", python_ai::wrapper_get_version, METH_VARARGS,
+        "Returns a string containing current Wesnoth version"},
+        { NULL, NULL, 0, NULL}
 };
 
 
@@ -1654,6 +1719,7 @@ static PyMethodDef wesnoth_python_methods[] = {
 
 python_ai::python_ai(ai_interface::info& info) : ai_interface(info), exception(QUIT)
 {
+    LOG_AI << "Running Python instance.\n";
 	running_instance = this;
 	if ( !init_ )
 	{
@@ -1678,51 +1744,66 @@ python_ai::python_ai(ai_interface::info& info) : ai_interface(info), exception(Q
 }
 
 python_ai::~python_ai()
-	{
-/*	Py_DECREF(&Wesnoth_UnitType);
-	Py_DECREF(&wesnoth_location_type);
-	Py_Finalize();*/
-	running_instance = NULL;
-	}
+{
+    // This is called whenever the AI is destroyed after its turn - the Python
+    // interpreter itself will be auto cleaned up at program exit.
+    LOG_AI << "Closing Python instance.\n";
+    running_instance = NULL;
+}
 
 void python_ai::play_turn()
 {
+	game_events::fire("ai turn");
+	
 	std::string script_name = current_team().ai_parameters()["python_script"];
+	if (script_name.substr(script_name.length() - 3) != ".py") {
+	    // Make sure the script ends in .py here - Wesnoth will not execute any
+	    // other files.
+	    std::cerr << "\"" << script_name << "\" is not a valid script name.\n";
+	    return;
+	}
 	std::string script = get_binary_file_location("data", "ais/" + script_name);
-	PyObject* file = PyFile_FromString((char*)script.c_str(),"rt");
 
     PyErr_Clear();
 
-    PyObject* dict = PyDict_New();
-    PyDict_SetItemString(dict, "__builtins__", PyEval_GetBuiltins());
+    PyObject* globals = PyDict_New();
+    PyDict_SetItemString(globals, "__builtins__", PyEval_GetBuiltins());
 
-	// Always execute an import statement including all the current binary
-	// pathes, so the python script can import any other python modules.
-	// e.g. sys.path.extend(['~/.wesnoth/data/ais', '/usr/share/wesnoth/data/ais', ])
-	std::string import_modules("import sys; sys.path.extend([");
-	const std::vector<std::string>& paths = get_binary_paths("data");
-	for(std::vector<std::string>::const_iterator i = paths.begin(); i != paths.end(); ++i) {
-		import_modules += "'" + *i + "ais', ";
-	}
-	import_modules += "])\n";
-	PyObject* pre = PyRun_String(import_modules.c_str(), Py_file_input, dict, dict);
+    LOG_AI << "Executing Python script \"" << script << "\".\n";
+    // Run the python script. We actually execute a short inline python
+    // script, which sets up the module search path to the current binary
+    // pathes, runs the script with execfile, and then resets the path.
+    std::string python_code;
+    python_code += "import sys\n"
+        "backup = sys.path[:]; sys.path.extend([";
+        const std::vector<std::string>& paths = get_binary_paths("data");
+        std::vector<std::string>::const_iterator i;
+        for (i = paths.begin(); i != paths.end(); ++i) {
+            python_code += "'" + *i + "ais', ";
+    }
+    python_code += "])\n"
+        "try:\n"
+        "    execfile(\"" + script + "\")\n"
+        "finally:\n"
+        "    sys.path = backup\n";
+    PyObject *ret = PyRun_String(python_code.c_str(), Py_file_input,
+        globals, globals);
 
-    // Now execute the actual python AI.
-    PyObject* ret = PyRun_File(PyFile_AsFile(file), script.c_str(), Py_file_input, dict, dict);
-
-    Py_XDECREF(pre);
     Py_XDECREF(ret);
-    Py_DECREF(dict);
-    Py_DECREF(file);
+    Py_DECREF(globals);
 
     if (PyErr_Occurred()) {
         // RuntimeError is the game-won exception, no need to print it
-        if (!PyErr_ExceptionMatches(PyExc_RuntimeError))
+        if (!PyErr_ExceptionMatches(PyExc_RuntimeError)) {
+            LOG_AI << "Python script has crashed.\n";
             PyErr_Print();
+        }
         // Otherwise, re-throw the exception here, so it will get handled
         // properly further up.
-        else
+        else {
+            LOG_AI << "Python script has been interrupted.\n";
             throw exception;
+        }
     }
 }
 
