@@ -1,4 +1,4 @@
-/* $Id: animated.cpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: animated.cpp 11346 2006-05-08 08:07:29Z boucman $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -100,17 +100,19 @@ void animated<T,T_void_value>::start_animation(int start_frame, int cycles, int
 	acceleration_ = acceleration;
 	// current_frame_ = frames_.begin();
 	current_frame_ = 0;
+	no_current_frame_ = false;
 
 	if (ending_frame_time_ >= start_frame_) {
 		duration_ = ending_frame_time_ - start_frame_;
 	} else {
 		duration_ = 0;
 	}
+	frame_changed_ = true;
 }
 
 
 template<typename T,  typename T_void_value>
-void animated<T,T_void_value>::update_current_frame_internal()
+void animated<T,T_void_value>::update_current_frame()
 {
 	// std::cerr << "--- updating frame ---\n";
 	if(does_not_change_)
@@ -180,10 +182,12 @@ void animated<T,T_void_value>::update_current_frame_internal()
 	} else {
 		// If the duration is void, the animation is automatically finished.
 		// current_cycle_ = cycles_;
-		if(cycles_ != -1)
+		if(cycles_ != -1) {
 			started_ = false;
+		} else {
 
 		does_not_change_ = true;
+		}
 		frame_changed_ = false;
 		// current_frame_ = frames_.size() - 1;
 		// frame_changed_ = false;
@@ -220,7 +224,7 @@ int animated<T,T_void_value>::get_animation_time() const
 }
 
 template<typename T,  typename T_void_value>
-int animated<T,T_void_value>::get_frame_time() const
+int animated<T,T_void_value>::get_cycle_time() const
 {
 	return current_time_ - start_ticks_ + start_frame_;
 }
@@ -236,6 +240,34 @@ const T& animated<T,T_void_value>::get_current_frame() const
 	return cur.value;
 }
 
+template<typename T,  typename T_void_value>
+const T& animated<T,T_void_value>::get_first_frame() const
+{
+	typename std::vector<frame>::const_iterator frame_;
+	for(frame_ = frames_.begin() ; frame_ != frames_.end(); frame_++ ) {
+		if(frame_->has_value)
+			return frame_->value;
+	}
+	return void_value_;
+}
+
+template<typename T,  typename T_void_value>
+const T& animated<T,T_void_value>::get_last_frame() const
+{
+	typename std::vector<frame>::const_reverse_iterator frame_;
+	for(frame_ = frames_.rbegin() ; frame_ != frames_.rend(); frame_++ ) {
+		if(frame_->has_value)
+			return frame_->value;
+	}
+	return void_value_;
+}
+
+template<typename T, typename T_void_value>
+int animated<T,T_void_value>::get_frames_count() const
+{
+	return frames_.size();
+}
+
 template<typename T,  typename T_void_value>
 int animated<T,T_void_value>::get_first_frame_time() const
 {
@@ -254,11 +286,12 @@ int animated<T,T_void_value>::get_last_frame_time() const
 	return 0;
 }
 
+
 // Force compilation of the following template instantiations
 
+#include "unit_frame.hpp"
 #include "image.hpp"
-#include "unit_types.hpp"
 
 template class animated< image::locator >;
 template class animated< std::string >;
-template class animated< unit_animation::frame >;
+template class animated< unit_frame >;
