@@ -1,4 +1,4 @@
-/* $Id: builder.cpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: builder.cpp 9702 2006-01-15 13:03:14Z j_daniel $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -828,7 +828,7 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, c
 
 		random %= 100;
 
-		if(random > rule.probability)
+		if(random > (unsigned int)rule.probability)
 			return false;
 	}
 
@@ -1077,7 +1077,7 @@ void terrain_builder::build_terrains()
 							continue;
 					}
 
-					if(rule_matches(rule->second, *itor - loc, rule_index, (biggest_constraint_adjacent + 1) != rule->second.constraints.size())) {
+					if(rule_matches(rule->second, *itor - loc, rule_index, (size_t)(biggest_constraint_adjacent + 1) != rule->second.constraints.size())) {
 						apply_rule(rule->second, *itor - loc);
 					}
 				}
