@@ -1,4 +1,4 @@
-/* $Id: campaign_server.cpp 12009 2006-06-10 18:59:00Z torangan $ */
+/* $Id: campaign_server.cpp 14699 2006-12-16 21:39:31Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -62,7 +62,7 @@ int campaign_server::load_config()
 {
 	scoped_istream stream = istream_file(file_);
 	read(cfg_, *stream);
-	return lexical_cast_default<int>(cfg_["port"], 15003);
+	return lexical_cast_default<int>(cfg_["port"], 15004);
 }
 
 campaign_server::campaign_server(const std::string& cfgfile)
@@ -87,6 +87,61 @@ void find_translations(const config& cfg, config& campaign)
         }
 }
 
+// Given an uploaded campaign, rename all .py files to .py.unchecked, unless
+// the contents are the same as a .py file in an existing campaign.
+// This means, a .py.unchecked file can be approved by simply renaming it
+// on the CS, and it will remain approved as long as it is not changed (but
+// it can be moved/renamed). If the .py file changes, it needs to be approved
+// again.
+std::string check_python_scripts(config &data, std::string filename)
+{
+	std::vector<config *> python_scripts = find_scripts(data, ".py");
+	if (!python_scripts.empty()) {
+		// Campaign contains python scripts.
+		config old_campaign;
+		scoped_istream stream = istream_file(filename);
+		read_compressed(old_campaign, *stream);
+		std::vector<config *> old_scripts = find_scripts(old_campaign, ".py");
+		std::string script_names = "";
+		std::vector<config *>::iterator i, j;
+		// Go through all newly uploaded python scripts.
+		for (i = python_scripts.begin(); i != python_scripts.end(); ++i) {
+			bool already = false;
+			// Compare to existing, approved scripts.
+			for (j = old_scripts.begin(); j != old_scripts.end(); ++j) {
+				if ((**i)["contents"] != (**j)["contents"]) continue;
+				already = true;
+				break;
+			}
+			if (!already) {
+				script_names += "\n" + (**i)["name"];
+				(**i)["name"] += ".unchecked";
+			}
+		}
+		if (script_names != "")
+			return "\nScripts awaiting approval:\n" + script_names;
+	}
+	return "";
+}
+
+// Add a file COPYING.txt with the GPL to an uploaded campaign.
+void add_license(config &data)
+{
+	config *dir = data.find_child("dir", "name", data["campaign_name"]);
+	// No top-level directory? Hm..
+	if (!dir) return;
+
+	// Don't add if it already exists.
+	if (dir->find_child("file", "name", "COPYING.txt")) return;
+	if (dir->find_child("file", "name", "COPYING")) return;
+
+	// Copy over COPYING.txt
+	std::string contents = read_file("data/COPYING.txt");
+	config &copying = dir->add_child("file");
+	copying["name"] = "COPYING.txt";
+	copying["contents"] = contents;
+}
+
 void campaign_server::run()
 {
 	for(int increment = 0; ; ++increment) {
@@ -168,6 +223,7 @@ void campaign_server::run()
 						config cfg;
 						scoped_istream stream = istream_file((*campaign)["filename"]);
 						read_compressed(cfg, *stream);
+						add_license(cfg);
 						network::queue_data(cfg,sock);
 
 						const int downloads = lexical_cast_default<int>((*campaign)["downloads"],0)+1;
@@ -188,6 +244,7 @@ void campaign_server::run()
 					} else if(check_names_legal(*data) == false) {
 						network::send_data(construct_error("The add-on contains an illegal file or directory name."),sock);
 					} else {
+						std::string message = "Add-on accepted.";
 						if(campaign == NULL) {
 							campaign = &campaigns().add_child("campaign");
 						}
@@ -217,16 +274,19 @@ void campaign_server::run()
 						(*data)["icon"] = (*campaign)["icon"];
 						(*campaign).clear_children("translation");
 						find_translations(*data, *campaign);
-						{
-							scoped_ostream campaign_file = ostream_file(filename);
-							write_compressed(*campaign_file, *data);
+
+						// Campaigns which have python="allowed" are not checked
+						if ((*campaign)["python"] != "allowed") {
+							message += check_python_scripts(*data, filename);
 						}
+						scoped_ostream campaign_file = ostream_file(filename);
+						write_compressed(*campaign_file, *data);
+
 						(*campaign)["size"] = lexical_cast<std::string>(
 							file_size(filename));
-
 						scoped_ostream cfgfile = ostream_file(file_);
 						write(*cfgfile, cfg_);
-						network::send_data(construct_message("Add-on accepted."),sock);
+						network::send_data(construct_message(message),sock);
 					}
 				} else if(const config* erase = data.child("delete")) {
 					config* const campaign = campaigns().find_child("campaign","name",(*erase)["name"]);
