@@ -1,4 +1,4 @@
-/* $Id: dialogs.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: dialogs.hpp 11657 2006-05-27 05:21:16Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -66,12 +66,12 @@ public:
 	enum TYPE { SHOW_ALL, SHOW_BASIC };
 
 	unit_preview_pane(display &disp, const gamemap* map, const unit& u, TYPE type=SHOW_ALL, bool left_side=true);
-	unit_preview_pane(display &disp, const gamemap* map, const std::vector<unit>& units, TYPE type=SHOW_ALL, bool left_side=true);
+	unit_preview_pane(display &disp, const gamemap* map, std::vector<unit>& units, TYPE type=SHOW_ALL, bool left_side=true);
 
 	bool show_above() const;
 	bool left_side() const;
 	void set_selection(int index);
-
+	void join();
 private:
 	display& disp_;
 
@@ -80,7 +80,7 @@ private:
 
 	gui::button details_button_;
 	const gamemap* map_;
-	const std::vector<unit>* units_;
+	std::vector<unit>* units_;
 	std::vector<unit> unit_store_;
 	int index_;
 	bool left_;
