@@ -1,4 +1,4 @@
-/* $Id: display.cpp 11959 2006-06-09 04:57:26Z sapient $ */
+/* $Id: display.cpp 14162 2006-11-05 17:20:46Z boucman $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -633,7 +633,10 @@ void display::scroll_to_leader(unit_map& units, int side)
 	const unit_map::iterator leader = find_leader(units,side);
 
 	if(leader != units_.end()) {
-		const hotkey::basic_handler key_events_handler(this);
+		/* YogiHH: I can't see why we need another key_handler here, therefore
+		i will comment it out
+		const hotkey::basic_handler key_events_handler(gui_);
+		*/
 		scroll_to_tile(leader->first.x,leader->first.y,ONSCREEN);
 	}
 }
@@ -829,6 +832,8 @@ void display::draw(bool update,bool force)
 			draw_label(video(),screen,*i);
 		}
 
+		create_buttons();
+
 		//invalidate the reports so they are redrawn
 		std::fill(reports_,reports_+sizeof(reports_)/sizeof(*reports_),reports::report());
 		invalidateGameStatus_ = true;
@@ -2015,6 +2020,12 @@ void display::invalidate(const gamemap::location& loc)
 					invalidate(*i);
 				}
 			}
+			if (temp_unit_  && temp_unit_loc_ == loc ) {
+				std::set<gamemap::location> overlaps = temp_unit_->overlaps(temp_unit_loc_);
+				for (std::set<gamemap::location>::iterator i = overlaps.begin(); i != overlaps.end(); i++) {
+					invalidate(*i);
+				}
+			}
 			// if neighbour has a unit which overlaps us invalidate him
 			gamemap::location adjacent[6];
 			get_adjacent_tiles(loc, adjacent);
@@ -2026,6 +2037,12 @@ void display::invalidate(const gamemap::location& loc)
 						invalidate(u->first);
 					}
 				}
+				if (temp_unit_  && temp_unit_loc_ == adjacent[i] ) {
+					std::set<gamemap::location> overlaps = temp_unit_->overlaps(temp_unit_loc_);
+					if (overlaps.find(loc) != overlaps.end()) {
+						invalidate(temp_unit_loc_);
+					}
+				}
 			}
 		}
 	}
@@ -2060,7 +2077,7 @@ void display::invalidate_animations()
 	for(int x = topleft.x; x <= bottomright.x; ++x) {
 		for(int y = topleft.y; y <= bottomright.y; ++y) {
 			gamemap::location loc(x,y);
-			if(builder_.update_animation(loc) || (map_.is_village(loc) && animate_flags)) {
+			if(builder_.update_animation(loc) || (map_.is_village(loc) && animate_flags && player_teams::village_owner(loc) != -1)) {
 				invalidate(loc);
 			}
 		}
@@ -2074,6 +2091,10 @@ void display::invalidate_animations()
 
 }
 
+void display::invalidate_theme(){
+	panelsDrawn_ = false;
+}
+
 void display::recalculate_minimap()
 {
 	if(minimap_ != NULL) {
@@ -2341,14 +2362,14 @@ namespace {
 	const SDL_Color chat_message_bg     = {0,0,0,140};
 }
 
-void display::add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type)
+void display::add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type, bool bell)
 {
 	config* cignore;
 	bool ignored = false;
 	if ((cignore = preferences::get_prefs()->child("ignore"))){
 		for(std::map<std::string,t_string>::const_iterator i = cignore->values.begin();
 		i != cignore->values.end(); ++i){
-			if(speaker == i->first){
+			if(speaker == i->first || speaker == "whisper: " + i->first){
 				if (i->second == "yes"){
 					ignored = true;
 				}
@@ -2360,6 +2381,10 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 		bool action;
 		std::string msg;
 
+		if (bell) {
+			sound::play_sound(game_config::sounds::receive_message);
+		}
+
 		if(message.find("/me ") == 0) {
 			msg.assign(message,4,message.size());
 			action = true;
