@@ -1,4 +1,4 @@
-/* $Id: display.hpp 11673 2006-05-28 05:21:57Z rusty $ */
+/* $Id: display.hpp 14020 2006-10-19 12:48:28Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -237,6 +237,10 @@ public:
 	//function to invalidate animated terrains which may have changed.
 	void invalidate_animations();
 
+	//function to invalidate controls and panels when changed after
+	//they have been drawn initially. Useful for dynamic theme modification.
+	void invalidate_theme();
+
 	//function to schedule the minimap for recalculation. Useful if any
 	//terrain in the map has changed.
 	void recalculate_minimap();
@@ -335,7 +339,7 @@ public:
 	void set_diagnostic(const std::string& msg);
 
 	enum MESSAGE_TYPE { MESSAGE_PUBLIC, MESSAGE_PRIVATE };
-	void add_chat_message(const std::string& speaker, int side, const std::string& msg, MESSAGE_TYPE type);
+	void add_chat_message(const std::string& speaker, int side, const std::string& msg, MESSAGE_TYPE type, bool bell);
 	void clear_chat_messages();
 
 	//function to draw the image of a unit at a certain location
