@@ -12,23 +12,24 @@
 
 #include "SDL.h"
 
-#include "../display.hpp"
-#include "../show_dialog.hpp"
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "../config.hpp"
+#include "../display.hpp"
 #include "../events.hpp"
+#include "../filesystem.hpp"
 #include "../game_config.hpp"
 #include "../gettext.hpp"
-#include "../mapgen.hpp"
-#include "../map_create.hpp"
-#include "../filesystem.hpp"
-#include "../font.hpp"
-#include "../events.hpp"
-#include "../util.hpp"
-#include "../widgets/slider.hpp"
 #include "../language.hpp"
 #include "../map.hpp"
-#include "../preferences.hpp"
+#include "../mapgen.hpp"
+#include "../map_create.hpp"
+#include "../marked-up_text.hpp"
+#include "../show_dialog.hpp"
+#include "../util.hpp"
+#include "../preferences_display.hpp"
 #include "../video.hpp"
+#include "../widgets/slider.hpp"
 
 #include "editor_dialogs.hpp"
 
