@@ -1,4 +1,4 @@
-/* $Id: events.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: events.cpp 11938 2006-06-08 09:53:13Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -17,7 +17,8 @@
 #include "cursor.hpp"
 #include "events.hpp"
 #include "mouse.hpp"
-#include "preferences.hpp"
+#include "preferences_display.hpp"
+#include "sound.hpp"
 #include "video.hpp"
 #include "wassert.hpp"
 
@@ -144,39 +145,49 @@ std::deque<context> event_contexts;
 
 } //end anon namespace
 
-event_context::event_context(bool create) : create_(create)
+event_context::event_context()
 {
-	if(create_) {
-		event_contexts.push_back(context());
-	}
+	event_contexts.push_back(context());
 }
 
 event_context::~event_context()
 {
-	if(create_) {
-		wassert(event_contexts.empty() == false);
-
-		event_contexts.pop_back();
-	}
+	wassert(event_contexts.empty() == false);
+	event_contexts.pop_back();
 }
 
-handler::handler() : unicode_(SDL_EnableUNICODE(1))
+handler::handler(bool auto_join) : unicode_(SDL_EnableUNICODE(1)), has_joined_(false)
 {
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
-	event_contexts.back().add_handler(this);
+	if(auto_join) {
+		join();
+	}
 }
 
 handler::~handler()
 {
-	wassert(event_contexts.empty() == false);
+	leave();
+	SDL_EnableUNICODE(unicode_);
+}
 
+void handler::join()
+{
+	if(has_joined_) {
+		leave(); // should not be in multiple event contexts
+	}
+	event_contexts.back().add_handler(this);
+	has_joined_ = true;
+}
+
+void handler::leave()
+{
+	wassert(event_contexts.empty() == false);
 	for(std::deque<context>::reverse_iterator i = event_contexts.rbegin(); i != event_contexts.rend(); ++i) {
 		if(i->remove_handler(this)) {
 			break;
 		}
 	}
-
-	SDL_EnableUNICODE(unicode_);
+	//has_joined_ = false;
 }
 
 void focus_handler(const handler* ptr)
@@ -331,6 +342,9 @@ void pump()
 		resize_dimensions.first = 0;
 		resize_dimensions.second = 0;
 	}
+
+	if (preferences::music_on())
+		sound::think_about_music();
 }
 
 void raise_process_event()
