@@ -1,4 +1,4 @@
-/* $Id: events.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: events.hpp 11643 2006-05-25 23:04:34Z sapient $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -48,14 +48,19 @@ public:
 
 	virtual bool requires_event_focus() const { return false; }
 
-	virtual void process_help_string(int mousex, int mousey) {}
+	virtual void process_help_string(int /*mousex*/, int /*mousey*/) {}
+
+	virtual void join(); /*joins the current event context*/
 
 protected:
-	handler();
+	handler(bool auto_join=true);
 	virtual ~handler();
 
 private:
+	void leave(); /*leave the event context*/
+
 	int unicode_;
+	bool has_joined_;
 };
 
 void focus_handler(const handler* ptr);
@@ -74,11 +79,8 @@ bool has_focus(const handler* ptr);
 //before their context is destroyed
 struct event_context
 {
-	event_context(bool create=true);
+	event_context();
 	~event_context();
-
-private:
-	bool create_;
 };
 
 //causes events to be dispatched to all handler objects.
