@@ -1,4 +1,4 @@
-/* $Id: font.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: font.hpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,7 +14,7 @@
 #define FONT_HPP_INCLUDED
 
 #include "SDL.h"
-#include "SDL_ttf.h"
+#include "sdl_ttf/SDL_ttf.h"
 
 class CVideo;
 #include "sdl_utils.hpp"
@@ -46,10 +46,7 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist);
 //various standard colours
 extern const SDL_Color NORMAL_COLOUR, LOBBY_COLOUR, GOOD_COLOUR, BAD_COLOUR,
                        BLACK_COLOUR, DARK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR,
-                       STONED_COLOUR, TITLE_COLOUR;
-
-//standard markups
-extern const char LARGE_TEXT, SMALL_TEXT, GOOD_TEXT, BAD_TEXT, NORMAL_TEXT, BLACK_TEXT, BOLD_TEXT, IMAGE, NULL_MARKUP;
+                       STONED_COLOUR, TITLE_COLOUR, DISABLED_COLOUR;
 
 // font sizes, to be made theme parameters
 #ifdef USE_TINY_GUI
@@ -72,41 +69,24 @@ const int
 	SIZE_LARGE      = relative_size(18),
 	SIZE_XLARGE     = relative_size(24)
   ;
-//function to draw text on the screen. The text will be clipped to area.
-//If the text runs outside of area horizontally, an ellipsis will be displayed
-//at the end of it. If use_tooltips is true, then text with an ellipsis will
-//have a tooltip set for it equivalent to the entire contents of the text.
-//
-//some very basic 'markup' will be done on the text:
-// - any line beginning in # will be displayed in BAD_COLOUR
-// - any line beginning in @ will be displayed in GOOD_COLOUR
-// - any line beginning in + will be displayed with size increased by 2
-// - any line beginning in - will be displayed with size decreased by 2
-// - any line beginning with 0x0n will be displayed in the colour of side n
-//
-//the above special characters can be quoted using a C-style backslash.
-//
-//a bounding rectangle of the text is returned. If gui is NULL, then the
-//text will not be drawn, and a bounding rectangle only will be returned.
-
-SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
-                   const SDL_Color& colour, const std::string& text,
-                   int x, int y, bool use_tooltips = false, int style = 0);
-
-//function which returns the size of text if it were to be drawn.
-SDL_Rect text_area(const std::string& text, int size, int style=0);
 
 // Returns a SDL surface containing the text rendered in a given colour.
 surface get_rendered_text(const std::string& text, int size, const SDL_Color& colour, int style=0);
 
+SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
+						const SDL_Color& colour, const std::string& text,
+						int x, int y, bool use_tooltips, int style);
+
+SDL_Rect draw_text_line(CVideo* gui, const SDL_Rect& area, int size,
+                        const SDL_Color& colour, const std::string& text,
+                        int x, int y, bool use_tooltips, int style);
+
 //measures the width and height of a single ucs2 text line
 SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::const_iterator last, int font_size = SIZE_NORMAL, int style = TTF_STYLE_NORMAL);
 
 // Returns the maximum height of a font, in pixels
 int get_max_height(int size);
 
-bool is_format_char(char c);
-
 ///
 /// Determine the width of a line of text given a certain font size.
 /// The font type used is the default wesnoth font type.
@@ -119,33 +99,12 @@ int line_width(const std::string& line, int font_size, int style=TTF_STYLE_NORMA
 ///
 SDL_Rect line_size(const std::string& line, int font_size, int style=TTF_STYLE_NORMAL);
 
-///
-/// If the text exceedes the specified max width, wrap it one a word basis.
-/// If the is not possible, e.g. the word is too big to fit, wrap it on a
-/// char basis.
-///
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height=-1, int max_lines=-1);
-
 ///
 /// If the text excedes the specified max width, end it with an ellipsis (...)
 ///
 std::string make_text_ellipsis(const std::string& text, int font_size, int max_width);
 
 
-///
-/// Draw text on the screen. This method makes sure that the text
-/// fits within a given maximum width. If a line exceedes this width it
-/// will be wrapped on a word basis if possible, otherwise on a char
-/// basis. This method is otherwise similar to the draw_text method,
-/// but it doesn't support special markup or tooltips.
-///
-/// @return a bounding rectangle of the text.
-///
-SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
-			     const SDL_Color& colour, const std::string& text,
-			     int x, int y, int max_width);
-
-
 /// structure which will hide all current floating labels, and cause floating labels
 /// instantiated after it is created to be displayed
 struct floating_label_context
@@ -195,9 +154,6 @@ void undraw_floating_labels(surface screen);
 
 bool load_font_config();
 
-//function to chop up one long string of text into lines
-size_t text_to_lines(std::string& text, size_t max_length);
-
 }
 
 #endif
