@@ -1,4 +1,4 @@
-/* $Id: game.cpp 12798 2006-07-26 06:54:36Z isaac $ */
+/* $Id: game.cpp 14156 2006-11-05 11:09:59Z elias $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -808,6 +808,9 @@ bool game_controller::new_campaign()
 		defines_map_.clear();
 		defines_map_[difficulties[res]] = preproc_define();
 	}
+#ifdef HAVE_PYTHON
+	defines_map_["PYTHON"] = preproc_define();
+#endif
 
 	state_.campaign_define = campaign["define"];
 
@@ -864,14 +867,20 @@ void game_controller::download_campaigns()
 	}
 
 	const std::vector<std::string> items = utils::split(host, ':');
+	if(items.empty()) {
+		return;
+	}
+
 	host = items.front();
 	preferences::set_campaign_server(host);
 
 	try {
 		const network::manager net_manager;
-		const network::connection sock = network::connect(items.front(),lexical_cast_default<int>(items.back(),15003));
+		const network::connection sock = gui::network_connect_dialog(disp(), _("Connecting to Server..."),
+										items.front(), lexical_cast_default<int>(items.back(),15004) );
 		if(!sock) {
 			gui::show_error_message(disp(), _("Could not connect to host."));
+			preferences::set_campaign_server("");
 			return;
 		}
 
@@ -1037,7 +1046,18 @@ void game_controller::download_campaigns()
 
 		clear_binary_paths_cache();
 
-		gui::show_dialog(disp(),NULL,_("Add-on Installed"),_("The add-on has been installed."),gui::OK_ONLY);
+		std::string warning = "";
+		std::vector<config *> scripts = find_scripts(cfg, ".unchecked");
+		if (!scripts.empty()) {
+			warning += "\nUnchecked script files found:";
+			std::vector<config *>::iterator i;
+			for (i = scripts.begin(); i != scripts.end(); ++i) {
+				warning += "\n" + (**i)["name"];
+			}
+		}
+
+		gui::show_dialog(disp(),NULL,_("Add-on Installed"),_("The add-on has been installed.") +
+		    warning,gui::OK_ONLY);
 	} catch(config::error&) {
 		gui::show_error_message(disp(), _("Network communication error."));
 	} catch(network::error&) {
@@ -1167,7 +1187,7 @@ bool game_controller::play_multiplayer()
 	do {
 		res = gui::show_dialog2(disp(), NULL, _("Multiplayer"), "",
 					   gui::OK_CANCEL, &host_or_join, NULL,
-					   _("Login: "), &login);
+					   _("Login: "), &login, 18);
 		if(login.size() > 18) {
 			gui::show_error_message(disp(), _("The login name you chose is too long, please use a login with less than 18 characters"));
 		}
@@ -1281,7 +1301,7 @@ void game_controller::read_game_cfg(const preproc_map& defines, config& cfg, boo
 {
 	log_scope("read_game_cfg");
 
-	if(defines.size() < 4) {
+	if(defines.size() < 5) {
 		bool is_valid = true;
 		std::stringstream str;
 		str << "-v" << game_config::version;
@@ -1712,13 +1732,13 @@ int play_game(int argc, char** argv)
 		return 0;
 	}
 
-	loadscreen::global_loadscreen->increment_progress(5, "Loading game configuration.");
+	loadscreen::global_loadscreen->increment_progress(5, _ ("Loading game configuration."));
 	res = game.init_config();
 	if(res == false) {
 		std::cerr << "could not initialize game config\n";
 		return 0;
 	}
-	loadscreen::global_loadscreen->increment_progress(10, "Re-initialize fonts for the current language.");
+	loadscreen::global_loadscreen->increment_progress(10, _ ("Re-initialize fonts for the current language."));
 
 	res = font::load_font_config();
 	if(res == false) {
@@ -1801,9 +1821,7 @@ int play_game(int argc, char** argv)
 			about::show_about(game.disp());
 			continue;
 		} else if(res == gui::SHOW_HELP) {
-			config dummy_help_cfg;
-			gamemap dummy_help_map(dummy_help_cfg, "1");
-			help::help_manager help_manager(&game.game_config(), &game.units_data(), &dummy_help_map);
+			help::help_manager help_manager(&game.game_config(), &game.units_data(), NULL);
 			help::show_help(game.disp());
 			continue;
 		} else if(res == gui::GET_ADDONS) {
