@@ -1,4 +1,4 @@
-/* $Id: game_config.cpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: game_config.cpp 12691 2006-07-21 12:30:34Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -24,18 +24,16 @@ namespace game_config
 {
 	int base_income = 2;
 	int village_income = 1;
-	int heal_amount = 4;
-	int healer_heals_per_turn = 8;
-	int cure_amount = 8;
-	int curer_heals_per_turn = 18;
+	int poison_amount= 8;
 	int rest_heal_amount= 2;
 	int recall_cost = 20;
 	int kill_experience = 8;
 	int leadership_bonus = 25;
 	const std::string version = VERSION;
-	bool debug = false, editor = false, ignore_replay_errors = false;
+	bool debug = false, editor = false, ignore_replay_errors = false, mp_debug = false, exit_at_end = false, no_delay = false, disable_autosave = false;
 
-	std::string game_icon = "wesnoth-icon.png", game_title, game_logo, title_music;
+	std::string game_icon = "wesnoth-icon.png", game_title, game_logo, title_music, anonymous_music,
+			victory_music, defeat_music;
 	int title_logo_x = 0, title_logo_y = 0, title_buttons_x = 0, title_buttons_y = 0, title_buttons_padding = 0,
 	    title_tip_x = 0, title_tip_y = 0, title_tip_width = 0, title_tip_padding = 0;
 
@@ -47,12 +45,14 @@ namespace game_config
 	std::string rightside_image = "misc/rightside.png";
 	std::string rightside_image_bot = "misc/rightside-bottom.png";
 
-	std::string moved_energy_image = "misc/bar-energy-moved.png";
-	std::string unmoved_energy_image = "misc/bar-energy-unmoved.png";
-	std::string partmoved_energy_image = "misc/bar-energy-partmoved.png";
-	std::string enemy_energy_image = "misc/bar-energy-enemy.png";
-	std::string ally_energy_image = "misc/bar-energy-ally.png";
-	std::string flag_image = "terrain/flag-team%d-1.png:150,terrain/flag-team%d-2.png:150";
+	std::string energy_image = "misc/bar-energy.png";
+	std::string moved_ball_image = "misc/ball-moved.png";
+	std::string unmoved_ball_image = "misc/ball-unmoved.png";
+	std::string partmoved_ball_image = "misc/ball-partmoved.png";
+	std::string enemy_ball_image = "misc/ball-enemy.png";
+	std::string ally_ball_image = "misc/ball-ally.png";
+	std::string flag_image = "terrain/flag-1.png:150,terrain/flag-2.png:150";
+  std::vector<Uint32> flag_rgb;
 
 	std::string dot_image = "misc/dot.png";
 	std::string cross_image = "misc/cross.png";
@@ -66,6 +66,14 @@ namespace game_config
 
 	std::string download_campaign_image;
 
+	std::string level_image;
+	std::string ellipsis_image;
+
+        std::map<int, color_range > team_rgb_range;
+        std::map<int, std::string > team_rgb_name;
+	
+	std::map<std::string, std::vector<Uint32> > team_rgb_colors;
+	
 	namespace sounds {
 		const std::string turn_bell = "bell.wav",
 		                  receive_message = "receive.wav",
@@ -75,10 +83,14 @@ namespace game_config
 
 
 
+#ifdef __AMIGAOS4__
+	std::string path = "PROGDIR:";
+#else
 #ifdef WESNOTH_PATH
 	std::string path = WESNOTH_PATH;
 #else
 	std::string path = "";
+#endif
 #endif
 
 	void load_config(const config* cfg)
@@ -90,10 +102,7 @@ namespace game_config
 
 		base_income = atoi(v["base_income"].c_str());
 		village_income = atoi(v["village_income"].c_str());
-		heal_amount = atoi(v["heal_amount"].c_str());
-		healer_heals_per_turn = atoi(v["healer_heals_per_turn"].c_str());
-		cure_amount = atoi(v["cure_amount"].c_str());
-		curer_heals_per_turn = atoi(v["curer_heals_per_turn"].c_str());
+		poison_amount = atoi(v["poison_amount"].c_str());
 		rest_heal_amount = atoi(v["rest_heal_amount"].c_str());
 		recall_cost = atoi(v["recall_cost"].c_str());
 		kill_experience = atoi(v["kill_experience"].c_str());
@@ -102,6 +111,9 @@ namespace game_config
 		game_title = v["title"];
 		game_logo = v["logo"];
 		title_music = v["title_music"];
+		anonymous_music = v["anonymous_music"];
+		victory_music = v["victory_music"];
+		defeat_music = v["defeat_music"];
 
 		title_logo_x = atoi(v["logo_x"].c_str());
 		title_logo_y = atoi(v["logo_y"].c_str());
@@ -117,13 +129,13 @@ namespace game_config
 		rightside_image = v["sidebar_image"];
 		rightside_image_bot = v["sidebar_image_bottom"];
 
-		moved_energy_image = v["moved_energy_image"];
-		unmoved_energy_image = v["unmoved_energy_image"];
-		partmoved_energy_image = v["partmoved_energy_image"];
-		enemy_energy_image = v["enemy_energy_image"];
-		ally_energy_image = v["ally_energy_image"];
+		energy_image = v["energy_image"];
+		moved_ball_image = v["moved_ball_image"];
+		unmoved_ball_image = v["unmoved_ball_image"];
+		partmoved_ball_image = v["partmoved_ball_image"];
+		enemy_ball_image = v["enemy_ball_image"];
+		ally_ball_image = v["ally_ball_image"];
 		flag_image = v["flag_image"];
-
 		cross_image = v["cross_image"];
 		dot_image = v["dot_image"];
 
@@ -140,5 +152,50 @@ namespace game_config
 		observer_image = v["observer_image"];
 
 		download_campaign_image = v["download_campaign_image"];
+		level_image = v["level_image"];
+		ellipsis_image = v["ellipsis_image"];
+
+		const config::child_list& team_colors = v.get_children("team_color");
+		for(config::child_list::const_iterator teamC = team_colors.begin(); teamC != team_colors.end(); ++teamC) {
+		    if(!(**teamC)["side"].empty() && !(**teamC)["team_rgb"].empty()){
+		    int side = atoi((**teamC)["side"].c_str());
+		    std::vector<Uint32> temp = string2rgb((**teamC)["team_rgb"]);
+		    team_rgb_range[side] = color_range(temp);
+		    team_rgb_name[side] = (**teamC)["name"];
+		  }
+		}
+		
+		const config* rgbv = v.child("team_colors");
+		if(rgbv) {
+			for(string_map::const_iterator rgb_it = rgbv->values.begin(); rgb_it != rgbv->values.end(); ++rgb_it) {
+				try {
+					team_rgb_colors.insert(std::make_pair(rgb_it->first,string2rgb(rgb_it->second)));
+				} catch(bad_lexical_cast&) {
+					//throw config::error("Invalid team color: " + rgb_it->second);
+				}
+			}
+		}
+		flag_rgb = tc_info(v["flag_rgb"]);
+		if( !flag_rgb.size()){
+		  //set green as old_flag_color
+		  for(int i=255;i>0;i--){
+		    flag_rgb.push_back((Uint32)(i<<8));
+		  }
+		}
+	}
+	const std::vector<Uint32>& tc_info(const std::string& name)
+	{
+		std::map<std::string, std::vector<Uint32> >::const_iterator i = team_rgb_colors.find(name);
+		if(i == team_rgb_colors.end()) {
+			try {
+				team_rgb_colors.insert(std::make_pair(name,string2rgb(name)));
+				return tc_info(name);
+			} catch(bad_lexical_cast&) {
+				static std::vector<Uint32> stv;
+				//throw config::error("Invalid team color: " + name);
+				return stv;
+			}
+		}
+		return i->second;
 	}
 }
