@@ -1,5 +1,5 @@
-/* $Id: game_events.cpp 12700 2006-07-22 08:48:58Z isaac $ */
-/* $Id: game_events.cpp 12700 2006-07-22 08:48:58Z isaac $ */
+/* $Id: game_events.cpp 14656 2006-12-14 02:18:25Z sapient $ */
+/* $Id: game_events.cpp 14656 2006-12-14 02:18:25Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -173,7 +173,16 @@ namespace {
 
 std::set<std::string> used_items;
 
-const size_t MaxLoop = 1024;
+/*
+jhinrichs, 12.11.2006:
+This variable controls the maximum number of hexes in a map, that can be parsed by an event.
+It was set to 1024 before and for larger maps this could become a problem. So it is raised to 
+65536 now, which can feature a map of size 256*256 (maps really shouldn't be bigger than that :-).
+This constant also controls the maximum number of loops for a WML while loop (hence its name).
+If this is felt to be too high now, we must split it into two constants, but i don't feel the need
+at the moment.
+*/
+const size_t MaxLoop = 65536;
 
 bool events_init() { return screen != NULL; }
 
@@ -485,12 +494,14 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 	else if(cmd == "scroll_to") {
 		std::string x = cfg["x"];
 		std::string y = cfg["y"];
+		std::string check_fogged = cfg["check_fogged"];
 		wassert(state_of_game != NULL);
 		x = utils::interpolate_variables_into_string(x, *state_of_game);
 		y = utils::interpolate_variables_into_string(y, *state_of_game);
+		check_fogged = utils::interpolate_variables_into_string(check_fogged, *state_of_game);
 		const int xpos = atoi(x.c_str());
 		const int ypos = atoi(y.c_str());
-		screen->scroll_to_tile(xpos,ypos);
+		screen->scroll_to_tile(xpos,ypos,display::SCROLL,utils::string_bool(check_fogged,false));
 	}
 
 	else if(cmd == "scroll_to_unit") {
@@ -499,9 +510,10 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			if(game_events::unit_matches_filter(u,cfg))
 				break;
 		}
-
+		std::string check_fogged = cfg["check_fogged"];
+		check_fogged = utils::interpolate_variables_into_string(check_fogged, *state_of_game);
 		if(u != units->end()) {
-			screen->scroll_to_tile(u->first.x,u->first.y);
+			screen->scroll_to_tile(u->first.x,u->first.y,display::SCROLL,utils::string_bool(check_fogged,false));
 		}
 	}
 
@@ -578,7 +590,8 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		if(add != "") {
 			status_ptr->modify_turns(add);
 		} else {
-			status_ptr->add_turns(lexical_cast_default<int>(value,0));
+			status_ptr->add_turns(-status_ptr->number_of_turns());
+			status_ptr->add_turns(lexical_cast_default<int>(value,50));
 		}
 
 	}
@@ -839,9 +852,9 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		if(random.empty() == false) {
 			std::string random_value, word;
 			std::vector<std::string> words;
-			std::vector<std::pair<size_t,size_t> > ranges;
+			std::vector<std::pair<long,long> > ranges;
 			int num_choices = 0;
-			std::string::size_type pos = 0, pos2 = std::string::npos, tmp;
+			std::string::size_type pos = 0, pos2 = std::string::npos;
 			std::stringstream ss(std::stringstream::in|std::stringstream::out);
 			while (pos2 != random.length()) {
 				pos = pos2+1;
@@ -852,7 +865,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 
 				word = random.substr(pos, pos2-pos);
 				words.push_back(word);
-				tmp = word.find("..");
+				std::string::size_type tmp = word.find("..");
 
 
 				if (tmp == std::string::npos) {
@@ -866,7 +879,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 					const std::string second = word.substr(tmp+2,
 							random.length());
 
-					size_t low, high;
+					long low, high;
 					ss << first + " " + second;
 					ss >> low;
 					ss >> high;
@@ -877,13 +890,13 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 						low = high;
 						high = tmp;
 					}
-					ranges.push_back(std::pair<int, int>(low,high));
+					ranges.push_back(std::pair<long, long>(low,high));
 					num_choices += (high - low) + 1;
 				}
 			}
 
 			size_t choice = get_random() % num_choices;
-			tmp = 0;
+			unsigned long tmp = 0;
 			for(size_t i = 0; i < ranges.size(); i++) {
 				tmp += (ranges[i].second - ranges[i].first) + 1;
 				if (tmp > choice) {
@@ -1160,7 +1173,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 					screen->invalidate(loc);
 					screen->draw();
 					events::pump();
-					screen->non_turbo_delay();
+					screen->delay(10);
 				}
 				un->second.set_standing(*screen,un->first);
 			}
@@ -1197,8 +1210,9 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 				u->set_game_context(game_data_ptr,units,game_map,status_ptr,teams);
 				if(game_events::unit_matches_filter(*u, cfg,gamemap::location())) {
 					gamemap::location loc = cfg_to_loc(cfg);
-					recruit_unit(*game_map,index+1,*units,*u,loc,utils::string_bool(cfg["show"],true) ? NULL : screen,false,true);
-					avail.erase(u);
+					unit to_recruit(*u);
+					avail.erase(u); //erase before recruiting, since recruiting can fire more events
+					recruit_unit(*game_map,index+1,*units,to_recruit,loc,utils::string_bool(cfg["show"],true) ? NULL : screen,false,true);
 					unit_recalled = true;
 					break;
 				}
@@ -1743,6 +1757,14 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 	}
 
 	else if(cmd == "redraw") {
+		std::string side = cfg["side"];
+		wassert(state_of_game != NULL);
+		side = utils::interpolate_variables_into_string(side, *state_of_game);
+		if(side != "") {
+			const int side_num = lexical_cast_default<int>(side);
+			recalculate_fog(*game_map,*status_ptr,*game_data_ptr,*units,*teams,side_num-1);
+			screen->recalculate_minimap();
+		}
 		screen->invalidate_all();
 		screen->draw(true,true);
 	}
@@ -1770,7 +1792,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 				screen->invalidate(u->first);
 				screen->draw();
 				events::pump();
-				screen->non_turbo_delay();
+				screen->delay(10);
 			}
 			u->second.set_standing(*screen,u->first);
 			screen->invalidate(u->first);
@@ -2123,6 +2145,18 @@ bool fire(const std::string& event,
 	return pump();
 }
 
+void add_events(const config::child_list& cfgs,const std::string& id)
+{
+	if(std::find(unit_wml_ids.begin(),unit_wml_ids.end(),id) == unit_wml_ids.end()) {
+		unit_wml_ids.insert(id);
+		for(config::child_list::const_iterator new_ev = cfgs.begin(); new_ev != cfgs.end(); ++ new_ev) {
+			unit_wml_configs.push_back(new config(**new_ev));
+			event_handler new_handler(*unit_wml_configs.back());
+			events_map.insert(std::pair<std::string,event_handler>(new_handler.name(), new_handler));
+		}
+	}
+}
+
 bool pump()
 {
 	if(!events_init())
@@ -2130,20 +2164,6 @@ bool pump()
 
 	bool result = false;
 
-	if(units != NULL) {
-		for(unit_map::const_iterator u_it = units->begin(); u_it != units->end(); ++u_it) {
-			if(std::find(unit_wml_ids.begin(),unit_wml_ids.end(),u_it->second.id()) == unit_wml_ids.end()) {
-				unit_wml_ids.insert(u_it->second.id());
-				config::child_list ev_list = u_it->second.wml_events();
-				for(config::child_list::const_iterator new_ev = ev_list.begin(); new_ev != ev_list.end(); ++ new_ev) {
-					unit_wml_configs.push_back(new config(**new_ev));
-					event_handler new_handler(*unit_wml_configs.back());
-					events_map.insert(std::pair<std::string,event_handler>(new_handler.name(), new_handler));
-				}
-			}
-		}
-	}
-
 	while(events_queue.empty() == false) {
 		queued_event ev = events_queue.front();
 		events_queue.pop_front(); //pop now for exception safety
