@@ -1,4 +1,4 @@
-/* $Id: gamestatus.cpp 12768 2006-07-25 16:32:50Z jhinrichs $ */
+/* $Id: gamestatus.cpp 14736 2006-12-19 10:37:39Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -38,6 +38,76 @@
 #define WRN_NG lg::warn(lg::engine)
 #define ERR_NG lg::err(lg::engine)
 
+#ifdef _WIN32
+#include <windows.h>
+
+// conv_ansi_utf8()
+//   - Convert a string between ANSI encoding (for Windows filename) and UTF-8
+//  string &name
+//     - filename to be converted
+//  bool a2u
+//     - if true, convert the string from ANSI to UTF-8.
+//     - if false, reverse. (convert it from UTF-8 to ANSI)
+void conv_ansi_utf8(std::string &name, bool a2u) {
+	int wlen = MultiByteToWideChar(a2u ? CP_ACP : CP_UTF8, 0,
+                                   name.c_str(), -1, NULL, 0);
+	if (wlen == 0) return;
+	WCHAR *wc = new WCHAR[wlen];
+	if (wc == NULL) return;
+	if (MultiByteToWideChar(a2u ? CP_ACP : CP_UTF8, 0, name.c_str(), -1,
+                            wc, wlen) == 0) {
+		delete wc;
+		return;
+	}
+	int alen = WideCharToMultiByte(!a2u ? CP_ACP : CP_UTF8, 0, wc, wlen,
+                                   NULL, 0, NULL, NULL);
+	if (alen == 0) {
+		delete wc;
+		return;
+	}
+	CHAR *ac = new CHAR[alen];
+	if (ac == NULL) {
+		delete wc;
+		return;
+	}
+	WideCharToMultiByte(!a2u ? CP_ACP : CP_UTF8, 0, wc, wlen,
+                        ac, alen, NULL, NULL);
+	delete wc;
+	if (ac == NULL) {
+		return;
+	}
+	name = ac;
+	delete ac;
+
+	return;
+}
+
+void replace_underbar2space(std::string &name) {
+LOG_NG << "conv(A2U)-from:[" << name << "]" << std::endl;
+    conv_ansi_utf8(name, true);
+LOG_NG << "conv(A2U)-to:[" << name << "]" << std::endl;
+LOG_NG << "replace_underbar2space-from:[" << name << "]" << std::endl;
+    std::replace(name.begin(), name.end(), '_', ' ');
+LOG_NG << "replace_underbar2space-to:[" << name << "]" << std::endl;
+}
+
+void replace_space2underbar(std::string &name) {
+LOG_NG << "conv(U2A)-from:[" << name << "]" << std::endl;
+    conv_ansi_utf8(name, false);
+LOG_NG << "conv(U2A)-to:[" << name << "]" << std::endl;
+LOG_NG << "replace_underbar2space-from:[" << name << "]" << std::endl;
+    std::replace(name.begin(), name.end(), ' ', '_');
+LOG_NG << "replace_underbar2space-to:[" << name << "]" << std::endl;
+}
+#else /* ! _WIN32 */
+void replace_underbar2space(std::string &name) {
+    std::replace(name.begin(),name.end(),'_',' ');
+}
+void replace_space2underbar(std::string &name) {
+    std::replace(name.begin(),name.end(),' ','_');
+}
+#endif /* _WIN32 */
+
 player_info* game_state::get_player(const std::string& id) {
 	std::map< std::string, player_info >::iterator found = players.find(id);
 	if (found == players.end()) {
@@ -93,10 +163,13 @@ void parse_times(const config& cfg, std::vector<time_of_day>& normal_times)
 
 }
 
-gamestatus::gamestatus(const config& time_cfg, int num_turns) :
+gamestatus::gamestatus(const config& time_cfg, int num_turns, game_state* s_o_g) :
                  turn_(1),numTurns_(num_turns)
 {
-	const std::string& turn_at = time_cfg["turn_at"];
+        std::string turn_at = time_cfg["turn_at"];
+	if (s_o_g) {
+	        turn_at = utils::interpolate_variables_into_string(turn_at, *s_o_g);
+	}
 	if(turn_at.empty() == false) {
 		turn_ = atoi(turn_at.c_str());
 	}
@@ -229,6 +302,8 @@ player_info read_player(const game_data& data, const config* cfg)
 {
 	player_info res;
 
+	res.name = (*cfg)["name"];
+
 	res.gold = atoi((*cfg)["gold"].c_str());
 
 	const config::child_list& units = cfg->get_children("unit");
@@ -335,6 +410,8 @@ game_state read_game(const game_data& data, const config* cfg)
 
 void write_player(const player_info& player, config& cfg)
 {
+	cfg["name"] = player.name;
+
 	char buf[50];
 	snprintf(buf,sizeof(buf),"%d",player.gold);
 
@@ -416,7 +493,7 @@ std::vector<save_info> get_saves_list(const std::string *dir)
 	for(std::vector<std::string>::iterator i = saves.begin(); i != saves.end(); ++i) {
 		const time_t modified = file_create_time(saves_dir + "/" + *i);
 
-		std::replace(i->begin(),i->end(),'_',' ');
+		replace_underbar2space(*i);
 		res.push_back(save_info(*i,modified));
 	}
 
@@ -428,7 +505,7 @@ std::vector<save_info> get_saves_list(const std::string *dir)
 bool save_game_exists(const std::string& name)
 {
 	std::string fname = name;
-	std::replace(fname.begin(),fname.end(),' ','_');
+	replace_space2underbar(fname);
 
 	return file_exists(get_saves_dir() + "/" + fname);
 }
@@ -436,7 +513,7 @@ bool save_game_exists(const std::string& name)
 void delete_game(const std::string& name)
 {
 	std::string modified_name = name;
-	std::replace(modified_name.begin(),modified_name.end(),' ','_');
+	replace_space2underbar(modified_name);
 
 	remove((get_saves_dir() + "/" + name).c_str());
 	remove((get_saves_dir() + "/" + modified_name).c_str());
@@ -445,7 +522,7 @@ void delete_game(const std::string& name)
 void read_save_file(const std::string& name, config& cfg, std::string* error_log)
 {
 	std::string modified_name = name;
-	std::replace(modified_name.begin(),modified_name.end(),' ','_');
+	replace_space2underbar(modified_name);
 
 	//try reading the file both with and without underscores
 	scoped_istream file_stream = istream_file(get_saves_dir() + "/" + modified_name);
@@ -453,7 +530,13 @@ void read_save_file(const std::string& name, config& cfg, std::string* error_log
 		file_stream = istream_file(get_saves_dir() + "/" + name);
 
 	cfg.clear();
+	try{
 	detect_format_and_read(cfg, *file_stream, error_log);
+	} catch (config::error &err)
+	{
+		std::cerr << err.message;
+		throw game::load_game_failed();
+	}
 
 	if(cfg.empty()) {
 		std::cerr << "Could not parse file data into config\n";
@@ -486,7 +569,7 @@ void save_game(const game_state& state)
 	log_scope("save_game");
 
 	std::string name = state.label;
-	std::replace(name.begin(),name.end(),' ','_');
+	replace_space2underbar(name);
 
 	config cfg;
 	try {
@@ -897,7 +980,3 @@ void game_state::clear_variable(const std::string& varname)
 		vars->clear_children(key);
 	}
 }
-
-
-
-
