@@ -1,4 +1,4 @@
-/* $Id: gamestatus.hpp 12768 2006-07-25 16:32:50Z jhinrichs $ */
+/* $Id: gamestatus.hpp 14736 2006-12-19 10:37:39Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -45,58 +45,18 @@ struct time_of_day
 	int red, green, blue;
 };
 
-//class which contains the global status of the game -- namely
-//the current turn, the number of turns, and the time of day.
-class gamestatus
-{
-public:
-	gamestatus(const config& time_cfg, int num_turns);
-	void write(config& cfg) const;
-
-	time_of_day get_time_of_day() const;
-	time_of_day get_previous_time_of_day() const;
-	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc) const;
-	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc, int n_turn) const;
-	size_t turn() const;
-	int number_of_turns() const;
-	void modify_turns(const std::string& mod);
-	void add_turns(int num);
-
-	//function to move to the next turn. Returns true iff time
-	//has expired.
-	bool next_turn();
-
-private:
-	time_of_day get_time_of_day_turn(int nturn) const;
-
-	std::vector<time_of_day> times_;
-
-	struct area_time_of_day {
-		std::string xsrc, ysrc;
-		std::vector<time_of_day> times;
-		std::set<gamemap::location> hexes;
-	};
-
-	std::vector<area_time_of_day> areas_;
-
-	size_t turn_;
-	int numTurns_;
-};
-
 /** Information on a particular player of the game. */
 struct player_info
 {
 	player_info():gold(-1) {}
 
+	std::string name; /** < to sore the current_player name */
 	int gold; /** < amount of gold the player has saved */
 	std::vector<unit> available_units; /** < units the player may recall */
 
 	std::set<std::string> can_recruit; /** < units the player has the ability to recruit */
 };
 
-//object which holds all the data needed to start a scenario.
-//i.e. this is the object serialized to disk when saving/loading a game.
-//is also the object which needs to be created to start a new game
 struct game_state : public variable_set
 {
 	game_state() : difficulty("NORMAL") {}
@@ -148,6 +108,48 @@ private:
 			t_string** varout, config** cfgout);
 };
 
+//class which contains the global status of the game -- namely
+//the current turn, the number of turns, and the time of day.
+class gamestatus
+{
+public:
+        gamestatus(const config& time_cfg, int num_turns, game_state* s_o_g = 0);
+	void write(config& cfg) const;
+
+	time_of_day get_time_of_day() const;
+	time_of_day get_previous_time_of_day() const;
+	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc) const;
+	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc, int n_turn) const;
+	size_t turn() const;
+	int number_of_turns() const;
+	void modify_turns(const std::string& mod);
+	void add_turns(int num);
+
+	//function to move to the next turn. Returns true iff time
+	//has expired.
+	bool next_turn();
+
+private:
+	time_of_day get_time_of_day_turn(int nturn) const;
+
+	std::vector<time_of_day> times_;
+
+	struct area_time_of_day {
+		std::string xsrc, ysrc;
+		std::vector<time_of_day> times;
+		std::set<gamemap::location> hexes;
+	};
+
+	std::vector<area_time_of_day> areas_;
+
+	size_t turn_;
+	int numTurns_;
+};
+
+//object which holds all the data needed to start a scenario.
+//i.e. this is the object serialized to disk when saving/loading a game.
+//is also the object which needs to be created to start a new game
+
 struct save_info {
 	save_info(const std::string& n, time_t t) : name(n), time_modified(t) {}
 	std::string name;
