@@ -1,4 +1,4 @@
-/* $Id: gettext.hpp 8760 2005-11-06 19:55:06Z ott $ */
+/* $Id: gettext.hpp 8646 2005-10-24 21:03:08Z ydirson $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -21,10 +21,6 @@
 const char* egettext(const char*);
 const char* sgettext(const char*);
 const char* dsgettext(const char * domainname, const char *msgid);
-/* FIXME: workaround, dsgettext2 is dsgettext but not processed by xgettext */
-/* better is the approach in trunk of refactoring code to not use */
-/* multiple text domains in one file */
-#define dsgettext2 dsgettext
 
 #ifdef GETTEXT_DOMAIN
 # define _(String) dgettext(GETTEXT_DOMAIN,String)
@@ -32,6 +28,7 @@ const char* dsgettext(const char * domainname, const char *msgid);
 #  undef gettext
 # endif
 # define gettext(String) dgettext(GETTEXT_DOMAIN,String)
+# define sgettext(String) dsgettext(GETTEXT_DOMAIN,String)
 #else
 # define _(String) gettext(String)
 #endif
