@@ -1,4 +1,4 @@
-/* $Id: halo.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: halo.hpp 11362 2006-05-08 18:19:31Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -39,11 +39,12 @@ private:
 	bool old;
 };
 
-enum ORIENTATION { NORMAL, REVERSE };
+enum ORIENTATION { NORMAL, HREVERSE, VREVERSE, HVREVERSE };
 
 ///function to add a haloing effect using 'image'
 ///centered on (x,y)
 ///returns the handle to the halo object
+///0 is the invalid handle
 int add(int x, int y, const std::string& image, ORIENTATION orientation=NORMAL, int lifetime_cycles=-1);
 
 ///function to set the position of an existing haloing
