@@ -2650,11 +2650,17 @@ std::vector<std::string> split_in_width(const std::string &s, const int font_siz
 		const unsigned width)
 {
 	std::vector<std::string> res;
+	try {
 	const std::string& first_line = font::word_wrap_text(s, font_size, width, -1, 1);
 	res.push_back(first_line);
 	if(s.size() > first_line.size()) {
 		res.push_back(s.substr(first_line.size()));
 	}
+	}
+	catch (utils::invalid_utf8_exception e)
+	{
+		throw parse_error ("");
+	}
 
 	return res;
 }
