@@ -1,4 +1,4 @@
-/* $Id: image.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: image.cpp 12681 2006-07-21 05:40:01Z sapient $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -30,21 +30,17 @@
 #include <map>
 #include <string>
 
-#define LOG_DP LOG_STREAM(info, display)
 #define ERR_DP LOG_STREAM(err, display)
 
 namespace {
 
-typedef std::map<gamemap::TERRAIN, surface> mini_terrain_cache_map;
-mini_terrain_cache_map mini_terrain_cache;
-
 typedef std::map<image::locator::value, int> locator_finder_t;
 typedef std::pair<image::locator::value, int> locator_finder_pair;
 locator_finder_t locator_finder;
 
 // Definition of all image maps
 image::image_cache images_,scaled_images_,unmasked_images_,greyed_images_;
-image::image_cache brightened_images_,semi_brightened_images_;
+image::image_cache brightened_images_,semi_brightened_images_,darkened_images_;
 
 image::locator_cache alternative_images_;
 
@@ -58,8 +54,6 @@ int red_adjust = 0, green_adjust = 0, blue_adjust = 0;
 
 std::string image_mask;
 
-SDL_PixelFormat* pixel_format = NULL;
-
 #ifdef USE_TINY_GUI
 const int tile_size = 36;
 #else
@@ -86,12 +80,15 @@ void reset_cache(std::vector<image::cache_item<T> >& cache)
 
 namespace image {
 
+mini_terrain_cache_map mini_terrain_cache;
+
 void flush_cache()
 {
 	reset_cache(images_);
 	reset_cache(scaled_images_);
 	reset_cache(unmasked_images_);
 	reset_cache(greyed_images_);
+	reset_cache(darkened_images_);
 	reset_cache(brightened_images_);
 	reset_cache(semi_brightened_images_);
 	reset_cache(alternative_images_);
@@ -113,6 +110,7 @@ void locator::init_index()
 		scaled_images_.push_back(cache_item<surface>());
 		unmasked_images_.push_back(cache_item<surface>());
 		greyed_images_.push_back(cache_item<surface>());
+		darkened_images_.push_back(cache_item<surface>());
 		brightened_images_.push_back(cache_item<surface>());
 		semi_brightened_images_.push_back(cache_item<surface>());
 
@@ -121,6 +119,41 @@ void locator::init_index()
 		index_ = i->second;
 	}
 }
+void locator::get_tc_info(const std::string& field)
+{
+	// field should be of the format "team,team_color_id"
+	size_t comma = field.find(',');
+	if(comma == std::string::npos) {
+		return;
+	}
+	int team_n = lexical_cast_default<int>(field.substr(0,comma));
+	std::string color_id = field.substr(comma+1);
+	val_.new_color = team::get_side_color_range(team_n);
+	val_.swap_colors = game_config::tc_info(color_id);
+	val_.type_ = SUB_FILE;
+}
+void locator::parse_arguments()
+{
+	std::string& fn = val_.filename_;
+	if(fn.empty()) {
+		return;
+	}
+	size_t markup_field = fn.find('~');
+	while(markup_field != std::string::npos) {
+		size_t open_field = fn.find('(',markup_field);
+		size_t close_field = fn.find(')',markup_field);
+		if(open_field == std::string::npos || close_field == std::string::npos) {
+			return;
+		}
+		std::string field = fn.substr(open_field+1,close_field-open_field-1);
+		std::string function = fn.substr(markup_field+1,open_field-markup_field-1);
+		fn = fn.substr(0,markup_field) + fn.substr(close_field+1);
+		if(function == "TC") {
+			get_tc_info(field);
+		}
+		markup_field = fn.find('~');
+	}
+}
 
 locator::locator() :
 	index_(-1)
@@ -135,17 +168,31 @@ locator::locator(const locator &a):
 locator::locator(const char *filename) :
 	val_(filename)
 {
+	parse_arguments();
 	init_index();
 }
 
 locator::locator(const std::string &filename) :
 	val_(filename)
 {
+	parse_arguments();
 	init_index();
 }
 
-locator::locator(const std::string &filename, const gamemap::location &loc) :
-	val_(filename, loc)
+locator::locator(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+	val_(filename, new_rgb, swap_rgb)
+{
+	init_index();
+}
+
+locator::locator(const std::string &filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+	val_(filename, new_rgb, swap_rgb)
+{
+	init_index();
+}
+
+locator::locator(const std::string &filename, const gamemap::location &loc, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+	val_(filename, loc, new_rgb, swap_rgb)
 {
 	init_index();
 }
@@ -159,7 +206,8 @@ locator& locator::operator=(const locator &a)
 }
 
 locator::value::value(const locator::value& a) :
-	type_(a.type_), filename_(a.filename_), loc_(a.loc_)
+  type_(a.type_), filename_(a.filename_), loc_(a.loc_),
+  new_color(a.new_color), swap_colors(a.swap_colors)
 {
 }
 
@@ -168,17 +216,28 @@ locator::value::value() :
 {}
 
 locator::value::value(const char *filename) :
-	type_(FILE), filename_(filename)
+  type_(FILE), filename_(filename)
+{
+}
+
+
+locator::value::value(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+  type_(SUB_FILE), filename_(filename), new_color(new_rgb), swap_colors(swap_rgb)
 {
 }
 
 locator::value::value(const std::string& filename) :
-	type_(FILE), filename_(filename)
+  type_(FILE), filename_(filename)
 {
 }
 
-locator::value::value(const std::string& filename, const gamemap::location& loc) :
-	type_(SUB_FILE), filename_(filename), loc_(loc)
+locator::value::value(const std::string& filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+  type_(SUB_FILE), filename_(filename), new_color(new_rgb), swap_colors(swap_rgb)
+{
+}
+
+locator::value::value(const std::string& filename, const gamemap::location& loc, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
+  type_(SUB_FILE), filename_(filename), loc_(loc), new_color(new_rgb), swap_colors(swap_rgb)
 {
 }
 
@@ -189,7 +248,7 @@ bool locator::value::operator==(const value& a) const
 	} else if(type_ == FILE) {
 		return filename_ == a.filename_;
 	} else if(type_ == SUB_FILE) {
-		return filename_ == a.filename_ && loc_ == a.loc_;
+	  return filename_ == a.filename_ && loc_ == a.loc_ && new_color == a.new_color; //note not checking swap_colors purposely
 	} else {
 		return false;
 	}
@@ -204,8 +263,9 @@ bool locator::value::operator<(const value& a) const
 	} else if(type_ == SUB_FILE) {
 		if(filename_ != a.filename_)
 			return filename_ < a.filename_;
-
-		return loc_ < a.loc_;
+		if(loc_ != a.loc_)
+		        return loc_ < a.loc_;
+		return(new_color < a.new_color);
 	} else {
 		return false;
 	}
@@ -215,22 +275,34 @@ surface locator::load_image_file() const
 {
 	surface res;
 
-	std::string const &location = get_binary_file_location("images", val_.filename_);
-	if (!location.empty()) {
-#ifdef USE_ZIPIOS
-		std::string const &s = read_file(location);
-		if (!s.empty()) {
-			SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
-			res = IMG_Load_RW(ops, 0);
-			SDL_FreeRW(ops);
-		}
-#else
-		res = IMG_Load(location.c_str());
-#endif
-	}
+	std::string location = get_binary_file_location("images", val_.filename_);
 
-	if (res.null())
+	bool try_units = false;
+
+	do {
+		if (!location.empty()) {
+#ifdef USE_ZIPIOS
+			std::string const &s = read_file(location);
+			if (!s.empty()) {
+				SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
+				res = IMG_Load_RW(ops, 0);
+				SDL_FreeRW(ops);
+			}
+#else
+			res = IMG_Load(location.c_str());
+#endif
+		}
+		if (res.null() && (!try_units)) {
+			try_units = true;
+			location = get_binary_file_location("images", "units/" + val_.filename_);
+		} else {
+			try_units = false;
+		}
+	} while (try_units);
+
+	if (res.null()) {
 		ERR_DP << "could not open image '" << val_.filename_ << "'\n";
+	}
 
 	return res;
 }
@@ -245,15 +317,20 @@ surface locator::load_image_sub_file() const
 	if(mask == NULL)
 		return surface(NULL);
 
-	SDL_Rect srcrect = {
-		((tile_size*3) / 4) * val_.loc_.x,
-		tile_size * val_.loc_.y + (tile_size/2) * (val_.loc_.x % 2),
-		tile_size, tile_size
-	};
-
-	surface tmp(cut_surface(mother_surface, srcrect));
-	surface surf(mask_surface(tmp, mask));
+	surface surf=mother_surface;
+	if(val_.loc_.x>-1 && val_.loc_.y>-1){
+	  SDL_Rect srcrect = {
+	    ((tile_size*3) / 4) * val_.loc_.x,
+	    tile_size * val_.loc_.y + (tile_size/2) * (val_.loc_.x % 2),
+	    tile_size, tile_size
+	  };
 
+	  surface tmp(cut_surface(mother_surface, srcrect));
+	  surf=mask_surface(tmp, mask);
+	}
+	if(val_.swap_colors.size()){
+	  surf=recolor_image(surf,get_new_color(),get_swap_colors());
+	}
 	return surf;
 }
 
@@ -359,6 +436,8 @@ void set_wm_icon()
 #endif
 }
 
+SDL_PixelFormat* pixel_format = NULL;
+
 void set_pixel_format(SDL_PixelFormat* format)
 {
 	pixel_format = format;
@@ -373,6 +452,7 @@ void set_colour_adjustment(int r, int g, int b)
 		blue_adjust = b;
 		reset_cache(scaled_images_);
 		reset_cache(greyed_images_);
+		reset_cache(darkened_images_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
 		reset_cache(alternative_images_);
@@ -403,6 +483,7 @@ void set_image_mask(const std::string& image)
 		image_mask = image;
 		reset_cache(scaled_images_);
 		reset_cache(greyed_images_);
+		reset_cache(darkened_images_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
 		reset_cache(alternative_images_);
@@ -417,6 +498,7 @@ void set_zoom(int amount)
 		zoom = amount;
 		reset_cache(scaled_images_);
 		reset_cache(greyed_images_);
+		reset_cache(darkened_images_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
 		reset_cache(unmasked_images_);
@@ -472,6 +554,13 @@ surface get_greyed(const locator i_locator, COLOUR_ADJUSTMENT adj)
 	return surface(greyscale_image(image));
 }
 
+surface get_darkened(const locator i_locator, COLOUR_ADJUSTMENT adj)
+{
+	surface image(get_image(i_locator, SCALED, adj));
+
+	return surface(darken_image(image));
+}
+
 surface get_brightened(const locator i_locator, COLOUR_ADJUSTMENT adj)
 {
 	surface image(get_image(i_locator, SCALED, adj));
@@ -505,6 +594,9 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 	case GREYED:
 		imap = &greyed_images_;
 		break;
+	case DARKENED:
+		imap = &darkened_images_;
+		break;
 	case BRIGHTENED:
 		imap = &brightened_images_;
 		break;
@@ -541,6 +633,9 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 		case GREYED:
 			res = get_greyed(i_locator, adj);
 			break;
+		case DARKENED:
+			res = get_darkened(i_locator, adj);
+			break;
 		case BRIGHTENED:
 			res = get_brightened(i_locator, adj);
 			break;
@@ -613,7 +708,7 @@ locator get_alternative(const image::locator &i_locator, const std::string &alt)
 		res = locator(alternative);
 		break;
 	case locator::SUB_FILE:
-		res = locator(alternative, i_locator.get_loc());
+		res = locator(alternative, i_locator.get_loc(), i_locator.get_new_color(), i_locator.get_swap_colors());
 		break;
 	default:
 		wassert(false);
@@ -647,83 +742,5 @@ bool exists(const image::locator& i_locator)
 	return cache;
 }
 
-surface getMinimap(int w, int h, const gamemap& map, const team* tm)
-{
-	const int scale = 8;
-
-	LOG_DP << "creating minimap " << int(map.x()*scale*0.75) << "," << int(map.y()*scale) << "\n";
-
-	const size_t map_width = map.x()*scale*3/4;
-	const size_t map_height = map.y()*scale;
-	if(map_width == 0 || map_height == 0) {
-		return surface(NULL);
-	}
-
-	surface minimap(SDL_CreateRGBSurface(SDL_SWSURFACE,
-	                               map_width,map_height,
-	                               pixel_format->BitsPerPixel,
-	                               pixel_format->Rmask,
-	                               pixel_format->Gmask,
-	                               pixel_format->Bmask,
-	                               pixel_format->Amask));
-	if(minimap == NULL)
-		return surface(NULL);
-
-	typedef mini_terrain_cache_map cache_map;
-	cache_map& cache = mini_terrain_cache;
-
-	for(int y = 0; y != map.y(); ++y) {
-		for(int x = 0; x != map.x(); ++x) {
-
-			surface surf(NULL);
-
-			const gamemap::location loc(x,y);
-			if(map.on_board(loc)) {
-				const bool shrouded = tm != NULL && tm->shrouded(x,y);
-				const bool fogged = tm != NULL && tm->fogged(x,y) && !shrouded;
-				const gamemap::TERRAIN terrain = shrouded ? gamemap::VOID_TERRAIN : map[x][y];
-				cache_map::iterator i = cache.find(terrain);
-
-				if(i == cache.end()) {
-					surface tile(get_image("terrain/" + map.get_terrain_info(terrain).symbol_image() + ".png",image::UNSCALED));
-
-					if(tile == NULL) {
-						ERR_DP << "could not get image for terrrain '"
-						          << terrain << "'\n";
-						continue;
-					}
-
-					surf = surface(scale_surface_blended(tile,scale,scale));
-
-					if(surf == NULL) {
-						continue;
-					}
-
-					i = cache.insert(cache_map::value_type(terrain,surf)).first;
-				} else {
-					surf = surface(i->second);
-				}
-
-				if(fogged) {
-					surf = surface(adjust_surface_colour(surf,-50,-50,-50));
-				}
-
-				wassert(surf != NULL);
-
-				SDL_Rect maprect = {x*scale*3/4,y*scale + (is_odd(x) ? scale/2 : 0),0,0};
-				SDL_BlitSurface(surf, NULL, minimap, &maprect);
-			}
-		}
-	}
-
-	if((minimap->w != w || minimap->h != h) && w != 0) {
-		const surface surf(minimap);
-		minimap = surface(scale_surface(surf,w,h));
-	}
-
-	LOG_DP << "done generating minimap\n";
-
-	return minimap;
-}
 
 }
