@@ -1,4 +1,4 @@
-/* $Id: language.cpp 8326 2005-09-19 19:38:01Z ott $ */
+/* $Id: language.cpp 12503 2006-07-11 20:57:03Z noyga $ */
 /* vim:set encoding=utf-8: */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
@@ -23,6 +23,7 @@
 #include "util.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
+#include "wassert.hpp"
 #include "wesconfig.h"
 
 #include <algorithm>
@@ -37,54 +38,29 @@ namespace {
 	string_map strings_;
 }
 
-language_def known_languages[] = {
-	language_def("", N_("System default language")),
-	language_def("af_ZA", "Afrikaans"),
-	//language_def("bg_BG", "Български (Bulgarski)"),
-	language_def("ca_ES", "Català"),
-	language_def("cs_CZ", "Čeština"),
-	//language_def("da_DK", "Dansk"),
-	language_def("de_DE", "Deutsch"),
-	//language_def("el_GR", "Ελληνικά (Ellinika)"),
-	//language_def("et_EE", "Eesti"),
-	language_def("en_GB", "English [GB]"),
-	language_def("C",     "English [US]"),
-	language_def("es_ES", "Español"),
-	//language_def("eu_ES", "Euskera"),
-	language_def("fr_FR", "Français"),
-	language_def("it_IT", "Italiano"),
-	language_def("la_IT", "Latina"),
-	language_def("hu_HU", "Magyar"),
-	language_def("nl_NL", "Nederlands"),
-	language_def("ja_JP", "日本語 (Nihongo)"),
-	//language_def("no_NO", "Norsk"),
-        //language_def("oc_FR", "Occitan"),
-	//language_def("pl_PL", "Polski"),
-	//language_def("pt_BR", "Português do Brasil"),
-	language_def("ru_RU", "Русский (Russkij)"),
-	language_def("sk_SK", "Slovenčina"),
-	//language_def("sl_SI", "Slovenščina"),
-	//language_def("sr_CS", "Srpski"),
-	language_def("tr_TR", "Türkçe"),
-	language_def("fi_FI", "Suomi"),
-	language_def("sv_SE", "Svenska"),
-	//language_def("zh_CN", "中文 (Zhongwen)"),
-
-	// end of list marker, do not remove
-	language_def("", "")
-};
+std::vector<language_def> known_languages;
 
 std::string languagedef_name (const language_def& def)
 {
 	return def.language;
 }
 
+bool languagedef_rtl (const language_def& def)
+{
+	return def.rtl;
+}
+
+bool current_language_rtl()
+{
+	return get_language().rtl;
+}
+
 bool languagedef_lessthan_p (const language_def& def1, const language_def& def2)
 {
 	return (def1.language < def2.language);
 }
 
-bool language_def::operator== (const language_def& a)
+bool language_def::operator== (const language_def& a) const
 {
 	return ((language == a.language) /* && (localename == a.localename) */ );
 }
@@ -112,15 +88,32 @@ const t_string& symbol_table::operator[](const char* key) const
 	return (*this)[std::string(key)];
 }
 
-std::vector<language_def> get_languages()
+bool load_language_list()
 {
-	std::vector<language_def> res;
-
-	for(int i = 0; known_languages[i].language[0] != '\0'; i++) {
-		res.push_back(known_languages[i]);
+	config cfg;
+	try {
+		scoped_istream stream = preprocess_file("data/language.cfg");
+		read(cfg, *stream);
+	} catch(config::error &) {
+		return false;
 	}
 
-	return res;
+	known_languages.clear();
+	known_languages.push_back(
+		language_def("", t_string(N_("System default language"), "wesnoth"), "ltr"));
+
+	config::const_child_itors langs = cfg.child_range("locale");
+	for(;langs.first != langs.second; ++langs.first) {
+		known_languages.push_back(
+			language_def((**langs.first)["locale"], (**langs.first)["name"], (**langs.first)["dir"]));
+	}
+
+	return true;
+}
+
+std::vector<language_def> get_languages()
+{
+	return known_languages;
 }
 
 static void wesnoth_setlocale(int category, std::string const &slocale)
@@ -130,10 +123,14 @@ static void wesnoth_setlocale(int category, std::string const &slocale)
 	// use that value, so someone with es would get the game in Spanish
 	// instead of en_US the first time round
 	// LANGUAGE overrides other settings, so for now just get rid of it
-	// does Windows have unsetenv? if so, can make this unconditional
+	// FIXME: add configure check for unsetenv
 #ifndef _WIN32
+#ifndef SOLARIS
+#ifndef __AMIGAOS4__
 	unsetenv ("LANGUAGE"); // void so no return value to check
 #endif
+#endif
+#endif
 
 #ifdef __BEOS__
 	if(setenv ("LANG", locale, 1) == -1)
@@ -194,11 +191,10 @@ bool set_language(const language_def& locale)
 
 	current_language = locale;
 	wesnoth_setlocale(LC_MESSAGES, locale.localename);
-	known_languages[0].language = gettext("System default language");
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
-		scoped_istream stream = preprocess_file("data/translations/english.cfg");
+		scoped_istream stream = preprocess_file("data/english.cfg");
 		read(cfg, *stream);
 	} catch(config::error& e) {
 		std::cerr << "Could not read english.cfg\n";
@@ -216,6 +212,13 @@ bool set_language(const language_def& locale)
 	}
 	// end of string_table fill
 
+	// Reset translations for the name of current languages
+	for (std::vector<language_def>::iterator itor = known_languages.begin();
+			itor != known_languages.end(); ++itor) {
+
+		itor->language.reset_translation();
+	}
+
 	return true;
 }
 
@@ -225,12 +228,15 @@ const language_def& get_locale()
 {
 	//TODO: Add in support for querying the locale on Windows
 
+	wassert(known_languages.size() != 0);
+
 	const std::string& prefs_locale = preferences::language();
 	if(prefs_locale.empty() == false) {
 		wesnoth_setlocale(LC_MESSAGES, prefs_locale);
-		for(int i = 0; known_languages[i].language[0] != '\0'; i++) {
-			if (prefs_locale == known_languages[i].localename)
-				return known_languages[i];
+		for(std::vector<language_def>::const_iterator i = known_languages.begin();
+				i != known_languages.end(); ++i) {
+			if (prefs_locale == i->localename)
+				return *i;
 		}
 		std::cerr << "locale not found in known array; defaulting to system locale\n";
 		return known_languages[0];
