@@ -1,4 +1,4 @@
-/* $Id: marked-up_text.cpp 10082 2006-02-13 00:48:47Z darthfool $ */
+/* $Id: marked-up_text.cpp 13726 2006-09-28 10:49:42Z edb $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -317,7 +317,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 		if(start_of_line) {
 			line_width = 0;
 			format_string = "";
-			while(ch != end && *ch < (wchar_t)0x100 && is_format_char(*ch)) {
+			while(ch != end && *ch < (wchar_t)0x100 && is_format_char(*ch) && !ch.next_is_end()) {
 				format_string.append(ch.substr().first, ch.substr().second);
 				++ch;
 			}
