@@ -112,9 +112,9 @@ namespace events{
 		teams_[team_num - 1].reset_objectives_changed();
 	}
 
-	void menu_handler::show_statistics()
+	void menu_handler::show_statistics(const unsigned int team_num)
 	{
-		const statistics::stats& stats = statistics::calculate_stats(0,gui_->viewing_team()+1);
+		const statistics::stats& stats = statistics::calculate_stats(0, team_num);
 		std::vector<std::string> items;
 
 		{
@@ -338,7 +338,7 @@ namespace events{
 			//cause it to be displayed in the correct colour
 			if(leader != units_.end()) {
 				str << IMAGE_PREFIX << leader->second.absolute_image() << "~TC(" << (n+1) << "," << leader->second.team_color() << ")" << COLUMN_SEPARATOR
-					<< "\033[3" << lexical_cast<char, size_t>(n+1) << 'm' << leader->second.description() << COLUMN_SEPARATOR;
+					<< "\033[3" << lexical_cast<char, size_t>(n+1) << 'm' << teams_[n].current_player() /*leader->second.description()*/ << COLUMN_SEPARATOR;
 			} else {
 				str << ' ' << COLUMN_SEPARATOR << "\033[3" << lexical_cast<char, size_t>(n+1) << "m-" << COLUMN_SEPARATOR;
 			}
@@ -840,9 +840,7 @@ namespace events{
 
 			action.starting_moves = u->second.movement_left();
 
-			u->second.set_hidden(true);
 			unit_display::move_unit(*gui_,map_,route,u->second,units_,teams_);
-			u->second.set_hidden(false);
 			std::pair<gamemap::location,unit> *up = units_.extract(u->first);
 			up->second.set_goto(gamemap::location());
 			up->second.set_movement(starting_moves);
@@ -963,9 +961,7 @@ namespace events{
 
 			action.starting_moves = u->second.movement_left();
 
-			u->second.set_hidden(true);
 			unit_display::move_unit(*gui_,map_,route,u->second,units_,teams_);
-			u->second.set_hidden(false);
 			std::pair<gamemap::location,unit> *up = units_.extract(u->first);
 			up->second.set_goto(gamemap::location());
 			up->second.set_movement(starting_moves);
@@ -1314,7 +1310,7 @@ namespace events{
 
 	void menu_handler::add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type)
 	{
-		gui_->add_chat_message(speaker,side,message,type);
+		gui_->add_chat_message(speaker,side,message,type,false);
 	}
 
 	chat_handler::~chat_handler()
