@@ -677,11 +677,17 @@ undo_stack_(undo_stack), redo_stack_(redo_stack)
 	path_turns_ = 0;
 	undo_ = false;
 	show_menu_ = false;
+	over_route_ = false;
+	team_num_ = 1;
 }
 
-void mouse_handler::mouse_motion(const SDL_MouseMotionEvent& event, const int player_number, const bool browse)
+void mouse_handler::set_team(const int team_number)
+{
+	team_num_ = team_number;
+}
+
+void mouse_handler::mouse_motion(const SDL_MouseMotionEvent& event, const bool browse)
 {
-	team_num_ = player_number;
 	mouse_motion(event.x,event.y, browse);
 }
 
@@ -748,6 +754,10 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse)
 			enemy_paths_ = false;
 			current_paths_ = paths();
 			gui_->unhighlight_reach();
+		} else if(over_route_) {
+			over_route_ = false;
+			current_route_.steps.clear();
+			(*gui_).set_route(NULL);
 		}
 
 		const gamemap::location& dest = attack_from.valid() ? attack_from : new_hex;
@@ -786,16 +796,42 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse)
 
 		unit_map::iterator un = find_unit(new_hex);
 
-		if(un != units_.end() && un->second.side() != team_num_ &&
-			current_paths_.routes.empty() && !(*gui_).fogged(un->first.x,un->first.y)) {
-			unit_movement_resetter move_reset(un->second);
+		if(un != units_.end() && current_paths_.routes.empty() && !(*gui_).fogged(un->first.x,un->first.y)) {
+			if (un->second.side() != team_num_) {
+				//unit under cursor is not on our team, highlight reach
+				unit_movement_resetter move_reset(un->second);
 
-			const bool ignore_zocs = un->second.get_ability_bool("skirmisher",un->first);
-			const bool teleport = un->second.get_ability_bool("teleport",un->first);
-			current_paths_ = paths(map_,status_,gameinfo_,units_,new_hex,teams_,
-								   ignore_zocs,teleport,viewing_team(),path_turns_);
-			gui_->highlight_reach(current_paths_);
-			enemy_paths_ = true;
+				const bool ignore_zocs = un->second.get_ability_bool("skirmisher",un->first);
+				const bool teleport = un->second.get_ability_bool("teleport",un->first);
+				current_paths_ = paths(map_,status_,gameinfo_,units_,new_hex,teams_,
+									ignore_zocs,teleport,viewing_team(),path_turns_);
+				gui_->highlight_reach(current_paths_);
+				enemy_paths_ = true;
+			} else {
+				//unit is on our team, show path if the unit has one
+				unit u = un->second;
+				const gamemap::location go_to = u.get_goto();
+				if(map_.on_board(go_to)) {
+					const shortest_path_calculator calc(u,current_team(),
+									visible_units(),teams_,map_);
+
+					const std::set<gamemap::location>* teleports = NULL;
+
+					std::set<gamemap::location> allowed_teleports;
+					if(u.get_ability_bool("teleport",un->first)) {
+						allowed_teleports = vacant_villages(current_team().villages(),units_);
+						teleports = &allowed_teleports;
+						if(current_team().villages().count(un->first))
+							allowed_teleports.insert(un->first);
+
+					}
+
+					paths::route route = a_star_search(un->first, go_to, 10000.0, &calc, map_.x(), map_.y(), teleports);
+					route.move_left = route_turns_to_complete(un->second,map_,route);
+					gui_->set_route(&route);
+				}
+				over_route_ = true;
+			}
 		}
 	}
 
@@ -894,10 +930,9 @@ const unit_map& mouse_handler::visible_units()
 	return visible_units_;
 }
 
-void mouse_handler::mouse_press(const SDL_MouseButtonEvent& event, const int player_number, const bool browse)
+void mouse_handler::mouse_press(const SDL_MouseButtonEvent& event, const bool browse)
 {
 	show_menu_ = false;
-	team_num_ = player_number;
 	mouse_motion(event.x, event.y, browse);
 
 	if(is_left_click(event) && event.state == SDL_RELEASED) {
@@ -1024,6 +1059,17 @@ void mouse_handler::left_click(const SDL_MouseButtonEvent& event, const bool bro
 				// enemy = find_unit(hex);
 				if(u != units_.end() && u->second.side() == team_num_ &&
 					enemy != units_.end() && current_team().is_enemy(enemy->second.side()) && !enemy->second.incapacitated()) {
+					//if shroud or fog is active, rememember nits a and after attack check if someone isn`t seen
+					std::set<gamemap::location> known_units;
+
+					if (teams_[team_num_-1].uses_shroud() || teams_[team_num_-1].uses_fog()){
+						 for(unit_map::const_iterator u = units_.begin(); u != units_.end(); ++u) {
+				       if(teams_[team_num_-1].fogged(u->first.x,u->first.y) == false) {
+				         known_units.insert(u->first);
+				         teams_[team_num_-1].see(u->second.side()-1);
+								}
+							}
+					}
 					if(!commands_disabled && attack_enemy(u,enemy) == false) {
 						undo_ = true;
 						selected_hex_ = src;
@@ -1032,6 +1078,27 @@ void mouse_handler::left_click(const SDL_MouseButtonEvent& event, const bool bro
 						gui_->highlight_reach(current_paths_);
 						return;
 					}
+					else //attack == true
+					{
+						if (teams_[team_num_-1].uses_shroud() || teams_[team_num_-1].uses_fog()){
+							//check if some new part of map discovered or is active delay shroud updates, which need special care
+							if (clear_shroud(*gui_, status_, map_, gameinfo_, units_, teams_, team_num_ - 1)||!teams_[team_num_-1].auto_shroud_updates()){
+								clear_undo_stack();
+								//some new part of map discovered
+								for(unit_map::const_iterator u = units_.begin(); u != units_.end(); ++u) {
+						       if(teams_[team_num_-1].fogged(u->first.x,u->first.y) == false) {
+											//check if unit is not known
+											if (known_units.find(u->first)==known_units.end())
+											{
+												game_events::raise("sighted",u->first,attack_from);
+											}
+									 }
+								}
+								game_events::pump();
+								return;
+							}
+						}
+					}
 				}
 			}
 
@@ -1210,11 +1277,11 @@ bool mouse_handler::attack_enemy(unit_map::iterator attacker, unit_map::iterator
 		}
 		atts << IMAGE_PREFIX << attw.icon() << COLUMN_SEPARATOR
 			 << font::BOLD_TEXT << attw.name() << "\n" << att.damage << "-"
-			 << att.num_blows << " " << attw.range() << " (" << att.chance_to_hit << "%)\n"
+			 << att.num_blows << " " << (attw.range().empty() ? "" : gettext(attw.range().c_str())) << " (" << att.chance_to_hit << "%)\n"
 			 << attw.weapon_specials() << special_pad
 			 << COLUMN_SEPARATOR << _("vs") << COLUMN_SEPARATOR
 			 << font::BOLD_TEXT << defw.name() << "\n" << def.damage << "-"
-			 << def.num_blows << " " << defw.range() << " (" << def.chance_to_hit << "%)\n"
+			 << def.num_blows << " " << (defw.range().empty() ? "" : gettext(defw.range().c_str())) << " (" << def.chance_to_hit << "%)\n"
 			 << defw.weapon_specials() << special_pad << COLUMN_SEPARATOR
 			 << IMAGE_PREFIX << defw.icon();
 
