@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp 12675 2006-07-20 18:00:46Z xan $ */
+/* $Id: multiplayer_connect.cpp 14257 2006-11-16 17:35:59Z edb $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -62,16 +62,20 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	slider_income_(parent.video()),
 	label_gold_(parent.video(), "100", font::SIZE_SMALL, font::LOBBY_COLOUR),
 	label_income_(parent.video(), _("Normal"), font::SIZE_SMALL, font::LOBBY_COLOUR),
-	enabled_(!parent_->params_.saved_game),
-	changed_(false),
+	allow_player_(utils::string_bool(cfg_["allow_player"], true)),
+	enabled_(!parent_->params_.saved_game), changed_(false),
 	llm_(parent.era_sides_, &parent.game_data_, enabled_ ? &combo_leader_ : NULL)
 {
-	if(cfg_["allow_player"] != "no" && enabled_) {
+	if(allow_player_ && enabled_) {
 		controller_ = parent_->default_controller_;
 	} else {
-		for(size_t i = CNTR_NETWORK; i != CNTR_LAST; ++i) {
+		size_t i = CNTR_NETWORK;
+		//if player isn't allowed, network controller doesn't make sense
+		if (!allow_player_)
+			++i;
+		for(; i != CNTR_LAST; ++i) {
 			if(cfg_["controller"] == controller_names[i]) {
-				controller_ = (mp::controller)i;
+				controller_ = static_cast<mp::controller>(i);
 				break;
 			}
 		}
@@ -95,6 +99,8 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	combo_colour_.enable(enabled_);
 	slider_gold_.hide(!enabled_);
 	slider_income_.hide(!enabled_);
+	label_gold_.hide(!enabled_);
+	label_income_.hide(!enabled_);
 
 	id_ = ""; // Id is reset, and not imported from loading savegames
 	save_id_ = cfg_["save_id"];
@@ -130,7 +136,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		config::const_child_itors side_units = cfg.child_range("unit");
 		std::string leader_type;
 		for(;side_units.first != side_units.second; ++side_units.first) {
-			if((**side_units.first)["canrecruit"] == "1") {
+			if(utils::string_bool((**side_units.first)["canrecruit"], false)) {
 				leader_type = (**side_units.first)["type"];
 				break;
 			}
@@ -149,6 +155,84 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		}
 		combo_leader_.set_items(leader_name_pseudolist);
 		combo_leader_.set_selected(0);
+	} else if(parent_->params_.use_map_settings) {
+		//gold, income, team, and colour are only suggestions unless explicitly locked
+		if(utils::string_bool(cfg_["gold_lock"], false)) {
+			slider_gold_.enable(false);
+			label_gold_.enable(false);
+		}
+		if(utils::string_bool(cfg_["income_lock"], false)) {
+			slider_income_.enable(false);
+			label_income_.enable(false);
+		}
+		if(utils::string_bool(cfg_["team_lock"], false)) {
+			combo_team_.enable(false);
+		}
+		if(utils::string_bool(cfg_["colour_lock"], false)) {
+			combo_colour_.enable(false);
+		}
+
+		//set the leader
+		leader_ = cfg_["type"];
+		if(!leader_.empty()) {
+			combo_leader_.enable(false);
+			llm_.set_combo(NULL);
+			std::vector<std::string> leader_name_pseudolist;
+			game_data::unit_type_map::const_iterator leader_name = parent_->game_data_.unit_types.find(leader_);
+			if(leader_name == parent_->game_data_.unit_types.end()) {
+				leader_name_pseudolist.push_back("?");
+			} else {
+				leader_name_pseudolist.push_back(leader_name->second.language_name());
+			}
+			combo_leader_.set_items(leader_name_pseudolist);
+			combo_leader_.set_selected(0);
+		}
+
+		//try to pick a faction for the sake of appearance and for filling in the blanks
+		if(faction_ == 0) {
+			std::vector<std::string> find;
+			std::string search_field;
+			if(!cfg_["faction"].empty()) {
+				//choose based on faction
+				find.push_back(cfg_["faction"]);
+				search_field = "id";
+			} else if(!cfg_["recruit"].empty()) {
+				//choose based on recruit
+				find = utils::split(cfg_["recruit"]);
+				search_field = "recruit";
+			} else if(!leader_.empty()) {
+				//choose based on leader
+				find.push_back(leader_);
+				search_field = "leader";
+			}
+
+			std::vector<std::string>::const_iterator search = find.begin();
+			while(search != find.end()) {
+				int faction_index = 0;
+				std::vector<config*>::const_iterator faction = parent.era_sides_.begin();
+				while(faction_ == 0 && faction != parent.era_sides_.end()) {
+					const config& side = (**faction);
+					std::vector<std::string> recruit;
+					recruit = utils::split(side[search_field]);
+					for(itor = recruit.begin(); itor != recruit.end(); ++itor) {
+						if(*itor == *search) {
+							faction_ = faction_index;
+							llm_.update_leader_list(faction_);
+							combo_faction_.enable(false);
+						}
+					}
+					++faction;
+					faction_index++;
+				}
+				//exit outmost loop if we've found a faction
+				if(!combo_faction_.enabled()) {
+					break;
+				}
+				++search;
+			}
+		} else {
+			combo_faction_.enable(false);
+		}
 	}
 
 	update_ui();
@@ -168,9 +252,10 @@ connect::side::side(const side& a) :
 	combo_team_(a.combo_team_), combo_colour_(a.combo_colour_),
 	slider_gold_(a.slider_gold_), slider_income_(a.slider_income_),
 	label_gold_(a.label_gold_), label_income_(a.label_income_),
-	enabled_(a.enabled_), changed_(a.changed_), llm_(a.llm_)
+	allow_player_(a.allow_player_), enabled_(a.enabled_),
+	changed_(a.changed_), llm_(a.llm_)
 {
-	llm_.set_combo(enabled_ ? &combo_leader_ : NULL);
+	llm_.set_combo((enabled_ && leader_.empty()) ? &combo_leader_ : NULL);
 }
 
 void connect::side::add_widgets_to_scrollpane(gui::scrollpane& pane, int pos)
@@ -286,7 +371,12 @@ bool connect::side::changed()
 
 bool connect::side::available() const
 {
-	return controller_ == CNTR_NETWORK && id_.empty();
+	return allow_player_ && controller_ == CNTR_NETWORK && id_.empty();
+}
+
+bool connect::side::allow_player() const
+{
+	return allow_player_;
 }
 
 void connect::side::update_controller_ui()
@@ -369,7 +459,7 @@ void connect::side::update_ui()
 
 config connect::side::get_config() const
 {
-	config res = cfg_;
+	config res(cfg_);
 
 	// If the user is allowed to change type, faction, leader etc, then
 	// import their new values in the config.
@@ -382,6 +472,7 @@ config connect::side::get_config() const
 	}
 	res["controller"] = controller_names[controller_];
 	res["description"] = id_;
+	res["current_player"] = id_;
 
 	if (id_.empty()) {
 		char const *description;
@@ -481,6 +572,23 @@ config connect::side::get_config() const
 		res["allow_changes"] = "no";
 	}
 
+	if(parent_->params_.use_map_settings && enabled_) {
+		config trimmed = cfg_;
+		trimmed["side"] = "";
+		trimmed["controller"] = "";
+		trimmed["description"] = "";
+		trimmed["team_name"] = "";
+		trimmed["colour"] = "";
+		trimmed["gold"] = "";
+		trimmed["income"] = "";
+		trimmed["allow_changes"] = "";
+		if(controller_ != CNTR_COMPUTER) {
+			//only override names for computer controlled players
+			trimmed["user_description"] = "";
+		}
+		trimmed.prune();
+		return res.merge_with(trimmed);
+	}
 	return res;
 }
 
@@ -545,12 +653,16 @@ void connect::side::import_network_user(const config& data)
 	id_ = data["name"];
 	controller_ = CNTR_NETWORK;
 
-	if (enabled_ && !parent_->era_sides_.empty()) {
-		faction_ = lexical_cast_default<int>(data["faction"], 0);
-		if (faction_ > int(parent_->era_sides_.size()))
-			faction_ = 0;
-		llm_.update_leader_list(faction_);
-		llm_.set_leader(data["leader"]);
+	if(enabled_ && !parent_->era_sides_.empty()) {
+		if(combo_faction_.enabled()) {
+			faction_ = lexical_cast_default<int>(data["faction"], 0);
+			if(faction_ > int(parent_->era_sides_.size()))
+				faction_ = 0;
+			llm_.update_leader_list(faction_);
+		}
+		if(combo_leader_.enabled()) {
+			llm_.set_leader(data["leader"]);
+		}
 	}
 
 	update_ui();
@@ -562,8 +674,10 @@ void connect::side::reset(mp::controller controller)
 	controller_ = controller;
 
 	if(enabled_ && !parent_->era_sides_.empty()) {
-		faction_ = 0;
-		llm_.update_leader_list(0);
+		if(combo_faction_.enabled())
+			faction_ = 0;
+		if(combo_leader_.enabled())
+			llm_.update_leader_list(0);
 	}
 
 	update_ui();
@@ -574,7 +688,7 @@ void connect::side::resolve_random()
 	if(!enabled_ || parent_->era_sides_.empty())
 		return;
 
-	if((*parent_->era_sides_[faction_])["random_faction"] == "yes") {
+	if(utils::string_bool((*parent_->era_sides_[faction_])["random_faction"], false)) {
 
 		// Builds the list of sides which aren't random
 		std::vector<int> nonrandom_sides;
@@ -670,13 +784,21 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 		throw config::error(_("The scenario is invalid because it has no sides."));
 	}
 
-	int side_choice = 0;
+	//take the first available side or available side with id == login
+	int side_choice = -1;
 	for(side_list::const_iterator s = sides_.begin(); s != sides_.end(); ++s) {
-		if(s->get_save_id() == preferences::login()) {
-			side_choice = s - sides_.begin();
+		if (s->allow_player()) {
+			if (side_choice == -1)
+				side_choice = s - sides_.begin();
+			if(s->get_save_id() == preferences::login()) {
+				side_choice = s - sides_.begin();
+				break;
+			}
 		}
 	}
-	sides_[side_choice].set_id(preferences::login());
+
+	if (side_choice != -1)
+		sides_[side_choice].set_id(preferences::login());
 
 	update_playerlist_state();
 
@@ -773,8 +895,10 @@ void connect::hide_children(bool hide)
 	faction_title_label_.hide(hide);
 	team_title_label_.hide(hide);
 	colour_title_label_.hide(hide);
-	gold_title_label_.hide(hide);
-	income_title_label_.hide(hide);
+	if (!params_.saved_game) {
+		gold_title_label_.hide(hide);
+		income_title_label_.hide(hide);
+	}
 
 	ai_.hide(hide);
 	launch_.hide(hide);
@@ -882,6 +1006,16 @@ void connect::process_network_data(const config& data, const network::connection
 		}
 	}
 
+	const config* change_faction = data.child("change_faction");
+	if(change_faction != NULL) {
+		int side_taken = find_player_side(change_faction->get_attribute("name"));
+		if(side_taken != -1) {
+			sides_[side_taken].import_network_user(*change_faction);
+			update_playerlist_state();
+			update_and_send_diff();
+		}
+	}
+
 	if(data.child("observer") != NULL) {
 		const t_string& observer_name = data.child("observer")->get_attribute("name");
 		if(!observer_name.empty()) {
@@ -1082,7 +1216,7 @@ void connect::lists_init()
 	for(side_list::iterator s = sides_.begin(); s != sides_.end(); ++s) {
 		const int side_num = s - sides_.begin();
 		const int spos = 60 * (side_num-offset);
-		if(s->get_config()["allow_player"] == "no") {
+		if(!utils::string_bool(s->get_config()["allow_player"], true)) {
 			offset++;
 			continue;
 		}
@@ -1173,6 +1307,9 @@ void connect::load_game()
 		level_.add_child("era", *era_cfg);
 		level_["experience_modifier"] = lexical_cast<std::string>(params_.xp_modifier);
 	}
+	gold_title_label_.hide(params_.saved_game);
+	income_title_label_.hide(params_.saved_game);
+
 	level_["mp_village_gold"] = lexical_cast<std::string>(params_.village_gold);
 	level_["mp_use_map_settings"] = params_.use_map_settings ? "yes" : "no";
 	level_["mp_fog"] = params_.fog_game ? "yes" : "no";
@@ -1300,5 +1437,3 @@ void connect::kick_player(const std::string& name)
 }
 
 }
-
-
