@@ -1,4 +1,4 @@
-/* $Id: multiplayer_wait.cpp 12673 2006-07-20 17:44:21Z xan $ */
+/* $Id: multiplayer_wait.cpp 14131 2006-11-01 12:38:03Z rusty $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -196,7 +196,7 @@ void wait::join_game(bool observe)
 		int side_choice = 0;
 		for(config::child_list::const_iterator s = sides_list.begin(); s != sides_list.end(); ++s) {
 			if((**s)["controller"] == "network" && (**s)["description"].empty()) {
-				if((**s)["save_id"] == preferences::login()) {
+				if((**s)["save_id"] == preferences::login() || (**s)["current_player"] == preferences::login()) {
 					side_choice = s - sides_list.begin();
 				}
 			}
@@ -209,6 +209,14 @@ void wait::join_game(bool observe)
 		std::string leader_choice;
 		size_t faction_choice = 0;
 
+		//we take the side now so noone will take it while we choose a faction
+		config cfg;
+		cfg["side"] = lexical_cast<std::string>(side_choice + 1);
+		cfg["name"] = preferences::login();
+		cfg["faction"] = "random";
+		cfg["leader"] = "random";
+		network::send_data(cfg);
+
 		if(allow_changes) {
 			events::event_context context;
 
@@ -245,15 +253,15 @@ void wait::join_game(bool observe)
 			leader_choice = leader_selector.get_selected_leader();
 
 			wassert(faction_choice < possible_sides.size());
+
+			config faction;
+			config& change = faction.add_child("change_faction");
+			change["name"] = preferences::login();
+			change["faction"] = lexical_cast<std::string>(faction_choice);
+			change["leader"] = leader_choice;
+			network::send_data(faction);
 		}
 
-		config response;
-		response["side"] = lexical_cast<std::string>(side_choice + 1);
-		response["name"] = preferences::login();
-		response["faction"] = lexical_cast<std::string>(faction_choice);
-		response["leader"] = leader_choice;
-
-		network::send_data(response);
 	}
 
 	generate_menu();
@@ -404,7 +412,10 @@ void wait::generate_menu()
 		// Mark parentheses translatable for languages like Japanese
 		if(!leader_name.empty())
 			str << _("(") << leader_name << _(")");
-		str << COLUMN_SEPARATOR << sd["gold"] << ' ' << sgettext("unit^Gold") << COLUMN_SEPARATOR;
+		str << COLUMN_SEPARATOR;
+		// Don't show gold for saved games
+		if(sd["allow_changes"] == "yes")
+			str << sd["gold"] << ' ' << sgettext("unit^Gold") << COLUMN_SEPARATOR;
 
 		int income_amt = lexical_cast_default<int>(sd["income"], 0);
 		if(income_amt != 0){
