@@ -1,4 +1,4 @@
-/* $Id: network.hpp 7718 2005-08-05 22:23:42Z j_daniel $ */
+/* $Id: network.hpp 11428 2006-05-12 03:24:09Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -86,8 +86,12 @@ connection connect(const std::string& host, int port, threading::waiter& waiter)
 connection accept_connection();
 
 //function to disconnect from a certain host, or close all
-//connections if connection_num is 0
-void disconnect(connection connection_num=0);
+//connections if connection_num is 0.
+//returns true if the connection was disconnected.
+//returns false on failure to disconnect, since the socket is
+//in the middle of sending/receiving data. The socket will be closed when
+//it has finished its send/receive
+bool disconnect(connection connection_num=0);
 
 //function to queue a disconnection. Next time receive_data is
 //called, it will generate an error on the given connection.
@@ -103,25 +107,9 @@ void queue_disconnect(connection connection_num);
 connection receive_data(config& cfg, connection connection_num=0);
 connection receive_data(config& cfg, connection connection_num, int timeout);
 
-//sets the default maximum number of bytes to send to a client at a time
-void set_default_send_size(size_t send_size);
-
-enum SEND_TYPE { SEND_DATA, QUEUE_ONLY };
-
 //function to send data down a given connection, or broadcast
 //to all peers if connection_num is 0. throws error.
-//it will send a maximum of 'max_size' bytes. If cfg when serialized is more than
-//'max_size' bytes, then any remaining bytes will be appended to the connection's
-//send queue. Also if there is data already in the send queue, then it will be
-//appended to the send queue, and max_size bytes in the send queue will be sent.
-//if 'max_size' is 0, then the entire contents of the send_queue as well as 'cfg'
-//will be sent.
-//if 'mode' is set to 'QUEUE_ONLY', then no data will be sent at all: only placed
-//in the send queue. This will guarantee that this function never throws
-//network::error exceptions
-//
-//data in the send queue can be sent using process_send_queue
-void send_data(const config& cfg, connection connection_num=0, size_t max_size=0, SEND_TYPE mode=SEND_DATA);
+void send_data(const config& cfg, connection connection_num=0);
 
 //function to queue data to be sent. queue_data(cfg,sock) is equivalent to send_data(cfg,sock,0,QUEUE_ONLY)
 void queue_data(const config& cfg, connection connection_num=0);
@@ -131,7 +119,7 @@ void queue_data(const config& cfg, connection connection_num=0);
 void process_send_queue(connection connection_num=0, size_t max_size=0);
 
 //function to send data to all peers except 'connection_num'
-void send_data_all_except(const config& cfg, connection connection_num, size_t max_size=0);
+void send_data_all_except(const config& cfg, connection connection_num);
 
 //function to get the remote ip address of a socket
 std::string ip_address(connection connection_num);
