@@ -1,6 +1,6 @@
-/* $Id: pathfind.cpp,v 1.71 2005/06/05 10:56:41 silene Exp $ */
+/* $Id: pathfind.cpp 11699 2006-05-29 10:11:22Z rusty $ */
 /*
-Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+Copyright (C) 2003 by David White <davidnwhite@verizon.net>
 Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
 Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -109,7 +109,7 @@ static void a_star_explore_neighbours(gamemap::location const &dst, const double
 		if (locLocation.valid(int(parWidth), int(parHeight)) == false)
 			continue;
 		locNextNode = aStarGameWorld.getNodeFromLocation(locLocation, locIsCreated);
-		locCost = locCostFather + costCalculator->cost(locLocation, locCostFather, locLocation == dst);
+		locCost = locCostFather + costCalculator->cost(parCurNode->loc,locLocation, locCostFather, locLocation == dst);
 		if (locIsCreated) {
 			locNextNode->initNode(locLocation, dst, locCost, parCurNode, teleports);
 			if (locNextNode->g + locNextNode->h < stop_at) {
@@ -162,7 +162,7 @@ paths::route a_star_search(gamemap::location const &src, gamemap::location const
 
 	LOG_PF << "A* search: " << src << " -> " << dst << '\n';
 
-	if (costCalculator->cost(dst, 0, true) >= stop_at) {
+	if (costCalculator->cost(src,dst, 0, true) >= stop_at) {
 		LOG_PF << "aborted A* search because Start or Dest is invalid\n";
 		locRoute.move_left = int(costCalculator->getNoPathValue());
 		return locRoute;
@@ -218,7 +218,7 @@ label_AStarSearch_end:
 
 namespace {
 	gamemap::location find_vacant(const gamemap& map,
-		const std::map<gamemap::location,unit>& units,
+		const unit_map& units,
 		const gamemap::location& loc, int depth,
 		VACANT_TILE_TYPE vacancy,
 		std::set<gamemap::location>& touched)
@@ -254,7 +254,7 @@ namespace {
 }
 
 gamemap::location find_vacant_tile(const gamemap& map,
-																	 const std::map<gamemap::location,unit>& units,
+																	 const unit_map& units,
 																	 const gamemap::location& loc,
 																	 VACANT_TILE_TYPE vacancy)
 {
@@ -326,19 +326,17 @@ void get_tiles_radius(gamemap const &map, std::vector<gamemap::location> const &
 	}
 }
 
-bool enemy_zoc(gamemap const &map, gamestatus const &status,
-               std::map<gamemap::location, unit> const &units,
+bool enemy_zoc(gamemap const &map,
+               unit_map const &units,
                std::vector<team> const &teams,
-               gamemap::location const &loc, team const &current_team, int side)
+               gamemap::location const &loc, team const &viewing_team, unsigned int side)
 {
 	gamemap::location locs[6];
+	const team &current_team = teams[side-1];
 	get_adjacent_tiles(loc,locs);
 	for(int i = 0; i != 6; ++i) {
-		const std::map<gamemap::location,unit>::const_iterator it
-			= find_visible_unit(units,locs[i],
-			map,
-			status.get_time_of_day().lawful_bonus,
-			teams,current_team);
+		const unit_map::const_iterator it
+			= find_visible_unit(units, locs[i], map, teams, viewing_team);
 		if (it != units.end() && it->second.side() != side &&
 		    current_team.is_enemy(it->second.side()) && it->second.emits_zoc()) {
 			return true;
@@ -352,13 +350,14 @@ namespace {
 
 	void find_routes(const gamemap& map, const gamestatus& status,
 		const game_data& gamedata,
-		const std::map<gamemap::location,unit>& units,
+		const unit_map& units,
 		const unit& u,
 		const gamemap::location& loc,
 		int move_left,
 		std::map<gamemap::location,paths::route>& routes,
 		std::vector<team> const &teams,
-		bool ignore_zocs, bool allow_teleport, int turns_left, bool starting_pos)
+		bool ignore_zocs, bool allow_teleport, int turns_left,
+	    bool starting_pos, const team &viewing_team)
 	{
 		if(size_t(u.side()-1) >= teams.size()) {
 			return;
@@ -373,7 +372,9 @@ namespace {
 		//check for teleporting units -- we must be on a vacant (or occupied by this unit)
 		//village, that is controlled by our team to be able to teleport.
 		if (allow_teleport && map.is_village(loc) &&
-		    current_team.owns_village(loc) && (starting_pos || units.count(loc) == 0)) {
+		    current_team.owns_village(loc) &&
+			(starting_pos || find_visible_unit(units, loc, map,
+											   teams, viewing_team) == units.end())) {
 			const std::vector<gamemap::location>& villages = map.villages();
 
 			//if we are on a village, see all friendly villages that we can
@@ -397,10 +398,8 @@ namespace {
 				continue;
 
 			//see if the tile is on top of an enemy unit
-			const std::map<gamemap::location,unit>::const_iterator unit_it =
-				find_visible_unit(units, locs[i], map,
-				                  status.get_time_of_day().lawful_bonus,
-				                  teams, current_team);
+			const unit_map::const_iterator unit_it =
+				find_visible_unit(units, locs[i], map, teams, viewing_team);
 
 			if (unit_it != units.end() &&
 			    current_team.is_enemy(unit_it->second.side()))
@@ -410,7 +409,7 @@ namespace {
 			const gamemap::TERRAIN terrain = map[currentloc.x][currentloc.y];
 
 			//find the movement cost of this type onto the terrain
-			const int move_cost = u.movement_cost(map,terrain);
+			const int move_cost = u.movement_cost(terrain);
 			if (move_cost <= move_left ||
 			    turns_left > 0 && move_cost <= u.total_movement()) {
 				int new_move_left = move_left - move_cost;
@@ -429,8 +428,8 @@ namespace {
 				if(rtit != routes.end() && rtit->second.move_left >= total_movement)
 					continue;
 
-				const bool zoc = !ignore_zocs && enemy_zoc(map,status,units,teams,currentloc,
-							current_team,u.side());
+				const bool zoc = !ignore_zocs && enemy_zoc(map,units,teams,currentloc,
+														   viewing_team,u.side());
 				paths::route new_route = routes[loc];
 				new_route.steps.push_back(loc);
 
@@ -441,7 +440,7 @@ namespace {
 				if (new_route.move_left > 0) {
 					find_routes(map, status, gamedata, units, u, currentloc,
 					            zoc_move_left, routes, teams, ignore_zocs,
-					            allow_teleport, new_turns_left, false);
+					            allow_teleport, new_turns_left, false, viewing_team);
 				}
 			}
 		}
@@ -451,12 +450,13 @@ namespace {
 
 paths::paths(gamemap const &map, gamestatus const &status,
              game_data const &gamedata,
-             std::map<gamemap::location, unit> const &units,
+             unit_map const &units,
              gamemap::location const &loc,
              std::vector<team> const &teams,
-             bool ignore_zocs, bool allow_teleport, int additional_turns)
+             bool ignore_zocs, bool allow_teleport, const team &viewing_team,
+			 int additional_turns)
 {
-	const std::map<gamemap::location,unit>::const_iterator i = units.find(loc);
+	const unit_map::const_iterator i = units.find(loc);
 	if(i == units.end()) {
 		std::cerr << "unit not found\n";
 		return;
@@ -465,10 +465,10 @@ paths::paths(gamemap const &map, gamestatus const &status,
 	routes[loc].move_left = i->second.movement_left();
 	find_routes(map,status,gamedata,units,i->second,loc,
 		i->second.movement_left(),routes,teams,
-		ignore_zocs,allow_teleport,additional_turns,true);
+		ignore_zocs,allow_teleport,additional_turns,true,viewing_team);
 }
 
-int route_turns_to_complete(unit const &u, gamemap const &map, paths::route const &rt)
+int route_turns_to_complete(unit const &u, gamemap const &map, paths::route &rt)
 {
 	if(rt.steps.empty())
 		return 0;
@@ -477,10 +477,11 @@ int route_turns_to_complete(unit const &u, gamemap const &map, paths::route cons
 	for(std::vector<gamemap::location>::const_iterator i = rt.steps.begin()+1;
 	    i != rt.steps.end(); ++i) {
 		wassert(map.on_board(*i));
-		const int move_cost = u.movement_cost(map, map[i->x][i->y]);
+		const int move_cost = u.movement_cost(map[i->x][i->y]);
 		movement -= move_cost;
 		if (movement < 0) {
 			++turns;
+			rt.turn_waypoints.insert(std::make_pair(*(i-1), turns));
 			movement = u.total_movement() - move_cost;
 			if(movement < 0) {
 				return -1;
@@ -488,22 +489,27 @@ int route_turns_to_complete(unit const &u, gamemap const &map, paths::route cons
 		}
 	}
 
+	//add "end-of-path" to waypoints.
+	if (turns > 0) {
+		rt.turn_waypoints.insert(std::make_pair(*(rt.steps.end()-1), turns+1));
+	}
+	else {
+		rt.turn_waypoints.insert(std::make_pair(*(rt.steps.end()-1), 0));
+	}
+
 	return turns;
 }
 
 
 shortest_path_calculator::shortest_path_calculator(unit const &u, team const &t, unit_map const &units,
-                                                   std::vector<team> const &teams, gamemap const &map,
-                                                   gamestatus const &status)
+                                                   std::vector<team> const &teams, gamemap const &map)
 	: unit_(u), team_(t), units_(units), teams_(teams), map_(map),
-	  lawful_bonus_(status.get_time_of_day().lawful_bonus),
-	  unit_is_skirmisher_(unit_.type().is_skirmisher()),
 	  movement_left_(unit_.movement_left()),
 	  total_movement_(unit_.total_movement())
 {
 }
 
-double shortest_path_calculator::cost(const gamemap::location& loc, const double so_far, const bool isDst) const
+double shortest_path_calculator::cost(const gamemap::location& src,const gamemap::location& loc, const double so_far, const bool isDst) const
 {
 	wassert(map_.on_board(loc));
 
@@ -517,27 +523,27 @@ double shortest_path_calculator::cost(const gamemap::location& loc, const double
 	if (team_.shrouded(loc.x, loc.y))
 		return getNoPathValue();
 
-	int const base_cost = unit_.movement_cost(map_, map_[loc.x][loc.y]);
+	int const base_cost = unit_.movement_cost(map_[loc.x][loc.y]);
 	wassert(base_cost >= 1); // pathfinding heuristic: the cost must be at least 1
 	if (total_movement_ < base_cost)
 		return getNoPathValue();
 
 	unit_map::const_iterator
-		enemy_unit = find_visible_unit(units_, loc, map_, lawful_bonus_, teams_, team_),
+		enemy_unit = find_visible_unit(units_, loc, map_, teams_, team_),
 		units_end = units_.end();
 
 	if (enemy_unit != units_end && team_.is_enemy(enemy_unit->second.side()))
 		return getNoPathValue();
 
-	if (!isDst && !unit_is_skirmisher_) {
+	if (!isDst && !unit_.get_ability_bool("skirmisher",src)) {
 		gamemap::location adj[6];
 		get_adjacent_tiles(loc, adj);
 
 		for (size_t i = 0; i != 6; ++i) {
-			enemy_unit = find_visible_unit(units_, adj[i], map_, lawful_bonus_, teams_, team_);
+			enemy_unit = find_visible_unit(units_, adj[i], map_, teams_, team_);
 
 			if (enemy_unit != units_end && team_.is_enemy(enemy_unit->second.side()) &&
-			    !team_.fogged(adj[i].x, adj[i].y) && enemy_unit->second.emits_zoc())
+			    enemy_unit->second.emits_zoc())
 				return getNoPathValue();
 		}
 	}
@@ -555,3 +561,15 @@ double shortest_path_calculator::cost(const gamemap::location& loc, const double
 	int additional_cost = base_cost > remaining_movement ? remaining_movement : 0;
 	return base_cost + additional_cost;
 }
+
+emergency_path_calculator::emergency_path_calculator(const unit& u, const gamemap& map)
+	: unit_(u), map_(map)
+{
+}
+
+double emergency_path_calculator::cost(const gamemap::location&,const gamemap::location& loc, const double, const bool) const
+{
+	wassert(map_.on_board(loc));
+
+	return unit_.movement_cost(map_[loc.x][loc.y]);
+}
