@@ -29,7 +29,7 @@ play_controller::play_controller(const config& level, const game_data& gameinfo,
 	mouse_handler_(gui_, teams_, units_, map_, status_, gameinfo, undo_stack_, redo_stack_),
 	menu_handler_(gui_, units_, teams_, level, gameinfo, map_, game_config, status_, state_of_game, undo_stack_, redo_stack_),
 	generator_setter(&recorder), statistics_context_(level["name"]), gameinfo_(gameinfo), level_(level), game_config_(game_config),
-	gamestate_(state_of_game), status_(level, num_turns),
+	gamestate_(state_of_game), status_(level, num_turns, &state_of_game),
 	map_(game_config, level["map_data"]), ticks_(ticks),
 	xp_mod_(atoi(level["experience_modifier"].c_str()) > 0 ? atoi(level["experience_modifier"].c_str()) : 100),
 	loading_game_(level["playing_team"].empty() == false),
@@ -185,7 +185,7 @@ void play_controller::objectives(){
 }
 
 void play_controller::show_statistics(){
-	menu_handler_.show_statistics();
+	menu_handler_.show_statistics(gui_->viewing_team()+1);
 }
 
 void play_controller::unit_list(){
@@ -288,6 +288,8 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 	log_scope("player turn");
 	team& current_team = teams_[team_index];
 
+	mouse_handler_.set_team(team_index+1);
+
 	if(team_manager_.is_observer()) {
 		gui_->set_team(size_t(team_index));
 	}
@@ -297,8 +299,8 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 	gamestate_.set_variable("side_number",player_number_str.str());
 
 	if(first_turn_) {
-		if(gui_->viewing_team() == team_index && !team_manager_.is_observer()) {
-			if (!is_replay){
+		if(!current_team.is_network()) {
+			if(!is_replay) {
 				recorder.add_event("turn 1");
 				recorder.add_event("new turn");
 				recorder.add_event("side turn");
@@ -306,18 +308,13 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 			game_events::fire("turn 1");
 			game_events::fire("new turn");
 			game_events::fire("side turn");
-		} else if(current_team.is_ai()) {
-			game_events::fire("turn 1");
-			game_events::fire("new turn");
-			game_events::fire("side turn");
 		}
 		first_turn_ = false;
 	} else
 	//fire side turn event only if real side change occurs not counting changes from void to a side
 	if (team_index != (first_player_ - 1) || status_.turn() > start_turn_) {
-		if(gui_->viewing_team() == team_index && !team_manager_.is_observer()
-			|| current_team.is_ai()){
-			if (!is_replay){
+		if(!current_team.is_network()) {
+			if(!is_replay) {
 				recorder.add_event("side turn");
 			}
 			game_events::fire("side turn");
@@ -367,13 +364,19 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 bool play_controller::do_replay(const bool replaying){
 	bool result = false;
 	if(replaying) {
+		/* YogiHH: I can't see why we need another key_handler here in addition
+		to the one defined in play_controller. Since this is causing problems with
+		double execution of hotkeys i will comment it out
 		const hotkey::basic_handler key_events_handler(gui_);
+		*/
 		LOG_NG << "doing replay " << player_number_ << "\n";
 		try {
 			result = ::do_replay(*gui_,map_,gameinfo_,units_,teams_,
 						          player_number_,status_,gamestate_);
 		} catch(replay::error&) {
-			gui::show_dialog(*gui_,NULL,"",_("The file you have tried to load is corrupt"),gui::OK_ONLY);
+			//in next version after string freeze add to text ". continue playing?"
+			if(gui::show_dialog(*gui_,NULL,"",_("The file you have tried to load is corrupt"),gui::OK_CANCEL))
+				throw;
 
 			result = false;
 		}
@@ -511,6 +514,29 @@ void play_controller::enter_textbox()
 	menu_handler_.get_textbox().close(*gui_);
 }
 
+int play_controller::find_human_team_before(const size_t team_num) const
+{
+	if (team_num > teams_.size())
+		return -2;
+
+	int human_side = -2;
+	for (int i = team_num-2; i > -1; --i) {
+		if (teams_[i].is_human()) {
+			human_side = i;
+			break;
+		}
+	}
+	if (human_side == -2) {
+		for (size_t i = teams_.size()-1; i > team_num-1; --i) {
+			if (teams_[i].is_human()) {
+				human_side = i;
+				break;
+			}
+		}
+	}
+	return human_side+1;
+}
+
 void play_controller::handle_event(const SDL_Event& event)
 {
 	if(gui::in_dialog()) {
@@ -523,12 +549,15 @@ void play_controller::handle_event(const SDL_Event& event)
 		//in which case the key press events should go only to it.
 		if(menu_handler_.get_textbox().active() == false) {
 			hotkey::key_event(*gui_,event.key,this);
-		} else if(event.key.keysym.sym == SDLK_ESCAPE) {
-			menu_handler_.get_textbox().close(*gui_);
-		} else if(event.key.keysym.sym == SDLK_TAB) {
-			menu_handler_.get_textbox().tab(teams_, units_, *gui_);
-		} else if(event.key.keysym.sym == SDLK_RETURN) {
-			enter_textbox();
+		} else {
+			if(event.key.keysym.sym == SDLK_ESCAPE) {
+				menu_handler_.get_textbox().close(*gui_);
+			} else if(event.key.keysym.sym == SDLK_TAB) {
+				menu_handler_.get_textbox().tab(teams_, units_, *gui_);
+			} else if(event.key.keysym.sym == SDLK_RETURN) {
+				enter_textbox();
+			}
+			break;
 		}
 
 		//intentionally fall-through
@@ -565,14 +594,14 @@ void play_controller::handle_event(const SDL_Event& event)
 					SDL_EVENTMASK(SDL_MOUSEMOTION)) > 0) {
 			while(SDL_PeepEvents(&new_event,1,SDL_GETEVENT,
 						SDL_EVENTMASK(SDL_MOUSEMOTION)) > 0);
-			mouse_handler_.mouse_motion(new_event.motion, player_number_, browse_);
+			mouse_handler_.mouse_motion(new_event.motion, browse_);
 		} else {
-			mouse_handler_.mouse_motion(event.motion, player_number_, browse_);
+			mouse_handler_.mouse_motion(event.motion, browse_);
 		}
 		break;
 	case SDL_MOUSEBUTTONDOWN:
 	case SDL_MOUSEBUTTONUP:
-		mouse_handler_.mouse_press(event.button, player_number_, browse_);
+		mouse_handler_.mouse_press(event.button, browse_);
 		if (mouse_handler_.get_undo()){
 			mouse_handler_.set_undo(false);
 			menu_handler_.undo(player_number_, mouse_handler_);
@@ -608,16 +637,16 @@ void play_controller::play_slice()
 
 	const int scroll_threshold = (preferences::mouse_scroll_enabled()) ? 5 : 0;
 
-	if(key[SDLK_UP] || mousey < scroll_threshold)
+	if((key[SDLK_UP] && !menu_handler_.get_textbox().active()) || mousey < scroll_threshold)
 		gui_->scroll(0,-preferences::scroll_speed());
 
-	if(key[SDLK_DOWN] || mousey > gui_->y()-scroll_threshold)
+	if((key[SDLK_DOWN] && !menu_handler_.get_textbox().active()) || mousey > gui_->y()-scroll_threshold)
 		gui_->scroll(0,preferences::scroll_speed());
 
-	if(key[SDLK_LEFT] || mousex < scroll_threshold)
+	if((key[SDLK_LEFT] && !menu_handler_.get_textbox().active()) || mousex < scroll_threshold)
 		gui_->scroll(-preferences::scroll_speed(),0);
 
-	if(key[SDLK_RIGHT] || mousex > gui_->x()-scroll_threshold)
+	if((key[SDLK_RIGHT] && !menu_handler_.get_textbox().active()) || mousex > gui_->x()-scroll_threshold)
 		gui_->scroll(preferences::scroll_speed(),0);
 
 	gui_->draw();
@@ -651,13 +680,19 @@ std::vector<std::string> play_controller::expand_menu(std::vector<std::string>&
 			// Make sure list doesn't get too long: keep top two,
 			// midpoint and bottom.
 			if (newitems.size() > 5) {
-				std::vector<std::string> subitems;
+				std::vector<std::string> subitems, subsavenames(i, "");
 				subitems.push_back(newitems[0]);
 				subitems.push_back(newitems[1]);
 				subitems.push_back(newitems[newitems.size() / 3]);
 				subitems.push_back(newitems[newitems.size() * 2 / 3]);
 				subitems.push_back(newitems.back());
+				subsavenames.push_back(savenames[i]);
+				subsavenames.push_back(savenames[i+1]);
+				subsavenames.push_back(savenames[i+newitems.size() / 3]);
+				subsavenames.push_back(savenames[i+newitems.size() * 2 / 3]);
+				subsavenames.push_back(savenames.back());
 				newitems = subitems;
+				savenames = subsavenames;
 			}
 
 			items.insert(items.begin()+i, newitems.begin(), newitems.end());
