@@ -1,4 +1,4 @@
-/* $Id: playcampaign.cpp 12055 2006-06-12 04:46:55Z soliton $ */
+/* $Id: playcampaign.cpp 14736 2006-12-19 10:37:39Z edb $ */
 /*
    Copyright (C) 2003-2005 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
@@ -190,7 +190,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 			for(config::child_list::const_iterator side = sides_list.begin();
 					side != sides_list.end(); ++side) {
 				if((**side)["controller"] == "network" &&
-						(**side)["description"] == preferences::login()) {
+						(**side)["current_player"] == preferences::login()) {
 					(**side)["controller"] = preferences::client_type();
 				} else if((**side)["controller"] != "null") {
 					(**side)["controller"] = "network";
@@ -203,6 +203,8 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 
 		bool save_game_after_scenario = true;
 
+		const set_random_generator generator_setter(&recorder);
+
 		try {
 			// preserve old label eg. replay
 			if (state.label.empty())
@@ -365,15 +367,22 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 				const config::child_list& sides_list = starting_pos.get_children("side");
 				for(config::child_list::const_iterator side = sides_list.begin();
 						side != sides_list.end(); ++side) {
-
 					std::string id = (**side)["save_id"];
 					if(id.empty()) {
 						continue;
 					}
 
+					/*Upadate side info to match current_player info to
+					  allow it taking the side in next scenario and to be
+					  set in the players list on side server */
 					controller_map::const_iterator ctr = controllers.find(id);
 					if(ctr != controllers.end()) {
-						(**side)["description"] = ctr->second.description;
+						player_info *player = state.get_player(id);
+						if (player) {
+							(**side)["current_player"] = player->name;
+							//TODO : remove (see TODO line 276 in server/game.cpp)
+							(**side)["user_description"] = player->name;
+						}
 						(**side)["controller"] = ctr->second.controller;
 					}
 				}
