@@ -185,8 +185,10 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			}
 		}
 
+		bool save = false;
 		for(; ; first_player_ = 1) {
-			play_turn();
+			play_turn(save);
+			save = true;
 		} //end for loop
 
 	} catch(game::load_game_exception&) {
@@ -206,6 +208,10 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 				config& info = cfg.add_child("info");
 				info["type"] = "termination";
 				info["condition"] = "game over";
+				if (end_level.result == VICTORY)
+					info["result"] = "victory";
+				else
+					info["result"] = "defeat";
 				network::send_data(cfg);
 			} else {
 				gui::show_dialog(*gui_, NULL, _("Game Over"),
@@ -242,7 +248,15 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			}
 
 			const bool has_next_scenario = !gamestate_.scenario.empty() &&
-			                               gamestate_.scenario != "null";
+											gamestate_.scenario != "null";
+
+			//save current_player name to reuse it when setting next_scenario side info
+			std::vector<team>::iterator i;
+			for (i = teams_.begin(); i != teams_.end(); ++i) {
+				player_info *player=gamestate_.get_player(i->save_id());
+				if (player)
+					player->name = i->current_player();
+			}
 
 			//add all the units that survived the scenario
 			for(unit_map::iterator un = units_.begin(); un != units_.end(); ++un) {
@@ -258,7 +272,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			//'continue' is like a victory, except it doesn't announce victory,
 			//and the player retains 100% of gold.
 			if(end_level.result == LEVEL_CONTINUE || end_level.result == LEVEL_CONTINUE_NO_SAVE) {
-				for(std::vector<team>::iterator i=teams_.begin(); i!=teams_.end(); ++i) {
+				for(i=teams_.begin(); i!=teams_.end(); ++i) {
 					player_info *player=gamestate_.get_player(i->save_id());
 					if(player) {
 						player->gold = i->gold();
@@ -271,7 +285,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 
 			std::stringstream report;
 
-			for(std::vector<team>::iterator i=teams_.begin(); i!=teams_.end(); ++i) {
+			for(i=teams_.begin(); i!=teams_.end(); ++i) {
 				if (!i->is_persistent())
 					continue;
 
@@ -293,7 +307,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 							report << "\n";
 						}
 
-						report << font::BOLD_TEXT << i->save_id() << "\n";
+						report << font::BOLD_TEXT << i->current_player() << "\n";
 					}
 
 					report << _("Remaining gold: ")
@@ -326,6 +340,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 
 			return VICTORY;
 		}
+		image::flush_cache();
 	} //end catch
 	catch(replay::error&) {
 		gui::show_dialog(*gui_,NULL,"",_("The file you have tried to load is corrupt"),
@@ -350,7 +365,8 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 	return QUIT;
 }
 
-void playsingle_controller::play_turn(){
+void playsingle_controller::play_turn(bool save)
+{
 	gui_->new_turn();
 	gui_->invalidate_game_status();
 	events::raise_draw_event();
@@ -363,7 +379,11 @@ void playsingle_controller::play_turn(){
 			init_side(player_number_ - 1);
 
 			if (replaying_){
+				/* YogiHH: I can't see why we need another key_handler here in addition
+				to the one defined in play_controller. Since this is causing problems with
+				double execution of hotkeys i will comment it out
 				const hotkey::basic_handler key_events_handler(gui_);
+				*/
 				LOG_NG << "doing replay " << player_number_ << "\n";
 				try {
 					replaying_ = ::do_replay(*gui_,map_,gameinfo_,units_,teams_,
@@ -376,7 +396,7 @@ void playsingle_controller::play_turn(){
 				LOG_NG << "result of replay: " << (replaying_?"true":"false") << "\n";
 			}
 			else{
-				play_side(player_number_);
+				play_side(player_number_, save);
 			}
 
 			finish_side_turn();
@@ -390,7 +410,8 @@ void playsingle_controller::play_turn(){
 	finish_turn();
 }
 
-void playsingle_controller::play_side(const unsigned int team_index){
+void playsingle_controller::play_side(const unsigned int team_index, bool save)
+{
 //goto this label if the type of a team (human/ai/networked) has changed mid-turn
 redo_turn:
 	//although this flag is used only in this method it has to be a class member
@@ -401,12 +422,25 @@ redo_turn:
 	if(current_team().is_human()) {
 		LOG_NG << "is human...\n";
 		try{
-			before_human_turn();
+			before_human_turn(save);
 			play_human_turn();
 			after_human_turn();
 		} catch(end_turn_exception& end_turn) {
-			if (end_turn.redo == team_index)
+			if (end_turn.redo == team_index) {
 				player_type_changed_ = true;
+				// if new controller is not human,
+				// reset gui to prev human one
+				if (!teams_[team_index-1].is_human()) {
+					int t = find_human_team_before(team_index);
+					if (t > 0) {
+						gui_->set_team(t-1);
+						gui_->recalculate_minimap();
+						gui_->invalidate_all();
+						gui_->draw();
+						gui_->update_display();
+					}
+				}
+			}
 		}
 
 		if(game_config::debug)
@@ -420,7 +454,8 @@ redo_turn:
 	if (player_type_changed_) { goto redo_turn; }
 }
 
-void playsingle_controller::before_human_turn(){
+void playsingle_controller::before_human_turn(bool save)
+{
 	log_scope("player turn");
 	browse_ = false;
 
@@ -430,7 +465,9 @@ void playsingle_controller::before_human_turn(){
 	gui_->draw();
 	gui_->update_display();
 
-	menu_handler_.autosave(gamestate_.label, status_.turn(), gamestate_.starting_pos);
+	if (save) {
+		menu_handler_.autosave(gamestate_.label, status_.turn(), gamestate_.starting_pos);
+	}
 
 	if(preferences::turn_bell()) {
 		sound::play_sound(game_config::sounds::turn_bell);
