@@ -1,4 +1,4 @@
-/* $Id: preferences.cpp 12479 2006-07-09 12:51:30Z sapient $ */
+/* $Id: preferences.cpp 14081 2006-10-26 10:22:11Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -176,6 +176,11 @@ void set_language(const std::string& s)
 	prefs["locale"] = s;
 }
 
+unsigned int sample_rate()
+{
+	return lexical_cast_default<unsigned int>(prefs["sample_rate"], 22050);
+}
+
 int music_volume()
 {
 	return lexical_cast_default<int>(prefs["music_volume"], 100);
