@@ -1,4 +1,4 @@
-/* $Id: publish_campaign.cpp 8762 2005-11-06 23:51:57Z jhinrichs $ */
+/* $Id: publish_campaign.cpp 14698 2006-12-16 19:12:05Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -25,7 +25,11 @@ namespace {
 
 const std::string& campaign_dir()
 {
+	#ifdef __amigaos4__
+	static const std::string res = "data/campaigns";
+	#else
 	static const std::string res = get_user_data_dir() + "/data/campaigns";
+	#endif
 	return res;
 }
 
@@ -68,6 +72,32 @@ std::vector<std::string> available_campaigns()
 	return res;
 }
 
+// Return a vector of detected scripts.
+std::vector<config *> find_scripts(const config &cfg, std::string extension)
+{
+	std::vector<config *> python_scripts;
+	const config::child_list& dirs = cfg.get_children("dir");
+	config::child_list::const_iterator i;
+	for(i = dirs.begin(); i != dirs.end(); ++i) {
+		const config::child_list& files = (**i).get_children("file");
+		config::child_list::const_iterator j;
+		for(j = files.begin(); j != files.end(); ++j) {
+			std::string filename = (**j)["name"].str();
+			if (filename.length() > extension.length()) {
+				if (filename.substr(filename.length() - extension.length()) ==
+					extension) {
+					python_scripts.push_back(*j);
+				}
+			}
+		}
+		// Recursively look for files in sub directories.
+		std::vector<config *> childs = find_scripts(**i, extension);
+		python_scripts.insert(python_scripts.end(),
+			childs.begin(), childs.end());
+	}
+	return python_scripts;
+}
+
 namespace {
 
 const char escape_char = 1;
@@ -149,7 +179,12 @@ void unarchive_file(const std::string& path, const config& cfg)
 
 void unarchive_dir(const std::string& path, const config& cfg)
 {
-	const std::string dir = path + '/' + cfg["name"].str();
+	std::string dir;
+	if (cfg["name"].empty())
+		dir = path;
+	else
+		dir = path + '/' + cfg["name"].str();
+
 	make_directory(dir);
 
 	const config::child_list& dirs = cfg.get_children("dir");
