@@ -1,4 +1,4 @@
-/* $Id: replay.cpp 12695 2006-07-21 16:39:08Z jhinrichs $ */
+/* $Id: replay.cpp 14848 2006-12-23 07:00:45Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -522,7 +522,14 @@ config* replay::get_next_action()
 void replay::pre_replay()
 {
 	if ((rng::random() == NULL) && (commands().size() > 0)){
-		set_random(commands()[pos_]);
+		if (at_end())
+		{
+			add_command(true);
+		}
+		else
+		{
+			set_random(commands()[pos_]);
+		}
 	}
 }
 
@@ -605,7 +612,7 @@ void check_checksums(display& disp,const unit_map& units,const config& cfg)
 		if(u == units.end()) {
 			std::stringstream message;
 			message << "non existant unit to checksum at " << loc.x+1 << "," << loc.y+1 << "!";
-			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE);
+			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE,false);
 			continue;
 		}
 		std::string check;
@@ -613,7 +620,7 @@ void check_checksums(display& disp,const unit_map& units,const config& cfg)
 		if(check != (**ci)["value"]) {
 			std::stringstream message;
 			message << "checksum mismatch at " << loc.x+1 << "," << loc.y+1 << "!";
-			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE);
+			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE,false);
 		}
 	}
 }
@@ -689,18 +696,15 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 		} else if((child = cfg->child("speak")) != NULL) {
 			const std::string& team_name = (*child)["team_name"];
+			const std::string& speaker_name = (*child)["description"];
 			if(team_name == "" || teams[disp.viewing_team()].team_name() == team_name) {
-				bool is_lobby_join = ((*child)["description"] == "server"
+				bool is_lobby_join = (speaker_name == "server"
 							&& (*child)["message"].value().find("has logged into the lobby") != std::string::npos);
-
-				if(!replayer.is_skipping() && (!is_lobby_join || preferences::lobby_joins())) {
-					if(preferences::message_bell()) {
-						sound::play_sound(game_config::sounds::receive_message);
-					}
-
+				bool is_whisper = (speaker_name.find("whisper: ") == 0);
+				if((!replayer.is_skipping() || is_whisper) && (!is_lobby_join || preferences::lobby_joins())) {
 					const int side = lexical_cast_default<int>((*child)["side"].c_str(),0);
-					disp.add_chat_message((*child)["description"],side,(*child)["message"],
-										  team_name == "" ? display::MESSAGE_PUBLIC : display::MESSAGE_PRIVATE);
+					disp.add_chat_message(speaker_name,side,(*child)["message"],
+										  team_name == "" ? display::MESSAGE_PUBLIC : display::MESSAGE_PRIVATE, preferences::message_bell());
 				}
 			}
 		} else if((child = cfg->child("label")) != NULL) {
@@ -719,14 +723,19 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const std::string& name = (*child)["name"];
 
 			unit_map::iterator u = units.find(loc);
-
-			if(u->second.unrenamable()) {
+			if(u != units.end()) {
+				if(u->second.unrenamable()) {
+					std::stringstream errbuf;
+					errbuf << "renaming unrenamable unit " << u->second.name() << "\n";
+					replay::throw_error(errbuf.str());
+				}
+				u->second.rename(name);
+			} else {
 				std::stringstream errbuf;
-				errbuf << "renaming unrenamable unit " << u->second.name() << "\n";
+				errbuf << "attempt to rename unit at location: "
+				   << loc << ", where none exists.\n";
 				replay::throw_error(errbuf.str());
 			}
-
-			u->second.rename(name);
 		}
 
 		//if there is an end turn directive
@@ -903,9 +912,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			}
 
 			if(!replayer.is_skipping()) {
-				u->second.set_hidden(true);
 				unit_display::move_unit(disp,map,rt->second.steps,u->second,units,teams);
-				u->second.set_hidden(false);
 			}
 			else{
 				//unit location needs to be updated
@@ -916,6 +923,11 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			std::pair<gamemap::location,unit> *up = units.extract(u->first);
 			up->first = dst;
 			units.add(up);
+			if (up->first == up->second.get_goto())
+			{
+				//if unit has arrived to destination, goto variable is cleaned
+				up->second.set_goto(gamemap::location());
+			}
 			u = units.find(dst);
 			check_checksums(disp,units,*cfg);
 			// Get side now, in case game events change the unit.
@@ -989,6 +1001,10 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				replay::throw_error(errbuf.str());
 			}
 
+			if(def_weapon_num >= (int)tgt->second.attacks().size()) {
+				replay::throw_error("illegal defender weapon type in attack\n");
+			}
+
 			attack(disp, map, teams, src, dst, weapon_num, def_weapon_num, units, state, gameinfo, !replayer.is_skipping());
 
 			u = units.find(src);
