@@ -1,4 +1,4 @@
-/* $Id: reports.cpp 12634 2006-07-18 21:30:30Z xan $ */
+/* $Id: reports.cpp 14820 2006-12-22 20:05:51Z mordante $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -394,7 +394,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 		}
 
 		if(underlying.size() != 1 || underlying[0] != terrain) {
-			str << "(";
+			str << " (";
 
 			for(std::string::const_iterator i = underlying.begin(); i != underlying.end(); ++i) {
 			str << map.get_terrain_info(*i).name();
@@ -451,8 +451,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 		const std::vector<std::string> items = utils::split(flag);
 		const std::vector<std::string> sub_items = utils::split(items[0], ':');
 		image::locator flag_image(sub_items[0], new_rgb, old_rgb);
-		u = find_leader(units,playing_side);
-		return report("",flag_image,u != units.end() ? u->second.description() : "");
+		return report("",flag_image,teams[playing_side-1].current_player());
 	}
 
 	case OBSERVERS: {
