@@ -1,6 +1,6 @@
-/* $Id: sdl_utils.cpp,v 1.79 2005/06/04 19:16:06 ott Exp $ */
+/* $Id: sdl_utils.cpp 12002 2006-06-10 16:20:25Z torangan $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -25,8 +25,6 @@
 #include <iostream>
 #include <map>
 
-#define ERR_DP LOG_STREAM(err, display)
-
 SDLKey sdl_keysym_from_name(std::string const &keyname)
 {
 	static bool initialized = false;
@@ -56,8 +54,8 @@ bool point_in_rect(int x, int y, const SDL_Rect& rect)
 
 bool rects_overlap(const SDL_Rect& rect1, const SDL_Rect& rect2)
 {
-	return point_in_rect(rect1.x,rect1.y,rect2) || point_in_rect(rect1.x+rect1.w,rect1.y,rect2) ||
-	       point_in_rect(rect1.x,rect1.y+rect1.h,rect2) || point_in_rect(rect1.x+rect1.w,rect1.y+rect1.h,rect2);
+	return point_in_rect(rect1.x,rect1.y,rect2) || point_in_rect(rect2.x+rect2.w,rect2.y,rect1) ||
+	       point_in_rect(rect2.x,rect2.y+rect2.h,rect1) || point_in_rect(rect1.x+rect1.w,rect1.y+rect1.h,rect2);
 }
 
 SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
@@ -126,14 +124,19 @@ surface create_optimized_surface(surface const &surf)
 	return result;
 }
 
+// don't pass this function 0 scaling arguments
 surface scale_surface(surface const &surf, int w, int h)
 {
+	assert(SDL_ALPHA_TRANSPARENT==0);
+
 	if(surf == NULL)
 		return NULL;
 
 	if(w == surf->w && h == surf->h) {
 		return surf;
 	}
+	wassert(w != 0);
+	wassert(h != 0);
 
 	surface dst(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,32,0xFF0000,0xFF00,0xFF,0xFF000000));
 	surface src(make_neutral_surface(surf));
@@ -160,7 +163,111 @@ surface scale_surface(surface const &surf, int w, int h)
 				const int xsrcint = fxptoi(xsrc);
 				const int ysrcint = fxptoi(ysrc);
 
-				dst_pixels[ydst*dst->w + xdst] = src_pixels[ysrcint*src->w + xsrcint];
+				Uint32* const src_word = src_pixels + ysrcint*src->w + xsrcint;
+				Uint32* const dst_word = dst_pixels +    ydst*dst->w + xdst;
+				const int dx = (xsrcint + 1 < src->w) ? 1 : 0;
+				const int dy = (ysrcint + 1 < src->h) ? src->w : 0;
+
+				Uint8 r,g,b,a;
+				Uint32 rr,gg,bb,aa;
+				Uint16 avg_r, avg_g, avg_b, avg_a;
+				Uint32 pix[4], bilin[4];
+
+				// This next part is the fixed point
+				// equivalent of "take everything to
+				// the right of the decimal point."
+				// These fundamental weights decide
+				// the contributions from various
+				// input pixels. The labels assume
+				// that the upper left corner of the
+				// screen ("northeast") is 0,0 but the
+				// code should still be consistant if
+				// the graphics origin is actually
+				// somewhere else.
+
+				const fixed_t e = 0x000000FF & xsrc;
+				const fixed_t s = 0x000000FF & ysrc;
+				const fixed_t n = 0xFF - s;
+				const fixed_t w = 0xFF - e;
+
+				pix[0] = *src_word;              // northwest
+			        pix[1] = *(src_word + dx);       // northeast
+				pix[2] = *(src_word + dy);       // southwest
+				pix[3] = *(src_word + dx + dy);  // southeast
+
+				bilin[0] = n*w;
+				bilin[1] = n*e;
+				bilin[2] = s*w;
+				bilin[3] = s*e;
+
+				// Scope out the neighboorhood, see
+				// what the pixel values are like.
+
+				int count = 0;
+				avg_r = avg_g = avg_b = avg_a = 0;
+				int loc;
+				for (loc=0; loc<4; loc++) {
+				  SDL_GetRGBA(pix[loc],src->format,&r,&g,&b,&a);
+				  if (a != 0) {
+				    avg_r += r;
+				    avg_g += g;
+				    avg_b += b;
+				    avg_a += a;
+				    count++;
+				  }
+				}
+				if (count>0) {
+				  avg_r /= count;
+				  avg_b /= count;
+				  avg_g /= count;
+				  avg_a /= count;
+				}
+
+				// Perform modified bilinear
+				// interpolation. Don't trust any
+				// color information from an RGBA
+				// sample when the alpha channel is
+				// set to fully transparent.
+				//
+				// Some of the input images are hex
+				// tiles, created using a hexagon
+				// shaped alpha channel that is either
+				// set to full-on or full-off. If
+				// intermediate alpha values are
+				// introduced along a hex edge, it
+				// produces a gametime artifact.
+				// Moving the mouse around will leave
+				// behind "hexagon halos" from the
+				// temporary highlighting. In other
+				// words the Wesnoth rendering engine
+				// freaks out.
+				//
+				// The alpha thresholding step
+				// attempts to accomodates this
+				// limitation. There is a small loss
+				// of quality. For example, skeleton
+				// bowstrings are not as good as they
+				// could be.
+
+				rr = gg = bb = aa = 0;
+				for (loc=0; loc<4; loc++) {
+				  SDL_GetRGBA(pix[loc],src->format,&r,&g,&b,&a);
+				  if (a == 0) {
+				    r = avg_r;
+				    g = avg_g;
+				    b = avg_b;
+				  }
+				  rr += r * bilin[loc];
+				  gg += g * bilin[loc];
+				  bb += b * bilin[loc];
+				  aa += a * bilin[loc];
+				}
+				r = rr >> 16;
+				g = gg >> 16;
+				b = bb >> 16;
+				a = aa >> 16;
+				a = (a < avg_a/2) ? 0 : avg_a;
+				*dst_word = SDL_MapRGBA(dst->format,r,g,b,a);
 			}
 		}
 	}
@@ -318,6 +425,141 @@ surface greyscale_image(surface const &surf)
 	return create_optimized_surface(nsurf);
 }
 
+surface darken_image(surface const &surf)
+{
+	if(surf == NULL)
+		return NULL;
+
+	surface nsurf(make_neutral_surface(surf));
+	if(nsurf == NULL) {
+		std::cerr << "failed to make neutral surface\n";
+		return NULL;
+	}
+
+	{
+		surface_lock lock(nsurf);
+		Uint32* beg = lock.pixels();
+		Uint32* end = beg + nsurf->w*surf->h;
+
+		while(beg != end) {
+			Uint8 red, green, blue, alpha;
+			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+
+			//const Uint8 avg = (red+green+blue)/3;
+
+			//use the correct formula for RGB to grayscale
+			//conversion. ok, this is no big deal :)
+			//the correct formula being:
+			//gray=0.299red+0.587green+0.114blue
+			const Uint8 avg = (Uint8)((77*(Uint16)red +
+						   150*(Uint16)green +
+						   29*(Uint16)blue) / 256);
+			// then tint 77%, 67%, 72%
+			const Uint8 r=(Uint8)((avg*196)>>8);
+			const Uint8 g=(Uint8)((avg*171)>>8);
+			const Uint8 b=(Uint8)((avg*184)>>8);
+
+
+			*beg = SDL_MapRGBA(nsurf->format,r,g,b,alpha);
+
+			++beg;
+		}
+	}
+
+	return create_optimized_surface(nsurf);
+}
+
+surface recolor_image(surface surf, const color_range& new_rgb, const std::vector<Uint32>& old_rgb)
+{
+  // function to replace vectors of colors with new color-range
+  // parallel input vectors chosen to prevent need to include SDL structures
+  // elsewhere in code or create new rgb structure.  It might be better to
+  // just pass a vector of SDL colors instead...
+
+  if(old_rgb.size()){
+    Uint16 new_red  = (new_rgb.mid() & 0x00FF0000)>>16;
+    Uint16 new_green= (new_rgb.mid() & 0x0000FF00)>>8;
+    Uint16 new_blue = (new_rgb.mid() & 0x000000FF);
+    Uint16 max_red  = (new_rgb.max() & 0x00FF0000)>>16;
+    Uint16 max_green= (new_rgb.max() & 0x0000FF00)>>8 ;
+    Uint16 max_blue = (new_rgb.max() & 0x000000FF)    ;
+    Uint16 min_red  = (new_rgb.min() & 0x00FF0000)>>16;
+    Uint16 min_green= (new_rgb.min() & 0x0000FF00)>>8 ;
+    Uint16 min_blue = (new_rgb.min() & 0x000000FF)    ;
+
+    if(surf == NULL)
+      return NULL;
+
+        surface nsurf(make_neutral_surface(surf));
+        if(nsurf == NULL) {
+	  std::cerr << "failed to make neutral surface\n";
+	  return NULL;
+        }
+
+	Uint16 reference_avg=0;
+	//map first color in vector to exact new color
+	Uint32 temp_rgb=old_rgb[0];
+	int old_r=(temp_rgb & 0X00FF0000)>>16;
+	int old_g=(temp_rgb & 0X0000FF00)>>8;
+	int old_b=(temp_rgb & 0X000000FF);
+	reference_avg = (Uint16)(((Uint16) old_r + (Uint16)old_g + (Uint16)old_b)
+			   / 3);
+
+	{
+	  for(std::vector<Uint32>::const_iterator temp_rgb = old_rgb.begin();
+	      temp_rgb!=old_rgb.end();temp_rgb++)
+	    {
+	      int old_r=((*temp_rgb) & 0X00FF0000)>>16;
+	      int old_g=((*temp_rgb) & 0X0000FF00)>>8;
+	      int old_b=((*temp_rgb) & 0X000000FF);
+
+	      //	    std::cout<<"recolor:"<<old_r<<","<<old_g<<","<<old_b<<"\n";
+	      surface_lock lock(nsurf);
+	      Uint32* beg = lock.pixels();
+	      Uint32* end = beg + nsurf->w*surf->h;
+
+	      //const Uint16 old_grey = (Uint16)((77*(Uint16) old_r +
+	      //                                  150*(Uint16)old_g +
+	      //                                  29*(Uint16)old_b) / 256);
+
+	      const Uint16 old_avg = (Uint16)(((Uint16) old_r +
+						(Uint16) old_g +
+						(Uint16) old_b) / 3);
+	      //calculate new color
+	      Uint8 new_r, new_g, new_b;
+
+	      if(reference_avg && old_avg <= reference_avg){
+		float old_rat = ((float)old_avg)/reference_avg;
+		new_r=(Uint8)( old_rat * new_red   + (1 - old_rat) * min_red);
+		new_g=(Uint8)( old_rat * new_green + (1 - old_rat) * min_green);
+		new_b=(Uint8)( old_rat * new_blue  + (1 - old_rat) * min_blue);
+	      }else if(255 - reference_avg){
+		float old_rat = ((float) 255 - old_avg) / (255 - reference_avg);
+		new_r=(Uint8)( old_rat * new_red   + (1 - old_rat) * max_red);
+		new_g=(Uint8)( old_rat * new_green + (1 - old_rat) * max_green);
+		new_b=(Uint8)( old_rat * new_blue  + (1 - old_rat) * max_blue);
+	      }else{
+		      new_r=0; new_g=0; new_b=0; //supress warning
+		      wassert(false);
+		//should never get here
+		//would imply old_avg > reference_avg = 255
+	      }
+
+	      while(beg != end) {
+                        Uint8 red, green, blue, alpha;
+                        SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+                        if(red==old_r && green==old_g && blue==old_b){
+                          *beg = SDL_MapRGBA(nsurf->format,new_r,new_g,new_b,alpha);
+                        }
+                        ++beg;
+	      }
+	    }
+	}
+        return create_optimized_surface(nsurf);
+  }
+      return NULL;
+}
+
 surface brighten_image(surface const &surf, fixed_t amount)
 {
 	if(surf == NULL) {
@@ -641,10 +883,10 @@ surface flip_surface(surface const &surf)
 		surface_lock lock(nsurf);
 		Uint32* const pixels = lock.pixels();
 
-		for(size_t y = 0; y != nsurf->h; ++y) {
-			for(size_t x = 0; x != nsurf->w/2; ++x) {
-				const size_t index1 = y*nsurf->w + x;
-				const size_t index2 = (y+1)*nsurf->w - x - 1;
+		for(int y = 0; y != nsurf->h; ++y) {
+			for(int x = 0; x != nsurf->w/2; ++x) {
+				const int index1 = y*nsurf->w + x;
+				const int index2 = (y+1)*nsurf->w - x - 1;
 				std::swap(pixels[index1],pixels[index2]);
 			}
 		}
@@ -670,10 +912,10 @@ surface flop_surface(surface const &surf)
 		surface_lock lock(nsurf);
 		Uint32* const pixels = lock.pixels();
 
-		for(size_t x = 0; x != nsurf->w; ++x) {
-			for(size_t y = 0; y != nsurf->h/2; ++y) {
-				const size_t index1 = y*nsurf->w + x;
-				const size_t index2 = (nsurf->h-y-1)*surf->w + x;
+		for(int x = 0; x != nsurf->w; ++x) {
+			for(int y = 0; y != nsurf->h/2; ++y) {
+				const int index1 = y*nsurf->w + x;
+				const int index2 = (nsurf->h-y-1)*surf->w + x;
 				std::swap(pixels[index1],pixels[index2]);
 			}
 		}
@@ -778,7 +1020,7 @@ SDL_Rect get_non_transperant_portion(surface const &surf)
 	surface_lock lock(nsurf);
 	const Uint32* const pixels = lock.pixels();
 
-	size_t n;
+	int n;
 	for(n = 0; n != nsurf->h; ++n) {
 		const Uint32* const start_row = pixels + n*nsurf->w;
 		const Uint32* const end_row = start_row + nsurf->w;
@@ -802,7 +1044,7 @@ SDL_Rect get_non_transperant_portion(surface const &surf)
 	res.h = nsurf->h - res.y - n;
 
 	for(n = 0; n != nsurf->w; ++n) {
-		size_t y;
+		int y;
 		for(y = 0; y != nsurf->h; ++y) {
 			const Uint32 pixel = pixels[y*nsurf->w + n];
 			if(calc(pixel))
@@ -816,7 +1058,7 @@ SDL_Rect get_non_transperant_portion(surface const &surf)
 	res.x = n;
 
 	for(n = 0; n != nsurf->w-res.x; ++n) {
-		size_t y;
+		int y;
 		for(y = 0; y != nsurf->h; ++y) {
 			const Uint32 pixel = pixels[y*nsurf->w + surf->w - n - 1];
 			if(calc(pixel))
@@ -855,6 +1097,7 @@ SDL_Color inverse(const SDL_Color& colour) {
 	inverse.r = 255 - colour.r;
 	inverse.g = 255 - colour.g;
 	inverse.b = 255 - colour.b;
+	inverse.unused = 0;
 	return inverse;
 }
 
@@ -936,11 +1179,6 @@ void surface_restorer::cancel()
 
 void draw_rectangle(int x, int y, int w, int h, Uint32 colour,surface target)
 {
-	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
-		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
-		          << "," << w << "," << h << "\n";
-		return;
-	}
 
 	SDL_Rect top = {x,y,w,1};
 	SDL_Rect bot = {x,y+h-1,w,1};
@@ -957,11 +1195,6 @@ void draw_solid_tinted_rectangle(int x, int y, int w, int h,
                                  int r, int g, int b,
                                  double alpha, surface target)
 {
-	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
-		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
-		          << "," << w << "," << h << "\n";
-		return;
-	}
 
 	SDL_Rect rect = {x,y,w,h};
 	fill_rect_alpha(rect,SDL_MapRGB(target->format,r,g,b),Uint8(alpha*255),target);
