@@ -1,4 +1,4 @@
-/* $Id: sdl_utils.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: sdl_utils.hpp 9174 2005-12-15 17:33:20Z darthfool $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -15,12 +15,14 @@
 
 #include "scoped_resource.hpp"
 #include "util.hpp"
+#include "color_range.hpp"
 
 #include "SDL.h"
 
 #include <cstdlib>
 #include <iostream>
 #include <string>
+#include <vector>
 
 //older versions of SDL don't define the
 //mouse wheel macros, so define them ourselves
@@ -104,6 +106,9 @@ surface scale_surface(surface const &surf, int w, int h);
 surface scale_surface_blended(surface const &surf, int w, int h);
 surface adjust_surface_colour(surface const &surf, int r, int g, int b);
 surface greyscale_image(surface const &surf);
+surface darken_image(surface const &surf);
+surface recolor_image(surface surf, const color_range& new_rgb, const std::vector<Uint32>& old_rgb);
+
 surface brighten_image(surface const &surf, fixed_t amount);
 surface get_surface_portion(surface const &surf, SDL_Rect &rect);
 surface adjust_surface_alpha(surface const &surf, fixed_t amount);
@@ -224,5 +229,4 @@ void draw_rectangle(int x, int y, int w, int h, Uint32 colour, surface tg);
 void draw_solid_tinted_rectangle(int x, int y, int w, int h,
                                  int r, int g, int b,
 				 double alpha, surface target);
-
 #endif
