@@ -1,4 +1,4 @@
-/* $Id: binary_wml.cpp 8844 2005-11-13 16:17:11Z brunowolff $ */
+/* $Id: binary_wml.cpp 10931 2006-04-02 08:15:29Z boucman $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -15,6 +15,7 @@
 #include "global.hpp"
 
 #include "config.hpp"
+#include "loadscreen.hpp"
 #include "log.hpp"
 #include "serialization/binary_wml.hpp"
 
@@ -154,6 +155,7 @@ void write_compressed(std::ostream &out, config const &cfg, compression_schema &
 
 static void read_compressed_internal(config &cfg, std::istream &in, compression_schema &schema, int level)
 {
+	increment_binary_wml_progress();
 	if (level >= max_recursion_levels)
 		throw config::error("Too many recursion levels in compressed config read");
 
