@@ -1,4 +1,4 @@
-/* $Id: parser.cpp 8216 2005-09-12 21:37:41Z gruikya $ */
+/* $Id: parser.cpp 12583 2006-07-15 04:24:57Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -13,18 +13,18 @@
    See the COPYING file for more details.
 */
 
-#include "global.hpp"
+#include "serialization/parser.hpp"
 
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
+#include "loadscreen.hpp"
 #include "log.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
 #include "wesconfig.h"
 #include "serialization/binary_wml.hpp"
-#include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
 #include "serialization/string_utils.hpp"
 #include "serialization/tokenizer.hpp"
@@ -36,7 +36,7 @@
 #define WRN_CF LOG_STREAM(warn, config)
 #define LOG_CF LOG_STREAM(info, config)
 
-static const size_t max_recursion_levels = 100;
+static const size_t max_recursion_levels = 1000;
 
 namespace {
 
@@ -112,6 +112,7 @@ void parser::operator()(std::string* error_log)
 
 			*error_log += e.message + '\n';
 		}
+		increment_parser_progress();
 	} while (tok_.current_token().type != token::END);
 
 	// The main element should be there. If it is not, this is a parser error.
@@ -215,6 +216,12 @@ void parser::parse_variable()
 	if(variables.back().empty())
 		error(_("Empty variable name"));
 
+	{
+		for(std::vector<std::string>::iterator curvar = variables.begin(); curvar != variables.end(); ++curvar) {
+			cfg[*curvar] = "";
+		}
+	}
+
 	std::vector<std::string>::const_iterator curvar = variables.begin();
 
 	bool ignore_next_newlines = false;
@@ -226,6 +233,7 @@ void parser::parse_variable()
 		case ',':
 			if ((curvar+1) != variables.end()) {
 				curvar++;
+				cfg[*curvar] = "";
 				continue;
 			} else {
 				cfg[*curvar] += ",";
@@ -340,10 +348,10 @@ static std::string escaped_string(const std::string& value) {
 	return std::string(res.begin(), res.end());
 }
 
-static void write_internal(config const &cfg, std::ostream &out, std::string textdomain, size_t tab = 0)
+static void write_internal(config const &cfg, std::ostream &out, std::string& textdomain, size_t tab = 0)
 {
 	if (tab > max_recursion_levels)
-		return;
+		throw config::error("Too many recursion levels in config write");
 
 	for(string_map::const_iterator i = cfg.values.begin(), i_end = cfg.values.end(); i != i_end; ++i) {
 		if (!i->second.empty()) {
@@ -409,5 +417,6 @@ static void write_internal(config const &cfg, std::ostream &out, std::string tex
 
 void write(std::ostream &out, config const &cfg)
 {
-	write_internal(cfg, out, PACKAGE);
+	std::string textdomain = PACKAGE;
+	write_internal(cfg, out, textdomain);
 }
