@@ -1,4 +1,4 @@
-/* $Id: string_utils.cpp 12503 2006-07-11 20:57:03Z noyga $ */
+/* $Id: string_utils.cpp 13744 2006-09-29 12:59:57Z queenkiller $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -220,7 +220,7 @@ private:
 
 };
 
-std::string interpolate_variables_into_string(const std::string &str, const string_map *symbols)
+std::string interpolate_variables_into_string(const std::string &str, const string_map * const symbols)
 {
 	string_map_variable_set set(*symbols);
 	return do_interpolation(str, set);
@@ -283,11 +283,13 @@ std::string &unescape(std::string &str)
 
 bool string_bool(const std::string& str,bool def)
 {
-	if(str == "yes" || str == "on" || str == "true" || lexical_cast_default<int>(str)) {
-		return true;
-	}
-	if(str == "no" || str == "off" || str == "false" || (str != "" && !lexical_cast_default<int>(str))) {
-		return false;
+	if(str != "") {
+		if(str == "yes" || str == "on" || str == "true" || lexical_cast_default<int>(str)) {
+			return true;
+		}
+		if(str == "no" || str == "off" || str == "false" || !lexical_cast_default<int>(str)) {
+			return false;
+		}
 	}
 	return def;
 }
@@ -435,6 +437,13 @@ wchar_t utf8_iterator::operator*() const
 	return current_char;
 }
 
+bool utf8_iterator::next_is_end()
+{
+	if(current_substr.second == string_end)
+		return true;
+	return false;
+}
+
 const std::pair<std::string::const_iterator, std::string::const_iterator>& utf8_iterator::substr() const
 {
 	return current_substr;
@@ -723,4 +732,3 @@ std::string vgettext(const char *msgid, const utils::string_map& symbols)
 	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
 	return msg;
 }
-
