@@ -1,4 +1,4 @@
-/* $Id: game.cpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
+/* $Id: game.cpp 14736 2006-12-19 10:37:39Z edb $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -28,9 +28,14 @@ int game::id_num = 1;
 game::game(const player_map& info) : player_info_(&info), id_(id_num++), sides_taken_(9), side_controllers_(9), started_(false), description_(NULL), end_turn_(0), allow_observers_(true), all_observers_muted_(false)
 {}
 
+void game::set_owner(network::connection player)
+{
+	owner_ = player;
+}
+
 bool game::is_owner(network::connection player) const
 {
-	return (!players_.empty() && player == players_.front());
+	return (player == owner_);
 }
 
 bool game::is_member(network::connection player) const
@@ -40,7 +45,7 @@ bool game::is_member(network::connection player) const
 
 bool game::is_needed(network::connection player) const
 {
-	return (!players_.empty() && player == players_.front());
+	return (!players_.empty() && player == owner_);
 }
 
 bool game::is_observer(network::connection player) const
@@ -126,12 +131,14 @@ void game::start_game()
 	//take control of any sides if they happen to have the same name as one of the descriptions
 	config::child_itors sides = level_.child_range("side");
 	for(; sides.first != sides.second; ++sides.first) {
-		(**sides.first)["controller"] = "human";
+		if ((**sides.first)["controller"] != "null")
+			(**sides.first)["controller"] = "human";
 	}
 
 	describe_slots();
 	if(description()) {
-		description()->values["turn"] = describe_turns(1,level()["turns"]);
+		int turn =  lexical_cast_default<int>(level()["turn_at"], 1);
+		description()->values["turn"] = describe_turns(turn, level()["turns"]);
 	}
 
 	allow_observers_ = level_["observer"] != "no";
@@ -187,7 +194,7 @@ bool game::take_side(network::connection player, const config& cfg)
 					sides_taken_[side_index] = true;
 					config new_cfg = cfg;
 					new_cfg["side"] = (**i)["side"];
-					network::queue_data(new_cfg, players_.front());
+					network::queue_data(new_cfg, owner_);
 					return true;
 				}
 			}
@@ -196,10 +203,29 @@ bool game::take_side(network::connection player, const config& cfg)
 		return false;
 	}
 	// else take the current side
-	side_controllers_[side_index] = "network";
+	if (player == owner_ && started_ && !cfg["controller"].empty()) {
+		//if the owner have transfer side to an ai or an human in game
+		//fake a "change_controller" command so other player update controller name
+		config fake;
+		config& change = fake.add_child("change_controller");
+		change["side"] = side;
+		change["player"] = cfg["name"];
+		change["controller"] = "network";
+		send_data(fake, owner_); //send change to all except owner
+		//send a server message displaying new controller name
+		send_data(construct_server_message(cfg["name"] + " takes control of side " + side));
+
+		//update level_ so observer who join display the good player name
+		config::child_itors it = level_.child_range("side");
+		it.first += side_index;
+		wassert(it.first != it.second);
+		(**it.first)["current_player"] = cfg["name"];
+		side_controllers_[side_index] = cfg["controller"];
+	} else
+		side_controllers_[side_index] = "network";
 	sides_.insert(std::pair<network::connection, size_t>(player, side_index));
 	sides_taken_[side_index] = true;
-	network::queue_data(cfg, players_.front());
+	network::queue_data(cfg, owner_);
 	return true;
 }
 
@@ -240,9 +266,12 @@ void game::update_side_data()
 			}
 			side_index = static_cast<size_t>(side_num - 1);
 
-			if (! sides_taken_[side_index]){
+			if (!sides_taken_[side_index]){
 				if((**sd)["controller"] == "network") {
 					side_controllers_[side_index] = "network";
+					/*TODO : change the next line (and all related code :o) to :
+					  if((**sd)["curent_player"] == info->second.name()) {
+					  I won't make it before 1.2 is out in case of regression */
 					if((**sd)["user_description"] == info->second.name()) {
 						sides_.insert(std::pair<network::connection, size_t>(*player, side_index));
 						sides_taken_[side_index] = true;
@@ -258,7 +287,7 @@ void game::update_side_data()
 					side_found = true;
 				}
 				else if((**sd)["controller"] == "human") {
-					sides_.insert(std::pair<network::connection,size_t>(players_.front(),side_index));
+					sides_.insert(std::pair<network::connection,size_t>(owner_,side_index));
 					sides_taken_[side_index] = true;
 					side_found = true;
 					side_controllers_[side_index] = "human";
@@ -309,6 +338,7 @@ void game::update_side_data()
 
 const std::string& game::transfer_side_control(const config& cfg)
 {
+	bool host_leave = false;
 	const std::string& player = cfg["player"];
 
 	//find the socket for the player that is passed control
@@ -322,18 +352,16 @@ const std::string& game::transfer_side_control(const config& cfg)
 		}
 	}
 
-	if (!found){
-		static const std::string notfound = "Player/Observer not found";
+	static const std::string notfound = "Player/Observer not found";
+	if (!found)
 		return notfound;
-	}
 
 	user_vector::iterator i = find_connection(sock_entering, observers_);
 	user_vector::iterator j = find_connection(sock_entering, players_);
 
-	if (i == observers_.end() && j == players_.end()){
-		static const std::string notfound = "Player/Observer not found";
+	if (i == observers_.end() && j == players_.end())
 		return notfound;
-	}
+
 
 	const std::string& side = cfg["side"];
 	static const std::string invalid = "Invalid side number";
@@ -357,22 +385,24 @@ const std::string& game::transfer_side_control(const config& cfg)
 		return already;
 	}
 
+	//get the socket of the player that issued the command
+	bool host = false; //we need to save this information before the player is erased
+	network::connection sock;
+	bool foundCommandPlayer = false;
+	std::multimap<network::connection, size_t>::iterator oldside;
+	for (std::multimap<network::connection, size_t>::iterator s = sides_.begin(); s != sides_.end(); s++){
+		if (s->second == side_index){
+			sock = s->first;
+			foundCommandPlayer = true;
+			oldside = s;
+			host = is_needed(sock);
+			break;
+		}
+	}
+
 	//The player owns this side
 	if(cfg["own_side"] == "yes") {
-		//get the socket of the player that issued the command
-		network::connection sock;
-		bool found = false;
-		std::multimap<network::connection, size_t>::iterator oldside;
-		for (std::multimap<network::connection, size_t>::iterator s = sides_.begin(); s != sides_.end(); s++){
-			if (s->second == side_index){
-				sock = s->first;
-				found = true;
-				oldside = s;
-				break;
-			}
-		}
-
-		if (!found){
+		if (!foundCommandPlayer){
 			static const std::string player_not_found = "This side is not listed for the game";
 			return player_not_found;
 		}
@@ -386,12 +416,15 @@ const std::string& game::transfer_side_control(const config& cfg)
 
 		//if a player gives up their last side, make them an observer
 		if (sides_.count(sock) == 1){
-			//we need to save this information before the player is erased
-			bool host = is_needed(sock);
-
 			observers_.push_back(*p);
 			players_.erase(p);
 
+			if (sock == owner_) {
+				host_leave = true;
+				if (!players_.empty())
+					owner_ = players_.front();
+			}
+
 			//tell others that the player becomes an observer
 			config cfg_observer = construct_server_message(find_player(sock)->name() + " becomes observer");
 			send_data(cfg_observer);
@@ -425,13 +458,33 @@ const std::string& game::transfer_side_control(const config& cfg)
 	config& change = response.add_child("change_controller");
 
 	change["side"] = side;
+	change["player"] = player;
 
 	change["controller"] = "network";
-	network::queue_data(response,players_.front());
+	network::send_data(response, owner_);
 
 	change["controller"] = "human";
 	network::queue_data(response, sock_entering);
 
+	//update level so observer who join display the good player name
+	config::child_itors it = level_.child_range("side");
+	it.first += side_index;
+	wassert(it.first != it.second);
+	(**it.first)["current_player"] = player;
+
+	//if the host left and there are ai sides, transfer them to the new host
+	if (host_leave) {
+		for (unsigned int i = 0; i < side_controllers_.size(); i++){
+			if (side_controllers_[i] == "ai"){
+				change["side"] = lexical_cast<std::string, unsigned int>(i + 1);
+				change["controller"] = "ai";
+				network::queue_data(response, owner_);
+				sides_.insert(std::pair<const network::connection,size_t>(owner_, i));
+			}
+		}
+		sides_.erase(sock);
+	}
+
 	if(sides_.count(sock_entering) < 2) {
 		//send everyone a message saying that the observer who is taking the side has quit
 		config observer_quit;
@@ -446,35 +499,24 @@ const std::string& game::transfer_side_control(const config& cfg)
 	return success;
 }
 
-size_t game::available_slots() const
-{
-	size_t available_slots = 0;
-	const config::child_list& sides = level_.get_children("side");
-	for(config::child_list::const_iterator i = sides.begin(); i != sides.end(); ++i) {
-		//std::cerr << "side controller: '" << (**i)["controller"] << "'\n";
-		//std::cerr << "description: '" << (**i)["description"] << "'\n";
-		if((**i)["controller"] == "network" && (**i)["description"].empty()) {
-			++available_slots;
-		}
-	}
-
-	return available_slots;
-}
-
 bool game::describe_slots()
 {
 	if(description() == NULL)
 		return false;
 
-	const int val = int(available_slots());
+	int available_slots = 0;
 	char buf[50];
 	int num_sides = level_.get_children("side").size();
-	for(config::child_list::const_iterator it = level_.get_children("side").begin(); it != level_.get_children("side").end(); ++it) {
-		if((**it)["allow_player"] == "no") {
+	int i = 0;
+	for(config::child_list::const_iterator it = level_.get_children("side").begin(); it != level_.get_children("side").end(); ++it, ++i) {
+		if((**it)["allow_player"] == "no" || (**it)["controller"] == "null") {
 			num_sides--;
+		} else {
+			if(!sides_taken_[i])
+			available_slots++;
 		}
 	}
-	snprintf(buf,sizeof(buf),"%d/%d",val,num_sides);
+	snprintf(buf,sizeof(buf), "%d/%d", available_slots, num_sides);
 
 	if(buf != (*description())["slots"]) {
 		description()->values["slots"] = buf;
@@ -651,6 +693,12 @@ void game::add_player(network::connection player, bool observer)
 void game::remove_player(network::connection player, bool notify_creator)
 {
 	LOG_SERVER << "removing player...\n";
+
+	bool host = false;
+	if (players_.size() > 0){
+		host = player == owner_;
+	}
+
 	{
 		const user_vector::iterator itor = std::find(players_.begin(),players_.end(),player);
 
@@ -665,20 +713,49 @@ void game::remove_player(network::connection player, bool notify_creator)
 			observers_.erase(itor);
 	}
 
+	if (host && !players_.empty())
+		owner_ = players_.front();
+
 	LOG_SERVER << debug_player_info();
 	bool observer = true;
-	std::pair<std::multimap<network::connection,size_t>::const_iterator,
-	          std::multimap<network::connection,size_t>::const_iterator> sides = sides_.equal_range(player);
-	while(sides.first != sides.second) {
+
+	//check for ai sides first and drop them, too, if the host left
+	if (host){
+		bool ai_transfer = false;
+		//can't do this with an iterator, because it doesn't know the side_index
+		for (size_t side = 0; side < side_controllers_.size(); ++side){
+			//send the host a notification of removal of this side
+			if(notify_creator && players_.empty() == false && side_controllers_[side] == "ai") {
+				ai_transfer = true;
+				config drop;
+				drop["side_drop"] = lexical_cast<std::string, size_t>(side + 1);
+				drop["controller"] = "ai";
+				network::queue_data(drop, owner_);
+				sides_taken_[side] = false;
+				observer = false;
+			}
+		}
+		if (ai_transfer) {
+			std::string msg = "AI transferred to new host";
+			send_data(construct_server_message(msg));
+		}
+	}
+
+	//look for all sides the player controlled and drop them
+	std::multimap<network::connection, size_t>::const_iterator side;
+	for (side = sides_.find(player); side != sides_.end(); ++side){
 		//send the host a notification of removal of this side
-		if(notify_creator && players_.empty() == false) {
-			config drop;
-			drop["side_drop"] = lexical_cast<std::string, size_t>(sides.first->second + 1);
-			network::queue_data(drop, players_.front());
+		if(side->first == player) {
+			if (notify_creator && players_.empty() == false) {
+				config drop;
+				drop["side_drop"] = lexical_cast<std::string, size_t>(side->second + 1);
+				drop["controller"] = side_controllers_[side->second];
+				network::queue_data(drop, owner_);
+			}
+			side_controllers_[side->second] = "null";
+			sides_taken_[side->second] = false;
+			observer = false;
 		}
-		sides_taken_[sides.first->second] = false;
-		observer = false;
-		++sides.first;
 	}
 	if(!observer)
 		sides_.erase(player);
