@@ -1,4 +1,4 @@
-/* $Id: game.hpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
+/* $Id: game.hpp 13699 2006-09-27 09:48:06Z edb $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -32,12 +32,13 @@ class game
 public:
 	game(const player_map& info);
 
-	bool is_owner(network::connection player) const;
-	bool is_member(network::connection player) const;
-	bool is_needed(network::connection player) const;
-	bool is_observer(network::connection player) const;
-	bool is_muted_observer(network::connection player) const;
-	bool is_player(network::connection player) const;
+	void set_owner(const network::connection player);
+	bool is_owner(const network::connection player) const;
+	bool is_member(const network::connection player) const;
+	bool is_needed(const network::connection player) const;
+	bool is_observer(const network::connection player) const;
+	bool is_muted_observer(const network::connection player) const;
+	bool is_player(const network::connection player) const;
 	bool all_observers_muted() const;
 
 	bool observers_can_label() const;
@@ -56,8 +57,6 @@ public:
 
 	const std::string& transfer_side_control(const config& cfg);
 
-	size_t available_slots() const;
-
 	//function to set the description to the number of slots
 	//returns true if the number of slots has changed
 	bool describe_slots();
@@ -145,6 +144,7 @@ private:
 
 	static int id_num;
 	int id_;
+	network::connection owner_;
 	user_vector players_;
 	user_vector observers_;
 	user_vector muted_observers_;
