@@ -1,4 +1,4 @@
-/* $Id: server.cpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
+/* $Id: server.cpp 14083 2006-10-26 15:07:17Z edb $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -452,6 +452,46 @@ void server::run()
 							i->send_data(construct_server_message(pl_name + " has disconnected",*i));
 						}
 						i->remove_player(e.socket);
+						if(i->nplayers() == 0) {
+
+							//tell all other players the game is over,
+							//because the last player has left
+							config cfg;
+							cfg.add_child("leave_game");
+							i->send_data(cfg);
+
+							//delete the game's description
+							config* const gamelist = initial_response_.child("gamelist");
+							wassert(gamelist != NULL);
+							const config::child_itors vg = gamelist->child_range("game");
+
+							const config::child_iterator desc = std::find(vg.first,vg.second,i->description());
+							if(desc != vg.second) {
+								gamelist->remove_child("game",desc - vg.first);
+							}
+
+							//update the state of the lobby to players in it.
+							//We have to sync the state of the lobby so we can
+							//send it to the players leaving the game
+							lobby_players_.send_data(sync_initial_response());
+
+							//set the availability status for all quitting players
+							for(player_map::iterator pl = players_.begin(); pl != players_.end(); ++pl) {
+								if(i->is_member(pl->first)) {
+									pl->second.mark_available(true,"");
+								}
+							}
+
+							//put the players back in the lobby and send
+							//them the game list and user list again
+							i->send_data(initial_response_);
+							metrics_.game_terminated(i->termination_reason());
+							lobby_players_.add_players(*i);
+							games_.erase(i);
+
+							//now sync players in the lobby again, to remove the game
+							lobby_players_.send_data(sync_initial_response());
+						}
 					}
 				}
 
@@ -744,6 +784,7 @@ void server::process_data_from_player_in_lobby(const network::connection sock, c
 		games_.push_back(game(players_));
 		games_.back().level() = *create_game;
 		lobby_players_.remove_player(sock);
+		games_.back().set_owner(sock);
 		games_.back().add_player(sock);
 
 		//store the game data here at the moment
@@ -871,7 +912,8 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 				const config& msg = construct_server_message(result,*g);
 				network::send_data(msg,sock);
 			}
-
+			g->describe_slots();
+			lobby_players_.send_data(sync_initial_response());
 			return;
 		}
 	}
@@ -940,7 +982,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 	//if the owner is banning someone from the game
 	if(g->is_owner(sock) && (data.child("ban") != NULL || data.child("kick") != NULL)) {
 		std::string name;
-		bool ban;
+		bool ban = false;;
 		if (data.child("ban") != NULL) {
 			const config& u = *data.child("ban");
 			name = u["username"];
@@ -1090,30 +1132,27 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 
 	const string_map::const_iterator side = data.values.find("side");
 	if(side != data.values.end()) {
-		if (g->is_observer(sock)){
+		const bool res = g->take_side(sock,data);
+		config response;
+		if(res) {
+			std::cerr << "player joined side\n";
+			response["side_secured"] = side->second;
+
+			//update the number of available slots
+			const bool res = g->describe_slots();
+			if(res) {
+				lobby_players_.send_data(sync_initial_response());
+			}
+		} else if (g->is_observer(sock)) {
 			const config& p_msg = construct_server_message("Sorry " + g->find_player(sock)->name() + ", someone else entered before you.",*g);
 			network::send_data(p_msg, sock);
 			return;
+		} else {
+			response["failed"] = "yes";
 		}
-		else{
-			const bool res = g->take_side(sock,data);
-			config response;
-			if(res) {
-				std::cerr << "player joined side\n";
-				response["side_secured"] = side->second;
 
-				//update the number of available slots
-				const bool res = g->describe_slots();
-				if(res) {
-					lobby_players_.send_data(sync_initial_response());
-				}
-			} else {
-				response["failed"] = "yes";
-			}
-
-			network::send_data(response,sock);
-			return;
-		}
+		network::send_data(response,sock);
+		return;
 	}
 
 	if(data.child("start_game")) {
@@ -1131,7 +1170,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 		g->remove_player(sock);
 		g->describe_slots();
 
-		if(g->nplayers() == 0) {
+		if( (g->nplayers() == 0) || (needed && (!g->started())) ) {
 
 			//tell all other players the game is over,
 			//because the last player has left
