@@ -1,4 +1,4 @@
-/* $Id: sound.cpp 12754 2006-07-24 19:13:05Z boucman $ */
+/* $Id: sound.cpp 14081 2006-10-26 10:22:11Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -210,7 +210,7 @@ bool init_sound() {
 			return false;
 
 	if(!mix_ok) {
-		if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,2,buf_size) == -1) {
+		if(Mix_OpenAudio(preferences::sample_rate(), MIX_DEFAULT_FORMAT, 2, buf_size) == -1) {
 			mix_ok = false;
 			ERR_AUDIO << "Could not initialize audio: " << Mix_GetError() << "\n";
 			return false;
