@@ -1,4 +1,4 @@
-/* $Id: statistics.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: statistics.hpp 12583 2006-07-15 04:24:57Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -44,6 +44,9 @@ namespace statistics
 
 		//expected value for damage inflicted/taken * 100, based on probability
 		//to hit, Use this long term to see how lucky a side is
+		// FIXME: Since integers are used, rounding errors accumulate. Also,
+		// slow isn't accounted for properly. Rusty's simulator could be used
+		// obtain valid values.
 		int expected_damage_inflicted, expected_damage_taken;
 	};
 
@@ -55,6 +58,7 @@ namespace statistics
 		~disabler();
 	};
 
+
 	struct scenario_context
 	{
 		scenario_context(const std::string& name);
@@ -63,19 +67,19 @@ namespace statistics
 
 	struct attack_context
 	{
-		attack_context(const unit& a, const unit& d, const battle_stats& stats);
+		attack_context(const unit& a, const unit& d, int a_cth, int d_cth);
 		~attack_context();
 
 		enum ATTACK_RESULT { MISSES, HITS, KILLS };
 
-		void attack_result(ATTACK_RESULT res);
-		void defend_result(ATTACK_RESULT res);
+		void attack_result(ATTACK_RESULT res, int damage);
+		void defend_result(ATTACK_RESULT res, int damage);
 
 	private:
 
 		std::string attacker_type, defender_type;
-		battle_stats bat_stats;
 		int attacker_side, defender_side;
+		int chance_to_hit_defender, chance_to_hit_attacker;
 		std::string attacker_res, defender_res;
 
 		stats& attacker_stats();
@@ -85,6 +89,7 @@ namespace statistics
 	void recruit_unit(const unit& u);
 	void recall_unit(const unit& u);
 	void un_recall_unit(const unit& u);
+	void un_recruit_unit(const unit& u);
 
 	void advance_unit(const unit& u);
 
