@@ -1,4 +1,4 @@
-/* $Id: team.cpp 11534 2006-05-19 08:15:18Z rusty $ */
+/* $Id: team.cpp 13883 2006-10-10 14:15:31Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -102,6 +102,10 @@ team::team_info::team_info(const config& cfg)
 	if(save_id.empty()) {
 		save_id = cfg["description"];
 	}
+	current_player = cfg["current_player"];
+	if (current_player.empty()) {
+		current_player = save_id;
+	}
 
 	countdown_time = cfg["countdown_time"];
 
@@ -121,9 +125,9 @@ team::team_info::team_info(const config& cfg)
 	std::map<int, color_range>::iterator global_rgb = game_config::team_rgb_range.find(side);
 
 	if(temp_rgb.size()){
-	  team_color_range_[side] = color_range(temp_rgb);
+		team_color_range_[side] = color_range(temp_rgb);
 	}else if(global_rgb != game_config::team_rgb_range.end()){
-	  team_color_range_[side] = global_rgb->second;
+		team_color_range_[side] = global_rgb->second;
 	}
 
 	flag = cfg["flag"];
@@ -221,6 +225,10 @@ team::team_info::team_info(const config& cfg)
 		recruitment_pattern.push_back("archer");
 	}
 
+	//keep a copy of the initial recruitment_pattern, since it can be changed
+	//on a per-time-of-day or per-turn basis inside [ai] sections
+	global_recruitment_pattern = recruitment_pattern;
+
 	//additional targets
 	config::const_child_itors tgts;
 	for(tgts = cfg.child_range("target"); tgts.first != tgts.second; ++tgts.first) {
@@ -254,6 +262,7 @@ void team::team_info::write(config& cfg) const
 	cfg["name"] = name;
 	cfg["team_name"] = team_name;
 	cfg["save_id"] = save_id;
+	cfg["current_player"] = current_player;
 	cfg["flag"] = flag;
 	cfg["description"] = description;
 	cfg["objectives"] = objectives;
@@ -309,8 +318,9 @@ void team::team_info::write(config& cfg) const
 	cfg["recruit"] = can_recruit_str.str();
 
 	std::stringstream recruit_pattern_str;
-	for(std::vector<std::string>::const_iterator p = recruitment_pattern.begin(); p != recruitment_pattern.end(); ++p) {
-		if(p != recruitment_pattern.begin())
+	std::vector<std::string> rp = global_recruitment_pattern;
+	for(std::vector<std::string>::const_iterator p = rp.begin(); p != rp.end(); ++p) {
+		if(p != rp.begin())
 			recruit_pattern_str << ",";
 
 		recruit_pattern_str << *p;
@@ -448,6 +458,11 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 		aiparams_.append(*i);
 	}
 
+    //get the recruitment pattern from the matching [ai] section, and fall back
+    //to tge global recruitment pattern otherwise
+	info_.recruitment_pattern = utils::split(aiparams_["recruitment_pattern"]);
+	if (info_.recruitment_pattern.empty())
+		info_.recruitment_pattern = info_.global_recruitment_pattern;
 	aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
 	caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
 }
@@ -480,6 +495,10 @@ void team::set_action_bonus_count(int count)
 	action_bonus_count_=count;
 }
 
+void team::set_current_player(const std::string player){
+	info_.current_player = player;
+}
+
 const std::set<std::string>& team::recruits() const
 {
 	return info_.can_recruit;
@@ -599,6 +618,10 @@ const std::string& team::save_id() const
 	return info_.save_id;
 }
 
+const std::string& team::current_player() const{
+	return info_.current_player;
+}
+
 void team::set_objectives(const t_string& new_objectives, bool silently)
 {
 	info_.objectives = new_objectives;
@@ -986,3 +1009,18 @@ void team::log_recruitable(){
 	}
 	LOG_NG << "Added all recruitable units\n";
 }
+
+namespace player_teams {
+int village_owner(const gamemap::location& loc)
+{
+	if(! teams) {
+		return -1;
+	}
+	for(size_t i = 0; i != teams->size(); ++i) {
+		if((*teams)[i].owns_village(loc))
+			return i;
+	}
+	return -1;
+}
+}
+
