@@ -1,4 +1,4 @@
-/* $Id: team.hpp 11534 2006-05-19 08:15:18Z rusty $ */
+/* $Id: team.hpp 13883 2006-10-10 14:15:31Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -69,10 +69,12 @@ public:
 		std::string income;
 		int income_per_village;
 		std::set<std::string> can_recruit;
+		std::vector<std::string> global_recruitment_pattern;
 		std::vector<std::string> recruitment_pattern;
 		std::vector<int> enemies;
 		std::string team_name;
 		std::string save_id;
+		std::string current_player;
 		std::string countdown_time;
 		int action_bonus_count;
 
@@ -93,7 +95,7 @@ public:
 		std::string ai_algorithm;
 
 		std::vector<config> ai_params;
-	        config ai_memory_;
+		config ai_memory_;
 
 		int villages_per_scout;
 		double leader_value, village_value;
@@ -107,7 +109,7 @@ public:
 		int colour;
 	};
 
-        static std::map<int, color_range> team_color_range_;
+	static std::map<int, color_range> team_color_range_;
 	team(const config& cfg, int gold=100);
 
 	void write(config& cfg) const;
@@ -129,12 +131,14 @@ public:
 	void set_countdown_time(int amount);
 	int action_bonus_count() const;
 	void set_action_bonus_count(int count);
+	void set_current_player(const std::string player);
 
 	const std::set<std::string>& recruits() const;
 	std::set<std::string>& recruits();
 	const std::vector<std::string>& recruitment_pattern() const;
 	const std::string& name() const;
 	const std::string& save_id() const;
+	const std::string& current_player() const;
 
 	void set_objectives(const t_string& new_objectives, bool silently=false);
 	void reset_objectives_changed();
@@ -187,8 +191,8 @@ public:
 
 	const std::string& ai_algorithm() const;
 	const config& ai_parameters() const;
-        const config& ai_memory() const;
-        void set_ai_memory(const config& ai_mem);
+	const config& ai_memory() const;
+	void set_ai_memory(const config& ai_mem);
 
 	double leader_value() const;
 	double village_value() const;
@@ -229,7 +233,7 @@ public:
 	static const Uint32 get_side_rgb(int side);
 	static const Uint32 get_side_rgb_max(int side);
 	static const Uint32 get_side_rgb_min(int side);
-        static const SDL_Color get_side_colour(int side);
+	static const SDL_Color get_side_colour(int side);
 	static int get_side_colour_index(int side);
 
 	void log_recruitable();
@@ -276,6 +280,10 @@ struct teams_manager {
 	bool is_observer();
 };
 
+namespace player_teams {
+	int village_owner(const gamemap::location& loc);
+}
+
 bool is_observer();
 
 //function which will validate a side. Trows game::game_error
