@@ -1,4 +1,4 @@
-/* $Id: terrain.hpp 7490 2005-07-16 18:51:56Z ott $ */
+/* $Id: terrain.hpp 10859 2006-03-29 12:09:39Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -34,16 +34,17 @@ public:
 	char letter() const;
 
 	//the underlying type of the terrain
-	const std::string& type() const;
+	const std::string& mvt_type() const;
+	const std::string& def_type() const;
+	const std::string& union_type() const;
 
 	bool is_nonnull() const;
-	bool is_light() const;
-	bool is_alias() const;
+	int light_modification() const;
 
 	int unit_height_adjust() const;
 	double unit_submerge() const;
 
-	bool gives_healing() const;
+	int gives_healing() const;
 	bool is_village() const;
 	bool is_castle() const;
 	bool is_keep() const;
@@ -57,15 +58,17 @@ private:
 	//terrain type. The 'type' is a list of the 'underlying types'
 	//of the terrain. This may simply be the same as the letter.
 	char letter_;
-	std::string type_;
+	std::string mvt_type_;
+	std::string def_type_;
+	std::string union_type_;
 
 	int height_adjust_;
 
 	double submerge_;
 
-	bool is_light_;
+	int light_modification_, heals_;
 
-	bool heals_, village_, castle_, keep_;
+	bool village_, castle_, keep_;
 };
 
 void create_terrain_maps(const std::vector<config*>& cfgs,
