@@ -1,4 +1,4 @@
-/* $Id: theme.cpp 11877 2006-06-05 20:02:25Z soliton $ */
+/* $Id: theme.cpp 14162 2006-11-05 17:20:46Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -596,7 +596,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 }
 
 theme::object& theme::add_object(const config& cfg){
-	theme::object& result = *(new theme::object());
+	static theme::object result;
 
 	const config* const main_map_cfg = cfg.child("main_map");
 	if(main_map_cfg != NULL) {
@@ -682,10 +682,9 @@ void theme::remove_object(std::string id){
 }
 
 void theme::set_object_location(theme::object& element, std::string rect_str, std::string ref_id){
-	theme::object& ref_element = *(new object());
+	theme::object ref_element = element;
 	if (ref_id.empty()) {
 		ref_id = element.get_id();
-		ref_element = element;
 	}
 	else {
 		ref_element = find_element(ref_id);
@@ -726,7 +725,8 @@ void theme::modify(const config* cfg){
 }
 
 theme::object& theme::find_element(std::string id){
-	theme::object* res = new theme::object();
+	static theme::object empty_object;
+	theme::object* res = &empty_object;
 	for (std::vector<theme::panel>::iterator p = panels_.begin(); p != panels_.end(); ++p){
 		if (p->get_id() == id) { res = &(*p); }
 	}
