@@ -1,4 +1,4 @@
-/* $Id: titlescreen.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: titlescreen.hpp 11957 2006-06-09 04:14:31Z sapient $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -19,8 +19,8 @@ class display;
 
 namespace gui {
 
-enum TITLE_RESULT { TUTORIAL = 0, NEW_CAMPAIGN, MULTIPLAYER, LOAD_GAME,
-                    CHANGE_LANGUAGE, EDIT_PREFERENCES, SHOW_ABOUT, QUIT_GAME, TITLE_CONTINUE };
+enum TITLE_RESULT { TUTORIAL = 0, NEW_CAMPAIGN, MULTIPLAYER, LOAD_GAME, GET_ADDONS,
+                    CHANGE_LANGUAGE, EDIT_PREFERENCES, SHOW_ABOUT, QUIT_GAME, TITLE_CONTINUE, SHOW_HELP, BEG_FOR_UPLOAD};
 
 TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip);
 
