@@ -1,4 +1,4 @@
-/* $Id: unit.cpp 12647 2006-07-19 18:54:27Z xan $ */
+/* $Id: unit.cpp 14848 2006-12-23 07:00:45Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -30,6 +30,7 @@
 #include "display.hpp"
 #include "gamestatus.hpp"
 #include "actions.hpp"
+#include "game_events.hpp"
 #include "sound.hpp"
 
 #include <ctime>
@@ -191,7 +192,7 @@ unit::unit(const game_data* gamedata, unit_map* unitmap, const gamemap* map,
 }
 
 unit::unit(const game_data& gamedata,const config& cfg) : movement_(0),
-			hold_position_(false), resting_(false),
+			hold_position_(false), resting_(false), state_(STATE_STANDING),
 			facing_(gamemap::location::NORTH_EAST),
 			flying_(false),anim_(NULL),unit_halo_(0),unit_anim_halo_(0),draw_bars_(false),gamedata_(&gamedata),
 			units_(NULL),map_(NULL), gamestatus_(NULL)
@@ -223,6 +224,7 @@ unit::unit(const game_data* gamedata, unit_map* unitmap, const gamemap* map,
 	gender_(dummy_unit ? gender : generate_gender(*t,use_traits)), resting_(false), state_(STATE_STANDING), facing_(gamemap::location::NORTH_EAST),draw_bars_(false),
            gamedata_(gamedata),units_(unitmap),map_(map),gamestatus_(game_status),teams_(teams)
 {
+	goto_ = gamemap::location();
 	side_ = side;
 	movement_ = 0;
 	attacks_left_ = 0;
@@ -255,6 +257,7 @@ unit::unit(const unit_type* t, int side, bool use_traits, bool dummy_unit, unit_
            gender_(dummy_unit ? gender : generate_gender(*t,use_traits)),state_(STATE_STANDING),facing_(gamemap::location::NORTH_EAST),draw_bars_(false),
 	   gamedata_(NULL), units_(NULL),map_(NULL),gamestatus_(NULL),teams_(NULL)
 {
+	goto_ = gamemap::location();
 	side_ = side;
 	movement_ = 0;
 	attacks_left_ = 0;
@@ -368,16 +371,33 @@ void unit::advance_to(const unit_type* t)
 	cfg_.clear_children("extra_anim");
 	cfg_.clear_children("death");
 	cfg_.clear_children("movement_anim");
+	cfg_.clear_children("leading_anim");
+	cfg_.clear_children("healing_anim");
+	cfg_.clear_children("standing_anim");
 	cfg_.clear_children("attack");
 	cfg_.clear_children("abilities");
-	cfg_.clear_children("event");
 	// clear cache of movement costs
 	movement_costs_.clear();
 
 	if(t->movement_type().get_parent()) {
 		cfg_ = cfg_.merge_with(t->movement_type().get_parent()->get_cfg());
 	}
+	//if unit has specific profile, remember it and have it after advaces
+	bool specific_profile = false;
+	std::string profile;
+	if (!id().empty() && gamedata_ != NULL)
+	{
+		specific_profile = (cfg_["profile"] != type().cfg_["profile"]);
+		if (specific_profile)
+		{
+			profile = cfg_["profile"];
+		}
+	}
 	cfg_ = cfg_.merge_with(t->cfg_);
+	if (specific_profile)
+	{
+	cfg_["profile"] = profile; 
+	}
 	cfg_.clear_children("male");
 	cfg_.clear_children("female");
 
@@ -419,6 +439,8 @@ void unit::advance_to(const unit_type* t)
 		cfg_["id"] = id_;
 		cfg_["gender_id"] = cfg_["gender"];
 	}
+	game_events::add_events(cfg_.get_children("event"),id_);
+	cfg_.clear_children("event");
 
 	set_state("poisoned","");
 	set_state("slowed","");
@@ -493,12 +515,6 @@ const std::string& unit::unit_description() const
 }
 
 
-const config::child_list unit::wml_events() const
-{
-	return cfg_.get_children("event");
-}
-
-
 const std::string& unit::undead_variation() const
 {
 	return undead_variation_;
@@ -1087,8 +1103,8 @@ void unit::read(const config& cfg)
 
 	bool type_set = false;
 	id_ = "";
+	wassert(gamedata_ != NULL);
 	if(cfg["type"] != "" && cfg["type"] != cfg["id"] || cfg["gender"] != cfg["gender_id"]) {
-		wassert(gamedata_ != NULL);
 		std::map<std::string,unit_type>::const_iterator i = gamedata_->unit_types.find(cfg["type"]);
 		if(i != gamedata_->unit_types.end()) {
 			advance_to(&i->second.get_gender_unit_type(gender_));
@@ -1147,12 +1163,55 @@ void unit::read(const config& cfg)
 		cfg_["profile"] = cfg["profile"];
 	}
 	
+	std::map<std::string,unit_type>::const_iterator uti = gamedata_->unit_types.find(cfg["type"]);
+	const unit_type* ut = NULL;
+	if(uti != gamedata_->unit_types.end()) {
+		ut = &uti->second.get_gender_unit_type(gender_).get_variation(variation_);
+	}
 	if(!type_set) {
-		for(config::const_child_itors range = cfg.child_range("attack");
-		    range.first != range.second; ++range.first) {
-			attacks_.push_back(attack_type(**range.first,id(),image_fighting((**range.first)["range"] == "ranged" ? attack_type::LONG_RANGE : attack_type::SHORT_RANGE)));
+		if(ut) {
+			if(cfg_["unit_description"] == "") {
+				cfg_["unit_description"] = ut->unit_description();
+			}
+			config t_atks;
+			config u_atks;
+			config::const_child_itors range;
+			for(range = ut->cfg_.child_range("attack");
+			    range.first != range.second; ++range.first) {
+				t_atks.add_child("attack",**range.first);
+			}
+			for(range = cfg.child_range("attack");
+			    range.first != range.second; ++range.first) {
+				u_atks.add_child("attack",**range.first);
+			}
+			u_atks = t_atks.merge_with(u_atks);
+			for(range = u_atks.child_range("attack");
+			    range.first != range.second; ++range.first) {
+				attacks_.push_back(attack_type(**range.first,id(),image_fighting((**range.first)["range"] == "ranged" ? attack_type::LONG_RANGE : attack_type::SHORT_RANGE)));
+			}
+			std::vector<attack_type>::iterator at;
+			for(at = attacks_.begin(); at != attacks_.end(); ++at) {
+				at->get_cfg().clear_children("animation");
+			}
+			for(at = attacks_b_.begin(); at != attacks_b_.end(); ++at) {
+				at->get_cfg().clear_children("animation");
+			}
+		} else {
+			for(config::const_child_itors range = cfg.child_range("attack");
+			    range.first != range.second; ++range.first) {
+				attacks_.push_back(attack_type(**range.first,id(),image_fighting((**range.first)["range"] == "ranged" ? attack_type::LONG_RANGE : attack_type::SHORT_RANGE)));
+			}
+		}
+	} else {
+		std::vector<attack_type>::iterator at;
+		for(at = attacks_.begin(); at != attacks_.end(); ++at) {
+			at->get_cfg().clear_children("animation");
+		}
+		for(at = attacks_b_.begin(); at != attacks_b_.end(); ++at) {
+			at->get_cfg().clear_children("animation");
 		}
 	}
+	cfg_.clear_children("attack");
 	const config* status_flags = cfg.child("status");
 	if(status_flags) {
 		for(string_map::const_iterator st = status_flags->values.begin(); st != status_flags->values.end(); ++st) {
@@ -1209,71 +1268,102 @@ void unit::read(const config& cfg)
 		custom_unit_description_ = generate_description();
 		cfg_["generate_description"] = "";
 	}
-
+	
 	if(!type_set) {
-		const config::child_list& defends = cfg_.get_children("defend");
-		for(config::child_list::const_iterator d = defends.begin(); d != defends.end(); ++d) {
-			defensive_animations_.push_back(defensive_animation(**d));
-		}
-		if(defensive_animations_.empty()) {
-			defensive_animations_.push_back(defensive_animation(absolute_image()));
-			// always have a defensive animation
-		}
-		const config::child_list& teleports = cfg_.get_children("teleport_anim");
-		for(config::child_list::const_iterator t = teleports.begin(); t != teleports.end(); ++t) {
-			teleport_animations_.push_back(unit_animation(**t));
-		}
-		if(teleport_animations_.empty()) {
-			teleport_animations_.push_back(unit_animation(absolute_image(),-20,20));
-			// always have a teleport animation
-		}
-		const config::child_list& extra_anims = cfg_.get_children("extra_anim");
-		{
-			for(config::child_list::const_iterator t = extra_anims.begin(); t != extra_anims.end(); ++t) {
+		if(ut) {
+			defensive_animations_ = ut->defensive_animations_;
+			teleport_animations_ = ut->teleport_animations_;
+			extra_animations_ = ut->extra_animations_;
+			death_animations_ = ut->death_animations_;
+			movement_animations_ = ut->movement_animations_;
+			standing_animations_ = ut->standing_animations_;
+			leading_animations_ = ut->leading_animations_;
+			healing_animations_ = ut->healing_animations_;
+			// remove animations from private cfg, since they're not needed there now
+			cfg_.clear_children("defend");
+			cfg_.clear_children("teleport_anim");
+			cfg_.clear_children("extra_anim");
+			cfg_.clear_children("death");
+			cfg_.clear_children("movement_anim");
+			cfg_.clear_children("healing_anim");
+			cfg_.clear_children("leading_anim");
+			cfg_.clear_children("standing_anim");
+		} else {
+			const config::child_list& defends = cfg_.get_children("defend");
+			const config::child_list& teleports = cfg_.get_children("teleport_anim");
+			const config::child_list& extra_anims = cfg_.get_children("extra_anim");
+			const config::child_list& deaths = cfg_.get_children("death");
+			const config::child_list& movement_anims = cfg_.get_children("movement_anim");
+			const config::child_list& standing_anims = cfg_.get_children("standing_anim");
+			const config::child_list& leading_anims = cfg_.get_children("leading_anim");
+			const config::child_list& healing_anims = cfg_.get_children("healing_anim");
+			for(config::child_list::const_iterator d = defends.begin(); d != defends.end(); ++d) {
+				defensive_animations_.push_back(defensive_animation(**d));
+			}
+			config::child_list::const_iterator t;
+			for(t = teleports.begin(); t != teleports.end(); ++t) {
+				teleport_animations_.push_back(unit_animation(**t));
+			}
+			for(t = extra_anims.begin(); t != extra_anims.end(); ++t) {
 				extra_animations_.insert(std::pair<std::string,unit_animation>((**t)["flag"],unit_animation(**t)));
 			}
+			for(config::child_list::const_iterator death = deaths.begin(); death != deaths.end(); ++death) {
+				death_animations_.push_back(death_animation(**death));
+			}
+			for(config::child_list::const_iterator movement_anim = movement_anims.begin(); movement_anim != movement_anims.end(); ++movement_anim) {
+				movement_animations_.push_back(movement_animation(**movement_anim));
+			}
+			for(config::child_list::const_iterator standing_anim = standing_anims.begin(); standing_anim != standing_anims.end(); ++standing_anim) {
+				standing_animations_.push_back(standing_animation(**standing_anim));
+			}
+			for(config::child_list::const_iterator leading_anim = leading_anims.begin(); leading_anim != leading_anims.end(); ++leading_anim) {
+				leading_animations_.push_back(leading_animation(**leading_anim));
+			}
+			for(config::child_list::const_iterator healing_anim = healing_anims.begin(); healing_anim != healing_anims.end(); ++healing_anim) {
+				healing_animations_.push_back(healing_animation(**healing_anim));
+			}
+			if(defensive_animations_.empty()) {
+				defensive_animations_.push_back(defensive_animation(absolute_image()));
+				// always have a defensive animation
+			}
+			if(teleport_animations_.empty()) {
+				teleport_animations_.push_back(unit_animation(absolute_image(),-20,20));
+				// always have a teleport animation
+			}
+			if(death_animations_.empty()) {
+				death_animations_.push_back(death_animation(absolute_image()));
+				// always have a death animation
+			}
+			if(movement_animations_.empty()) {
+				movement_animations_.push_back(movement_animation(absolute_image()));
+				// always have a movement animation
+			}
+			if(standing_animations_.empty()) {
+				standing_animations_.push_back(standing_animation(absolute_image()));
+				// always have a standing animation
+			}
+			if(leading_animations_.empty()) {
+				leading_animations_.push_back(leading_animation(absolute_image()));
+				// always have a leading animation
+			}
+			if(healing_animations_.empty()) {
+				healing_animations_.push_back(healing_animation(image_healing(),image_halo_healing()));
+				// always have a healing animation
+			}
 		}
-		const config::child_list& deaths = cfg_.get_children("death");
-		for(config::child_list::const_iterator death = deaths.begin(); death != deaths.end(); ++death) {
-			death_animations_.push_back(death_animation(**death));
-		}
-		if(death_animations_.empty()) {
-			death_animations_.push_back(death_animation(absolute_image()));
-			// always have a death animation
-		}
-		const config::child_list& movement_anims = cfg_.get_children("movement_anim");
-		for(config::child_list::const_iterator movement_anim = movement_anims.begin(); movement_anim != movement_anims.end(); ++movement_anim) {
-			movement_animations_.push_back(movement_animation(**movement_anim));
-		}
-		if(movement_animations_.empty()) {
-			movement_animations_.push_back(movement_animation(absolute_image()));
-			// always have a movement animation
-		}
-		const config::child_list& standing_anims = cfg_.get_children("standing_anim");
-		for(config::child_list::const_iterator standing_anim = standing_anims.begin(); standing_anim != standing_anims.end(); ++standing_anim) {
-			standing_animations_.push_back(standing_animation(**standing_anim));
-		}
-		if(standing_animations_.empty()) {
-			standing_animations_.push_back(standing_animation(absolute_image()));
-			// always have a standing animation
-		}
-		const config::child_list& leading_anims = cfg_.get_children("leading_anim");
-		for(config::child_list::const_iterator leading_anim = leading_anims.begin(); leading_anim != leading_anims.end(); ++leading_anim) {
-			leading_animations_.push_back(leading_animation(**leading_anim));
-		}
-		if(leading_animations_.empty()) {
-			leading_animations_.push_back(leading_animation(absolute_image()));
-			// always have a leading animation
-		}
-		const config::child_list& healing_anims = cfg_.get_children("healing_anim");
-		for(config::child_list::const_iterator healing_anim = healing_anims.begin(); healing_anim != healing_anims.end(); ++healing_anim) {
-			healing_animations_.push_back(healing_animation(**healing_anim));
-		}
-		if(healing_animations_.empty()) {
-			healing_animations_.push_back(healing_animation(image_healing(),image_halo_healing()));
-			// always have a leading animation
-		}
+	} else {
+		// remove animations from private cfg, since they're not needed there now
+		cfg_.clear_children("defend");
+		cfg_.clear_children("teleport_anim");
+		cfg_.clear_children("extra_anim");
+		cfg_.clear_children("death");
+		cfg_.clear_children("movement_anim");
+		cfg_.clear_children("healing_anim");
+		cfg_.clear_children("leading_anim");
+		cfg_.clear_children("standing_anim");
 	}
+	game_events::add_events(cfg_.get_children("event"),id_);
+	cfg_.clear_children("event");
 }
 void unit::write(config& cfg) const
 {
@@ -1291,6 +1381,14 @@ void unit::write(config& cfg) const
 	cfg["y"] = y;
 	cfg["id"] = id();
 	cfg["type"] = id();
+	std::map<std::string,unit_type>::const_iterator uti = gamedata_->unit_types.find(id());
+	const unit_type* ut = NULL;
+	if(uti != gamedata_->unit_types.end()) {
+		ut = &uti->second.get_gender_unit_type(gender_).get_variation(variation_);
+	}
+	if(ut && cfg["unit_description"] == ut->unit_description()) {
+		cfg["unit_description"] = "";
+	}
 
 	std::stringstream hp;
 	hp << hit_points_;
@@ -1383,7 +1481,7 @@ void unit::write(config& cfg) const
 	cfg["cost"] = lexical_cast_default<std::string>(unit_value_);
 	cfg.clear_children("modifications");
 	cfg.add_child("modifications",modifications_);
-
+	
 }
 
 void unit::assign_role(const std::string& role)
@@ -1705,7 +1803,10 @@ void unit::restart_animation(const display& disp,int start_time) {
 void unit::redraw_unit(display& disp,gamemap::location hex)
 {
 	const gamemap & map = disp.get_map();
-	if(hidden_) {
+	if(hidden_ || 
+	disp.fogged(hex.x,hex.y) ||
+	(invisible(hex,disp.get_units(),disp.get_teams()) &&
+			disp.get_teams()[disp.viewing_team()].is_enemy(side())) ){
 		if(unit_halo_) halo::remove(unit_halo_);
 		unit_halo_ = 0;
 		if(unit_anim_halo_) halo::remove(unit_anim_halo_);
@@ -1713,11 +1814,6 @@ void unit::redraw_unit(display& disp,gamemap::location hex)
 		return;
 	}
 	if(refreshing_) return;
-	if(disp.fogged(hex.x,hex.y)) { return;}
-	if(invisible(hex,disp.get_units(),disp.get_teams()) &&
-			disp.get_teams()[disp.viewing_team()].is_enemy(side())) {
-		return;
-	}
 	refreshing_ = true;
 	const gamemap::location dst= hex.get_direction(facing());
 	const double xsrc = disp.get_location_x(hex);
@@ -1782,7 +1878,7 @@ void unit::redraw_unit(display& disp,gamemap::location hex)
 	}else{
 		loc = image::locator(image_name);
 	}
-	surface image(image::get_image(loc,utils::string_bool(get_state("stoned"))?image::GREYED : image::SCALED));
+	surface image(image::get_image(loc,utils::string_bool(get_state("stoned"))?image::GREYED : image::UNSCALED));
 	if(image ==NULL) {
 		image = still_image();
 	}
@@ -1831,7 +1927,18 @@ void unit::redraw_unit(display& disp,gamemap::location hex)
 		ellipse_front.assign(image::get_image(image::locator(buf,team_rgb_range(),temp_rgb)));
 	}
 
-	disp.draw_unit(x, y -height_adjust, image, false, highlight_ratio,
+	
+	
+	int tmp_x = x;
+	int tmp_y = y;
+	
+	if (!image.null())
+	{
+		tmp_x += (disp.hex_size() - image.get()->w)/2;
+		tmp_y += (disp.hex_size() - image.get()->h)/2;
+	}
+
+	disp.draw_unit(tmp_x, tmp_y -height_adjust, image, false, highlight_ratio,
 			blend_with, blend_ratio, submerge,ellipse_back,ellipse_front);
 	if(!unit_halo_ && !image_halo().empty()) {
 		unit_halo_ = halo::add(0,0,image_halo());
@@ -1920,15 +2027,34 @@ std::set<gamemap::location> unit::overlaps(const gamemap::location &loc) const
 	}
 
 	switch (state()) {
-	case STATE_STANDING:
-		// Standing units don't overlap anything.
-		break;
-	default:
+		case STATE_STANDING:
+			// Standing units don't overlap anything.
+			break;
+		default:
+			gamemap::location arr[6];
+			get_adjacent_tiles(loc, arr);
+			for (unsigned int i = 0; i < 6; i++) {
+				over.insert(arr[i]);
+			}
+	}
+	if(offset_ > 0) {
+		gamemap::location adj_loc = loc.get_direction(facing_);
+		over.insert(adj_loc);
 		gamemap::location arr[6];
-		get_adjacent_tiles(loc, arr);
+		get_adjacent_tiles(adj_loc, arr);
 		for (unsigned int i = 0; i < 6; i++) {
 			over.insert(arr[i]);
 		}
+	} else if(offset_ < 0) {
+		gamemap::location opp_loc = loc.get_direction(loc.get_opposite_dir(facing_));
+		over.insert(opp_loc);
+		gamemap::location arr[6];
+		get_adjacent_tiles(opp_loc, arr);
+		for (unsigned int i = 0; i < 6; i++) {
+			over.insert(arr[i]);
+		}
+	} else {
+	// we stay in our hex, do nothing
 	}
 
 	return over;
@@ -2058,7 +2184,7 @@ int unit::defense_modifier(gamemap::TERRAIN terrain, int recurse_count) const
 
 	wassert(map_ != NULL);
 	//if this is an alias, then select the best of all underlying terrains
-	const std::string& underlying = map_->underlying_mvt_terrain(terrain);
+	const std::string& underlying = map_->underlying_def_terrain(terrain);
 	if(underlying.size() != 1 || underlying[0] != terrain) {
 		bool revert = (underlying[0] == '-'?true:false);
 		if(recurse_count >= 100) {
@@ -2080,9 +2206,6 @@ int unit::defense_modifier(gamemap::TERRAIN terrain, int recurse_count) const
 			} else if(value > ret_value && revert) {
 				ret_value = value;
 			}
-			if(value < ret_value) {
-				ret_value = value;
-			}
 		}
 
 //		defense_mods_.insert(std::pair<gamemap::TERRAIN,int>(terrain,ret_value));
@@ -2253,12 +2376,18 @@ void unit::backup_state()
 	attacks_b_ = attacks_;
 	if(cfg_.child("movement_costs")) {
 		movement_b_ = *cfg_.child("movement_costs");
+	} else {
+		movement_b_ = config();
 	}
 	if(cfg_.child("defense")) {
 		defense_b_ = *cfg_.child("defense");
+	} else {
+		defense_b_ = config();
 	}
 	if(cfg_.child("resistance")) {
 		resistance_b_ = *cfg_.child("resistance");
+	} else {
+		resistance_b_ = config();
 	}
 }
 
