@@ -1,4 +1,4 @@
-/* $Id: unit.hpp 12647 2006-07-19 18:54:27Z xan $ */
+/* $Id: unit.hpp 14337 2006-11-20 20:38:04Z boucman $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -82,8 +82,6 @@ class unit
 		//information about the unit -- a detailed description of it
 		const std::string& unit_description() const;
 
-		const config::child_list wml_events() const;
-
 		int hitpoints() const;
 		int max_hitpoints() const;
 		int experience() const;
@@ -189,6 +187,7 @@ class unit
 		int upkeep() const;
 
 		void set_hidden(bool state) {hidden_ = state;};
+		bool get_hidden() { return hidden_; };
 		bool is_flying() const;
 		int movement_cost(gamemap::TERRAIN terrain, int recurse_count=0) const;
 		int defense_modifier(gamemap::TERRAIN terrain, int recurse_count=0) const;
