@@ -1,4 +1,4 @@
-/* $Id: unit_display.cpp 12319 2006-06-28 21:19:02Z boucman $ */
+/* $Id: unit_display.cpp 14337 2006-11-20 20:38:04Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -47,7 +47,7 @@ void teleport_unit_between(display& disp, const gamemap::location& a, const game
 			disp.place_temporary_unit(temp_unit, a);
 			disp.draw();
 			events::pump();
-			disp.non_turbo_delay();
+			disp.delay(10);
 		}
 	}
 	if (!disp.fogged(b.x, b.y)) { // teleport
@@ -58,7 +58,7 @@ void teleport_unit_between(display& disp, const gamemap::location& a, const game
 			disp.place_temporary_unit(temp_unit, b);
 			disp.draw();
 			events::pump();
-			disp.non_turbo_delay();
+			disp.delay(10);
 		}
 	}
 	temp_unit.set_standing(disp,b);
@@ -84,22 +84,23 @@ void move_unit_between(display& disp, const gamemap& map, const gamemap::locatio
 
 	const int total_mvt_time = 150 * temp_unit.movement_cost(dst_terrain)/acceleration;
 	const unsigned int start_time = SDL_GetTicks();
-	int mvt_time = SDL_GetTicks() -start_time;
+	int mvt_time = 1;
 	disp.scroll_to_tiles(a.x,a.y,b.x,b.y,display::ONSCREEN);
 
-	while(mvt_time < total_mvt_time) {
+	while(mvt_time < total_mvt_time-1) {
+		disp.delay(10);
+		mvt_time = SDL_GetTicks() -start_time;
+		if(mvt_time >=total_mvt_time) mvt_time = total_mvt_time -1;
 		double pos =double(mvt_time)/total_mvt_time;
 		const gamemap::location& ref_loc =pos<0.5?a:b;
 		if(pos >= 0.5) pos = pos -1;
 		temp_unit.set_walking(disp,ref_loc);
-		disp.invalidate(ref_loc);
-		disp.place_temporary_unit(temp_unit,ref_loc);
 		temp_unit.set_offset(pos);
+		disp.place_temporary_unit(temp_unit,ref_loc);
+		disp.invalidate(ref_loc);
 		disp.draw();
 		events::pump();
-		disp.non_turbo_delay();
 
-		mvt_time = SDL_GetTicks() -start_time;
 	}
 }
 
@@ -127,8 +128,10 @@ void move_unit(display& disp, const gamemap& map, const std::vector<gamemap::loc
 	wassert(!path.empty());
 
 	bool previous_visible = false;
+	bool was_hidden = u.get_hidden();
 	// Original unit is usually hidden (but still on map, so count is correct)
 	unit temp_unit = u;
+	u.set_hidden(true);
 	temp_unit.set_hidden(false);
 
 	for(size_t i = 0; i+1 < path.size(); ++i) {
@@ -165,6 +168,7 @@ void move_unit(display& disp, const gamemap& map, const std::vector<gamemap::loc
 	for(std::vector<gamemap::location>::const_iterator it = path.begin(); it != path.end(); ++it) {
 		disp.invalidate(*it);
 	}
+	u.set_hidden(was_hidden);
 }
 
 void unit_die(display& disp,const gamemap::location& loc, unit& u, const attack_type* attack)
@@ -185,7 +189,7 @@ void unit_die(display& disp,const gamemap::location& loc, unit& u, const attack_
 		disp.invalidate(loc);
 		disp.draw();
 		events::pump();
-		disp.non_turbo_delay();
+		disp.delay(10);
 	}
 	u.set_standing(disp,loc);
 	disp.update_display();
@@ -344,6 +348,7 @@ bool unit_attack_ranged(display& disp, unit_map& units,
 			sound_played = true;
 			sound::play_sound(def->second.get_hit_sound());
 			disp.float_label(b,lexical_cast<std::string>(damage),255,0,0);
+			disp.invalidate_unit();
 		}
 		disp.draw();
 		events::pump();
@@ -356,7 +361,7 @@ bool unit_attack_ranged(display& disp, unit_map& units,
 		if(leader_loc.valid() && leader->second.get_animation()->animation_finished() ) {
 		   leader->second.set_standing(disp,leader_loc,true);
 		}
-		disp.non_turbo_delay();
+		disp.delay(10);
 		// we use missile animation because it's the only one not reseted in the middle to go to standing
 		animation_time = missile_animation.get_animation_time();
 		missile_animation.update_current_frame();
@@ -366,6 +371,7 @@ bool unit_attack_ranged(display& disp, unit_map& units,
 		sound_played = true;
 		sound::play_sound(def->second.get_hit_sound());
 		disp.float_label(b,lexical_cast<std::string>(damage),255,0,0);
+		disp.invalidate_unit();
 	}
 	halo::remove(missile_halo);
 	missile_halo = 0;
@@ -474,13 +480,14 @@ bool unit_attack(display& disp, unit_map& units,
 		if(leader_loc.valid()) disp.invalidate(leader_loc);
 		disp.draw();
 		events::pump();
-		disp.non_turbo_delay();
+		disp.delay(10);
 
 		animation_time = attacker.get_animation()->get_animation_time();
 	}
 	if(damage > 0 && !hide) {
 		sound::play_sound(def->second.get_hit_sound());
 		disp.float_label(b,lexical_cast<std::string>(damage),255,0,0);
+		disp.invalidate_unit();
 	}
 	if(def->second.take_hit(damage)) {
 		dead = true;
@@ -510,7 +517,7 @@ bool unit_attack(display& disp, unit_map& units,
 		if(leader_loc.valid() && leader->second.get_animation()->animation_finished() ) {
 		   leader->second.set_standing(disp,leader_loc,true);
 		}
-		disp.non_turbo_delay();
+		disp.delay(10);
 		animation_time = attacker.get_animation()->get_animation_time();
 	}
 
