@@ -1,4 +1,4 @@
-/* $Id: unit_types.cpp 12647 2006-07-19 18:54:27Z xan $ */
+/* $Id: unit_types.cpp 14022 2006-10-19 19:23:21Z mordante $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -63,7 +63,7 @@ attack_type::attack_type(const config& cfg,const std::string& id, const std::str
 	if(icon_.empty())
 		icon_ = "attacks/" + id_ + ".png";
 
-	range_ = cfg["range"];
+	range_ = cfg["range"].value();
 	damage_ = atol(cfg["damage"].c_str());
 	num_attacks_ = atol(cfg["number"].c_str());
 
@@ -88,6 +88,10 @@ attack_type::attack_type(const config& cfg,const std::string& id, const std::str
 	}
 }
 
+config& attack_type::get_cfg()
+{
+	return cfg_;
+}
 const config& attack_type::get_cfg() const
 {
 	return cfg_;
@@ -277,9 +281,9 @@ bool attack_type::apply_modification(const config& cfg,std::string* description)
 			sp["description"] = t_string("Plague:\nWhen a unit is killed by a Plague attack, that unit is replaced with a unit identical to and on the same side as the unit with the Plague attack. (This doesn't work on Undead or units in villages.)","wesnoth");
 		} else if(set_special.substr(0,7) == "plague(") {
 			config& sp = new_specials.add_child("plague");
-			sp["name"] = t_string("plague","wesnoth") + set_special.substr(6,set_special.size());
+			sp["name"] = t_string("plague","wesnoth") + set_special.substr(6);
 			sp["description"] = t_string("Plague:\nWhen a unit is killed by a Plague attack, that unit is replaced with a unit of the specified type on the same side as the unit with the Plague attack. (This doesn't work on Undead or units in villages.)","wesnoth");
-			sp["type"] = set_special.substr(7,set_special.size()-1);
+			sp["type"] = set_special.substr(7,set_special.size()-8);
 		} else if(set_special == "swarm") {
 			config& sp = new_specials.add_child("attacks");
 			sp["name"] = t_string("swarm","wesnoth");
@@ -470,7 +474,7 @@ int unit_movement_type::defense_modifier(const gamemap& map,gamemap::TERRAIN ter
 	}
 
 	//if this is an alias, then select the best of all underlying terrains
-	const std::string& underlying = map.underlying_mvt_terrain(terrain);
+	const std::string& underlying = map.underlying_def_terrain(terrain);
 	if(underlying.size() != 1 || underlying[0] != terrain) {
 		bool revert = (underlying[0] == '-'?true:false);
 		if(recurse_count >= 100) {
@@ -492,9 +496,6 @@ int unit_movement_type::defense_modifier(const gamemap& map,gamemap::TERRAIN ter
 			} else if(value > ret_value && revert) {
 				ret_value = value;
 			}
-			if(value < ret_value) {
-				ret_value = value;
-			}
 		}
 
 		defenseMods_.insert(std::pair<gamemap::TERRAIN,int>(terrain,ret_value));
