@@ -1,4 +1,4 @@
-/* $Id: button.cpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: button.cpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -11,15 +11,16 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "button.hpp"
-#include "../font.hpp"
-#include "../image.hpp"
-#include "../log.hpp"
-#include "../util.hpp"
-#include "../video.hpp"
-#include "../wml_separators.hpp"
+#include "widgets/button.hpp"
+#include "font.hpp"
+#include "marked-up_text.hpp"
+#include "image.hpp"
+#include "log.hpp"
+#include "util.hpp"
+#include "video.hpp"
+#include "wml_separators.hpp"
 #include "serialization/string_utils.hpp"
 
 namespace gui {
@@ -33,7 +34,7 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
                std::string button_image_name, SPACE_CONSUMPTION spacing)
 	: widget(video), label_(label),
 	  image_(NULL), pressedImage_(NULL), activeImage_(NULL), pressedActiveImage_(NULL),
-	  button_(true), state_(NORMAL), type_(type), enabled_(true), pressed_(false),
+	  button_(true), state_(NORMAL), type_(type), pressed_(false),
 	  spacing_(spacing), base_height_(0), base_width_(0)
 {
 	if(button_image_name.empty() && type == TYPE_PRESS) {
@@ -67,7 +68,9 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
 	base_height_ = button_image->h;
 	base_width_ = button_image->w;
 
-	set_label(label);
+	if (type_ != TYPE_IMAGE){
+		set_label(label);
+	}
 
 	if(type == TYPE_PRESS) {
 		image_.assign(scale_surface(button_image,location().w,location().h));
@@ -80,21 +83,35 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
 		if (type == TYPE_CHECK)
 			pressedActiveImage_.assign(scale_surface(pressed_active_image, button_image->w, button_image->h));
 	}
+
+	if (type_ == TYPE_IMAGE){
+		calculate_size();
+	}
 }
 
 void button::calculate_size()
 {
+	if (type_ == TYPE_IMAGE){
+		SDL_Rect loc_image = location();
+		loc_image.h = image_->h;
+		loc_image.w = image_->w;
+		set_location(loc_image);
+		return;
+	}
 	SDL_Rect const &loc = location();
 	bool change_size = loc.h == 0 || loc.w == 0;
 
 	if (!change_size) {
-		unsigned w = loc.w - (type_ == TYPE_PRESS ? horizontal_padding :
-		                                            checkbox_horizontal_padding + base_width_);
-		label_ = font::make_text_ellipsis(label_, font_size, w);
+		unsigned w = loc.w - (type_ == TYPE_PRESS ? horizontal_padding : checkbox_horizontal_padding + base_width_);
+		if (type_ != TYPE_IMAGE){
+			label_ = font::make_text_ellipsis(label_, font_size, w);
+		}
 	}
 
-	textRect_ = font::draw_text(NULL, screen_area(), font_size,
-	                            font::BUTTON_COLOUR, label_, 0, 0);
+	if (type_ != TYPE_IMAGE){
+		textRect_ = font::draw_text(NULL, screen_area(), font_size,
+		                            font::BUTTON_COLOUR, label_, 0, 0);
+	}
 
 	if (!change_size)
 		return;
@@ -137,19 +154,14 @@ bool button::checked() const
 
 void button::enable(bool new_val)
 {
-	if (enabled_ != new_val) {
-		enabled_ = new_val;
-		state_ = NORMAL;
+	if(new_val != enabled())
+	{
 		pressed_ = false;
-		set_dirty();
+		state_ = NORMAL;
+		widget::enable(new_val);
 	}
 }
 
-bool button::enabled() const
-{
-	return enabled_;
-}
-
 void button::draw_contents()
 {
 	surface image = image_;
@@ -172,25 +184,33 @@ void button::draw_contents()
 		break;
 	}
 
-	SDL_Rect const &clipArea = screen_area();
 	SDL_Rect const &loc = location();
+	SDL_Rect clipArea = loc;
 	const int texty = loc.y + loc.h / 2 - textRect_.h / 2 + offset;
 	int textx;
 
 	if (type_ != TYPE_CHECK)
 		textx = loc.x + image->w / 2 - textRect_.w / 2 + offset;
-	else
+	else {
+		clipArea.w += image_w + checkbox_horizontal_padding;
 		textx = loc.x + image_w + checkbox_horizontal_padding / 2;
+	}
 
 	SDL_Color button_colour = font::BUTTON_COLOUR;
 
-	if (!enabled_) {
+	if (!enabled()) {
 		image = greyscale_image(image);
 		button_colour = font::STONED_COLOUR;
 	}
 
 	video().blit_surface(loc.x, loc.y, image);
-	font::draw_text(&video(), clipArea, font_size, button_colour, label_, textx, texty);
+	if (type_ != TYPE_IMAGE){
+		clipArea.x += offset;
+		clipArea.y += offset;
+		clipArea.w -= 2*offset;
+		clipArea.h -= 2*offset;
+		font::draw_text(&video(), clipArea, font_size, button_colour, label_, textx, texty);
+	}
 
 	update_rect(loc);
 }
@@ -234,15 +254,22 @@ void button::mouse_motion(SDL_MouseMotionEvent const &event)
 		// the cursor is not over the widget
 		if (state_ == PRESSED_ACTIVE)
 			state_ = PRESSED;
-		else if (type_ != TYPE_CHECK || state_ != PRESSED)
+		else if (type_ != TYPE_CHECK && type_ != TYPE_IMAGE || state_ != PRESSED)
 			state_ = NORMAL;
 	}
 }
 
 void button::mouse_down(SDL_MouseButtonEvent const &event)
 {
-	if (hit(event.x, event.y) && event.button == SDL_BUTTON_LEFT && type_ != TYPE_CHECK)
-		state_ = PRESSED;
+	if (hit(event.x, event.y) && event.button == SDL_BUTTON_LEFT && type_ != TYPE_CHECK){
+		if (type_ != TYPE_IMAGE){
+			state_ = PRESSED;
+		}
+		else{
+			if (state_ == PRESSED) { state_ = ACTIVE; }
+			else { state_ = PRESSED; }
+		}
+	}
 }
 
 void button::mouse_up(SDL_MouseButtonEvent const &event)
@@ -264,12 +291,15 @@ void button::mouse_up(SDL_MouseButtonEvent const &event)
 	case TYPE_TURBO:
 		state_ = ACTIVE;
 		break;
+	case TYPE_IMAGE:
+		pressed_ = true;
+		break;
 	}
 }
 
 void button::handle_event(const SDL_Event& event)
 {
-	if (hidden() || !enabled_)
+	if (hidden() || !enabled())
 		return;
 
 	STATE start_state = state_;
