@@ -1,4 +1,4 @@
-/* $Id: button.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: button.hpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -30,7 +30,7 @@ class button : public widget
 public:
 	struct error {};
 
-	enum TYPE { TYPE_PRESS, TYPE_CHECK, TYPE_TURBO };
+	enum TYPE { TYPE_PRESS, TYPE_CHECK, TYPE_TURBO, TYPE_IMAGE };
 
 	enum SPACE_CONSUMPTION { DEFAULT_SPACE, MINIMUM_SPACE };
 
@@ -44,9 +44,8 @@ public:
 	void set_label(const std::string& val);
 
 	bool pressed();
-
-	void enable(bool new_val);
-	bool enabled() const;
+	bool hit(int x, int y) const;
+	virtual void enable(bool new_val=true);
 
 protected:
 	virtual void handle_event(const SDL_Event& event);
@@ -70,15 +69,12 @@ private:
 
 	TYPE type_;
 
-	bool enabled_;
-
 	bool pressed_;
 
 	SPACE_CONSUMPTION spacing_;
 
 	int base_height_, base_width_;
 
-	bool hit(int x, int y) const;
 }; //end class button
 
 }
