@@ -1,4 +1,4 @@
-/* $Id: combo.cpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: combo.cpp 11723 2006-05-30 03:00:33Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -11,13 +11,12 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "combo.hpp"
-#include "button.hpp"
-#include "../display.hpp"
-#include "../show_dialog.hpp"
-#include "../video.hpp"
+#include "widgets/combo.hpp"
+#include "display.hpp"
+#include "show_dialog.hpp"
+#include "video.hpp"
 
 namespace gui {
 
@@ -77,7 +76,7 @@ void combo::process_event()
 	if (!pressed())
 		return;
 	SDL_Rect const &loc = location();
-	set_selected_internal(gui::show_dialog(*disp_, NULL, "", "", gui::MESSAGE, &items_,
+	set_selected_internal(gui::show_dialog2(*disp_, NULL, "", "", gui::MESSAGE, &items_,
 	                                       NULL, "", NULL, -1, NULL, NULL, loc.x, loc.y + loc.h));
 }
 
