@@ -1,4 +1,4 @@
-/* $Id: menu.hpp 7948 2005-08-21 15:24:37Z gruikya $ */
+/* $Id: menu.hpp 11788 2006-06-01 20:20:30Z sapient $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -33,6 +33,67 @@ class menu : public scrollarea
 {
 public:
 
+	enum ROW_TYPE { NORMAL_ROW, SELECTED_ROW, HEADING_ROW };
+	//basic menu style
+	class style
+	{
+	public:
+		style();
+		virtual ~style();
+		virtual void init() {}
+
+		virtual SDL_Rect item_size(const std::string& item) const;
+		virtual void draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+
+		size_t get_font_size() const;
+		size_t get_cell_padding() const;
+		size_t get_thickness() const;
+
+	protected:
+		size_t font_size_;
+		size_t cell_padding_;
+		size_t thickness_;  //additional cell padding for style use only
+
+		int normal_rgb_, selected_rgb_, heading_rgb_;
+		double normal_alpha_, selected_alpha_, heading_alpha_;
+	};
+
+	//image-border selction style
+	class imgsel_style : public style
+	{
+	public:
+		imgsel_style(const std::string &img_base, bool has_bg,
+								 int normal_rgb, int selected_rgb, int heading_rgb,
+								 double normal_alpha, double selected_alpha, double heading_alpha);
+		virtual ~imgsel_style();
+
+		virtual SDL_Rect item_size(const std::string& item) const;
+		virtual void draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+
+		virtual void init() { load_images(); }
+		bool load_images();
+
+	protected:
+		const std::string img_base_;
+		std::map<std::string,surface> img_map_;
+
+	private:
+		bool load_image(const std::string &img_sub);
+		bool has_background_;
+		bool initialized_;
+		bool load_failed_;
+		int normal_rgb2_, selected_rgb2_, heading_rgb2_;
+		double normal_alpha2_, selected_alpha2_, heading_alpha2_;
+	};
+	friend class style;
+	friend class imgsel_style;
+	static style &default_style;
+	//static style bluebox_style;
+	static imgsel_style slateborder_style;
+	static imgsel_style bluebg_style;
+
 	struct item
 	{
 		item() : id(0)
@@ -78,7 +139,7 @@ public:
 
 	menu(CVideo& video, const std::vector<std::string>& items,
 	     bool click_selects=false, int max_height=-1, int max_width=-1,
-		 const sorter* sorter_obj=NULL);
+		 const sorter* sorter_obj=NULL, style *menu_style=NULL);
 
 	int selection() const;
 
@@ -115,16 +176,20 @@ public:
 	void set_click_selects(bool value);
 	void set_numeric_keypress_selection(bool value);
 
-	void scroll(int pos);
+	void scroll(unsigned int pos);
 
 	void sort_by(int column);
 
 protected:
+	bool item_ends_with_image(const std::string& item) const;
 	void handle_event(const SDL_Event& event);
 	void set_inner_location(const SDL_Rect& rect);
 
 	bool requires_event_focus() const { return true; };
+	const std::vector<int>& column_widths() const;
+	virtual void draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 
+	style *style_;
 private:
 	size_t max_items_onscreen() const;
 
@@ -159,18 +224,15 @@ private:
 
 	bool double_clicked_;
 
-	const std::vector<int>& column_widths() const;
 	void column_widths_item(const std::vector<std::string>& row, std::vector<int>& widths) const;
 
-	enum ROW_TYPE { NORMAL_ROW, SELECTED_ROW, HEADING_ROW };
-	void draw_row(const std::vector<std::string>& row, const SDL_Rect& rect, ROW_TYPE type);
 	void clear_item(int item);
 	void draw_contents();
 	void draw();
 	int hit(int x, int y) const;
 
 	std::pair<int,int> hit_cell(int x, int y) const;
-	int hit_column(int x, int y) const;
+	int hit_column(int x) const;
 
 	int hit_heading(int x, int y) const;
 
@@ -221,6 +283,8 @@ private:
 	std::set<int> invalid_;
 };
 
+
+
 }
 
 #endif
