@@ -1,4 +1,4 @@
-/* $Id: scrollarea.hpp 7943 2005-08-21 12:58:46Z gruikya $ */
+/* $Id: scrollarea.hpp 9798 2006-01-24 01:17:58Z rusty $ */
 /*
    Copyright (C) 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -35,7 +35,7 @@ protected:
 	virtual void update_location(SDL_Rect const &rect);
 	virtual void handle_event(const SDL_Event& event);
 	virtual void process_event();
-	virtual void scroll(int pos) = 0;
+	virtual void scroll(unsigned int pos) = 0;
 	virtual void set_inner_location(SDL_Rect const &rect) = 0;
 
 	SDL_Rect inner_location() const;
