@@ -1,4 +1,4 @@
-/* $Id: textbox.cpp 8239 2005-09-14 20:57:32Z ott $ */
+/* $Id: textbox.cpp 11788 2006-06-01 20:20:30Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -11,17 +11,17 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "textbox.hpp"
-#include "../clipboard.hpp"
-#include "../font.hpp"
-#include "../language.hpp"
-#include "../log.hpp"
-#include "../sdl_utils.hpp"
-#include "../serialization/string_utils.hpp"
-#include "../util.hpp"
-#include "../video.hpp"
+#include "widgets/textbox.hpp"
+#include "clipboard.hpp"
+#include "font.hpp"
+#include "language.hpp"
+#include "log.hpp"
+#include "sdl_utils.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
+#include "video.hpp"
 
 #include "SDL.h"
 
@@ -71,7 +71,7 @@ void textbox::set_text(const std::string& text)
 	update_text_cache(true);
 }
 
-void textbox::append_text(const std::string& text)
+void textbox::append_text(const std::string& text, bool auto_scroll)
 {
 	if(text_image_.get() == NULL) {
 		set_text(text);
@@ -82,7 +82,7 @@ void textbox::append_text(const std::string& text)
 	if(wrap_ == false && std::find_if(text.begin(),text.end(),utils::isnewline) != text.end()) {
 		return;
 	}
-
+	const bool is_at_bottom = get_position() == get_max_position();
 	const wide_string& wtext = utils::string_to_wstring(text);
 
 	const surface new_text = add_text_line(wtext);
@@ -102,6 +102,7 @@ void textbox::append_text(const std::string& text)
 
 	set_dirty(true);
 	update_text_cache(false);
+	if(auto_scroll && is_at_bottom) scroll_to_bottom();
 }
 
 void textbox::clear()
@@ -167,7 +168,7 @@ void textbox::draw_contents()
 
 				const clip_rect_setter clipper(surf, loc);
 
-				Uint32 colour = SDL_MapRGB(surf->format, 160, 0, 0);
+				Uint32 colour = SDL_MapRGB(surf->format, 0, 0, 160);
 				fill_rect_alpha(rect, colour, 140, surf);
 
 				starty += int(line_height_);
@@ -234,7 +235,7 @@ void textbox::set_location(const SDL_Rect& rect)
 	set_shown_size(location().h);
 }
 
-void textbox::scroll(int pos)
+void textbox::scroll(unsigned int pos)
 {
 	yscroll_ = pos;
 	set_dirty(true);
@@ -347,7 +348,7 @@ void textbox::erase_selection()
 }
 
 namespace {
-	const unsigned int copypaste_modifier = 
+	const unsigned int copypaste_modifier =
 #ifdef __APPLE__
 		KMOD_LMETA | KMOD_RMETA
 #else
@@ -388,8 +389,8 @@ void textbox::handle_event(const SDL_Event& event)
 		int pos = 0;
 		int distance = x;
 
-		for(int i = 1; i < int(char_x_.size()); ++i) {
-			if(yscroll_ + y < char_y_[i]) {
+		for(unsigned int i = 1; i < char_x_.size(); ++i) {
+			if((int)yscroll_ + y < char_y_[i]) {
 				break;
 			}
 
@@ -445,7 +446,7 @@ void textbox::handle_event(const SDL_Event& event)
 	if(c == SDLK_LEFT && cursor_ > 0)
 		--cursor_;
 
-	if(c == SDLK_RIGHT && cursor_ < text_.size())
+	if(c == SDLK_RIGHT && cursor_ < (int)text_.size())
 		++cursor_;
 
 	// ctrl-a, ctrl-e and ctrl-u are readline style shortcuts, even on Macs
@@ -482,7 +483,7 @@ void textbox::handle_event(const SDL_Event& event)
 		if(is_selection()) {
 			erase_selection();
 		} else {
-			if(cursor_ == text_.size()) {
+			if(cursor_ == (int)text_.size()) {
 				text_.resize(text_.size()-1);
 				--cursor_;
 			} else {
