@@ -1,6 +1,6 @@
-/* $Id: textbox.hpp,v 1.45 2005/06/04 19:16:06 ott Exp $ */
+/* $Id: textbox.hpp 9852 2006-01-27 22:38:26Z boucman $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@ public:
 
 	const std::string text() const;
 	void set_text(const std::string& text);
-	void append_text(const std::string& text);
+	void append_text(const std::string& text,bool auto_scroll = false);
 	void clear();
 	void process();
 
@@ -50,7 +50,7 @@ public:
 protected:
 	virtual void draw_contents();
 	virtual void set_inner_location(SDL_Rect const &);
-	virtual void scroll(int pos);
+	virtual void scroll(unsigned int pos);
 
 private:
 	size_t max_size_;
