@@ -1,4 +1,4 @@
-/* $Id: widget.cpp 12603 2006-07-17 09:53:55Z isaac $ */
+/* $Id: widget.cpp 13460 2006-09-08 08:49:00Z dirus $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -23,16 +23,19 @@ namespace {
 namespace gui {
 
 widget::widget(const widget &o)
-	: events::handler(), video_(o.video_), restorer_(o.restorer_), rect_(o.rect_),
-	  focus_(o.focus_), needs_restore_(o.needs_restore_), state_(o.state_),
-	  enabled_(o.enabled_), clip_(o.clip_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
-	  help_text_(o.help_text_), help_string_(o.help_string_), align_(o.align_)
+	: events::handler(), video_(o.video_), restorer_(o.restorer_),
+	  rect_(o.rect_), focus_(o.focus_), needs_restore_(o.needs_restore_),
+	  state_(o.state_), hidden_override_(o.hidden_override_),
+	  enabled_(o.enabled_), clip_(o.clip_), clip_rect_(o.clip_rect_),
+	  volatile_(o.volatile_), help_text_(o.help_text_),
+	  help_string_(o.help_string_), align_(o.align_)
 {
 }
 
 widget::widget(CVideo& video, bool auto_join)
-	: handler(auto_join), video_(&video), rect_(EmptyRect), focus_(true), needs_restore_(false),
-	  state_(UNINIT), enabled_(true), clip_(false), volatile_(false), help_string_(0),
+	: handler(auto_join), video_(&video), rect_(EmptyRect), focus_(true),
+	  needs_restore_(false), state_(UNINIT), hidden_override_(false),
+	  enabled_(true), clip_(false), volatile_(false), help_string_(0),
 	  align_(RIGHT_ALIGN)
 {
 }
@@ -144,13 +147,29 @@ bool widget::focus() const
 void widget::hide(bool value)
 {
 	if (value) {
-		if (state_ == DIRTY || state_ == DRAWN)
+		if ((state_ == DIRTY || state_ == DRAWN) && !hidden_override_)
 			bg_restore();
 		state_ = HIDDEN;
 	} else if (state_ == HIDDEN) {
 		state_ = DRAWN;
-		bg_update();
-		set_dirty(true);
+		if (!hidden_override_) {
+			bg_update();
+			set_dirty(true);
+		}
+	}
+}
+
+void widget::hide_override(bool value) {
+	if (hidden_override_ != value) {
+		hidden_override_ = value;
+		if (state_ == DIRTY || state_ == DRAWN) {
+			if (value) {
+				bg_restore();
+			} else {
+				bg_update();
+				set_dirty(true);
+			}
+		}
 	}
 }
 
@@ -163,7 +182,7 @@ void widget::set_clip_rect(const SDL_Rect& rect)
 
 bool widget::hidden() const
 {
-	return (state_ == HIDDEN || state_ == UNINIT
+	return (state_ == HIDDEN || hidden_override_ || state_ == UNINIT
 		|| (clip_ && !rects_overlap(clip_rect_, rect_)));
 }
 
@@ -182,7 +201,7 @@ bool widget::enabled() const
 
 void widget::set_dirty(bool dirty)
 {
-	if (dirty && (volatile_ || state_ != DRAWN) || !dirty && state_ != DIRTY)
+	if (dirty && (volatile_ || hidden_override_ || state_ != DRAWN) || !dirty && state_ != DIRTY)
 		return;
 
 	state_ = dirty ? DIRTY : DRAWN;
@@ -257,7 +276,7 @@ void widget::draw()
 
 void widget::volatile_draw()
 {
-	if (!volatile_ || state_ != DRAWN)
+	if (!volatile_ || state_ != DRAWN || hidden_override_)
 		return;
 	state_ = DIRTY;
 	bg_update();
